!{\src2tex{textfont=tt}}
!!****p* main/calccomm
!! NAME
!! Calculate commutator   
!!
!! FUNCTION
!! Set up rho-twiddle(G=0) using small vector q instead of zero
!! and k.p perturbation theory (for proper limit q->0) 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! cfnl, comm_lda, rginv, dosym, pdtqrc
!!
!! SOURCE

 subroutine calccomm(iparam,tr,trspace,bzstruct,elecstr,latt,symstr,&
                       kb,chemstr,trans,transtab)

  use ggwdef
  use dpdef
  use verbose
  use interfaces
  implicit none
  type(dpparam) iparam
  type(kb_potential) kb
  type(transition) trans(iparam%nt)
  type(bzstructure) bzstruct
  type(transposed_space) trspace
  type(electronic_structure) elecstr
  type(lattstruct) latt
  type(symmetry) symstr
  type(transgen) tr
  type(chemistry) chemstr

  integer transtab(iparam%nkbz,iparam%lomo:iparam%nbnds,iparam%lomo:iparam%nbnds,iparam%nsppol)
  complex :: wfg1(iparam%npwvec),wfg2(iparam%npwvec)
  real :: opinv(3,3)
  integer :: ikbz, ikibz, iop, iinv,  ibv, ibc 
  integer :: it,iq,itshf,itres,ir

  complex, external :: pdtqrc

  real :: qrot(3)
  complex :: rhotwx(3)
  real (kind(0.0d0)) :: ediff
  integer istat
  complex :: rhotwg0
  integer :: startcycle,stopcycle

  integer :: j,l,isppol
  complex :: rhotwxlda(3) 
  complex :: cfc

#ifdef MPI
  include 'mpif.h'
  integer ierr,rang,nprocs
  complex, allocatable :: aux(:,:)
  real starttime, endtime
  call mpi_comm_size(mpi_comm_world,nprocs,ierr)
  call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif

  if(iverbose > 2) call uverbose('b','calc_comm')
  if(iverbose > 0) then
     headline="Calculating commutator"
     call print_headline()
  endif
  
  if(kb%inclvkb) then
    kb%ng = iparam%npwwfn
    kb%nk = iparam%nkibz
    call cfnl(iparam,latt,trspace,bzstruct,kb,chemstr)
  endif
  if (.not. iparam%oldqplda) deallocate(chemstr%typeat) 



!   set up rho-twiddle(G=0) using small vector q instead of zero
!   and k.p perturbation theory

!for each spin channel, only resonant transitions
#ifdef MPI
   call split_trans(1,iparam%nt,startcycle,stopcycle)
   if(iparam%exciton) then
    allocate(tr%rhotwxqpol(iparam%nt,iparam%nqpol),stat=istat)
   else
    allocate(tr%rhotwxqpol(startcycle:stopcycle,iparam%nqpol),stat=istat)
   endif
   if (istat/=0) then 
    write(uout,*) 'rhotwxqpol not allocated' 
    stop
   endif
#else
   allocate(tr%rhotwxqpol(iparam%nt,iparam%nqpol),stat=istat)
   if (istat/=0) then 
    write(uout,*) 'rhotwxqpol not allocated' 
    stop
   endif
   startcycle = 1; stopcycle = iparam%nt
#endif

   tr%rhotwxqpol = 0
   if(iverbose > 2) then
    call memstat('(calcola_comm) tr%rhotwxqpol allocated =','c',iparam%nt,iparam%nqpol)
   endif
   do it=startcycle,stopcycle
   ikbz=trans(it)%k
   ikibz = bzstruct%ktab(ikbz)
   ibv = trans(it)%v
   ibc = trans(it)%c
   isppol = trans(it)%s

   if(.not. iparam%qps) then
    ediff =  trans(it)%en   
    if (.not.iparam%wf_distrib) then
     wfg1(:)=elecstr%wfg(:,ibc,ikibz,isppol)
     wfg2(:)=elecstr%wfg(:,ibv,ikibz,isppol)
    else
     ir=elecstr%wf_count(ikibz,ibc,isppol)
     wfg1(:)=elecstr%wfg2(:,ir)
     ir=elecstr%wf_count(ikibz,ibv,isppol)
     wfg2(:)=elecstr%wfg2(:,ir)
    endif
    call comm_lda(kb,wfg1,wfg2,ikibz,iparam%npwwfn,iparam%npwvec, & 
&     ediff,trspace%gvec, rhotwx)   
   endif

  
   if(iparam%qps) then
! for each couple  n= trans(it)%v, n' = trans(it)%c
! comm(n,n')=  \Sum_(jl) C_jn^* C_ln' comm_lda(j,l)
    rhotwx(:) = 0
    do j=1,iparam%nbndmax
      do l=1,iparam%nbndmax 
        cfc = conjg(elecstr%m_lda_to_qp(j, ibv,ikibz,isppol))* & 
&               elecstr%m_lda_to_qp(l, ibc,ikibz,isppol)
        if (abs(cfc) < 1.0e-6) cycle
         ediff=elecstr%energy(ikibz,l,isppol) - &
               elecstr%energy(ikibz,j,isppol)
         if (abs(ediff) < 1.0e-4) cycle !degenerate states excluded from sum

        wfg1(:)=elecstr%wfg(:,l,ikibz,isppol)
        wfg2(:)=elecstr%wfg(:,j,ikibz,isppol)

        call comm_lda(kb,wfg1,wfg2,ikibz,iparam%npwwfn,iparam%npwvec, & 
&             ediff,trspace%gvec, rhotwxlda) 

        rhotwx(:) = rhotwx(:) + cfc * rhotwxlda(:)  

      enddo  !l   
    enddo !j
   endif  


!   rhotwg0 = R^-1q * rhotwx_ibz
!   rhotwg0 = -R^-1q * conjg(rhotwx_ibz) for inversion
   iop = bzstruct%ktabo(ikbz)
   iinv = (3-bzstruct%ktabi(ikbz))/2
   opinv(:,:) = symstr%op(:,:,iop)
   call rginv(3,opinv,3)

   do iq = 1, iparam%nqpol
     call dosym(opinv,iinv,iparam%qpol(:,iq),qrot)
     rhotwg0 = pdtqrc(qrot,rhotwx,latt%b1,latt%b2,latt%b3)
     if(iinv == 2) rhotwg0 = conjg(rhotwg0)
     tr%rhotwxqpol(it,iq)=rhotwg0
   enddo

  enddo !it
#ifdef MPI
  if(iparam%exciton) then
   call mpi_barrier(mpi_comm_world,ierr)
   allocate(aux(iparam%nt,iparam%nqpol))
   aux=0
   call mpi_reduce(tr%rhotwxqpol,aux,iparam%nt*iparam%nqpol,mpi_complex,mpi_sum,0,mpi_comm_world,ierr)
   tr%rhotwxqpol=aux
   call mpi_bcast(tr%rhotwxqpol,iparam%nt*iparam%nqpol,mpi_complex,0,mpi_comm_world,ierr)
   deallocate(aux)
  endif
#endif
 

  if(kb%inclvkb) then
    deallocate(kb%vkbsign,kb%vkb,kb%vkbd)
    deallocate(kb%fnl,kb%fnld)
  endif
   
  if(.not.iparam%FXCMT) deallocate(bzstruct%ktabo,bzstruct%ktabi,bzstruct%ktab)
  if(.not.iparam%FXCMT) deallocate(symstr%op)
  deallocate(bzstruct%kibz)
  

   if(iverbose > 0) then
     call print_footline()
   endif
   if(iverbose > 2) call uverbose('e','calc_comm')
  end 
!!***


!{\src2tex{textfont=tt}}
!!****p* calccomm/comm_lda
!! NAME
!! Calculate commutator on the Kohn-Sham basis.   
!!
!! FUNCTION
!! Calculate the expectation values:
!! <n | \grad + [Vnl,r] | n'> / [en(n')-en(n)]
!! between a couple of Kohn-Sham states |n>,|n'>.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!
!! PARENTS
!! calccomm
!!
!! CHILDREN
!!  
!!
!! SOURCE

  subroutine comm_lda(kb,wfg1,wfg2,ikibz,npwwfn,npwvec,ediff,gvec,rhotwx)

  use ggwdef
  use verbose
  use interfaces
  implicit none

  type(kb_potential) kb

  integer :: npwwfn,ikibz,npwvec 
  complex :: wfg1(npwvec),wfg2(npwvec) 
  real (kind(0.0d0)) :: ediff 
  integer :: gvec(3,npwvec)   
  complex ::  rhotwx(3)
  integer :: ig


    rhotwx(:) = 0
!   first term <v,k|-i*grad|c,k>
    do ig = 1, npwwfn
     rhotwx(:) = rhotwx(:) + gvec(:,ig) *  conjg(wfg2(ig)) * wfg1(ig)
    enddo
!   second term <v,k|[Vnl,ir]|c,k>
    if(kb%inclvkb) then
     call vkbme(kb,npwwfn,npwvec,wfg1,wfg2,ikibz,rhotwx)
    endif
!   ediff = e_c - e_v
    rhotwx(:) = rhotwx(:) / ediff

   end
!!***

!{\src2tex{textfont=tt}}
!!****p* comm_lda/vkbme
!! NAME
!! Calculate commutator on the Kohn-Sham basis.   
!!
!! FUNCTION
!! Calculate the matrix element:
!! <n | [Vnl,ir] | n'> 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!
!! PARENTS
!! comm_lda, calccomm_drude
!!
!! SOURCE

subroutine vkbme(kb,npwwfn,npwvec,wfg1,wfg2,ikibz,rhotwx)

  use ggwdef
  use verbose
  implicit none

type(kb_potential) kb
integer npwwfn,npwvec,ikibz
complex :: wfg1(npwvec),wfg2(npwvec) 
complex ::  rhotwx(3)

integer is,ilm,ig
complex :: cta1, cta2(3), cta3(3), cta4
complex :: fnlkslm(npwwfn),fnldkslm(3,npwwfn)


!     non-separable form
!     do ig = 1, iparam%npwwfn
!       do igp = 1, iparam%npwwfn
!         ct = conjg(wfg(ig,ibv,ikibz)) * wfg(igp,ibc,ikibz)
!        must multiply by ucvol because yet gradvnl contain 1/ucvol
!         rhotwx(:) = rhotwx(:) + ct * gradvnl(:,ig,igp,ikibz)
!        enddo
!     enddo
!    separable form
     do is=1, kb%natom
      do ilm = 1, kb%mpsang*kb%mpsang
       cta1=0
       cta2=0
       cta3=0
       cta4=0
       if (kb%DRUDE) then
        fnlkslm(:)=kb%fnl(:,ilm,is,ikibz)
        fnldkslm(:,:)=kb%fnld(:,:,ilm,is,ikibz)
       else
        fnlkslm(:)=kb%fnl(:,ilm,is,kb%k_count(ikibz))
        fnldkslm(:,:)=kb%fnld(:,:,ilm,is,kb%k_count(ikibz))
       endif
       do ig = 1, npwwfn
        cta1 = cta1 + wfg2(ig)*fnlkslm(ig)
        cta2(:) = cta2(:) + wfg1(ig)*fnldkslm(:,ig)
        cta3(:) = cta3(:) + wfg2(ig)*fnldkslm(:,ig)
        cta4 = cta4 + wfg1(ig)*fnlkslm(ig)
       enddo
       rhotwx(:)=rhotwx(:)+conjg(cta1)*cta2(:)+conjg(cta3(:))*cta4
      enddo
     enddo

end
!!***
