!{\src2tex{textfont=tt}}
!!****p* main/checks
!! NAME
!! checks
!!
!! FUNCTION
!! Interface to execute some checks (Kramers-Kronig, sumrules)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! check_kramerskronig, check_fsumrule
!!
!! SOURCE

subroutine checks(iparam,res)

 use dpdef
 use ggwdef
 use verbose

 implicit none

 type(dpparam) iparam
 type(response) res 

#ifdef MPI
 include 'mpif.h'
 integer ierr,rang,nprocs
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
 if(rang==0)then
#endif

 if(iverbose>2)call uverbose('b','checks')  

 if (iverbose > 1) then
  if (.not. iparam%exciton) then
   write(uout,*) 'check Kramers Kronig on Macroscopic Epsilon NLF'
   call check_kramerskronig(iparam%nomega,real(iparam%omega(:)),res%epsmnlf(:,1))
  else
   write(uout,*) 'check Kramers Kronig on RPA NLF Macroscopic Epsilon'
   call check_kramerskronig(iparam%nomega,real(iparam%omega(:)),res%epsrpanlf(:,1))
   write(uout,*) 'check Kramers Kronig on GW NLF Macroscopic Epsilon'
   call check_kramerskronig(iparam%nomega,real(iparam%omega(:)),res%epsgwnlf(:,1))
  endif

   write(uout,*) 'check Kramers Kronig on Macroscopic Epsilon LF'
   call check_kramerskronig(iparam%nomega,real(iparam%omega(:)),res%epsmlf(:,1))

  if (.not. iparam%exciton) then

   write(uout,*) 'check f-sum rule on Macroscopic Epsilon NLF'
   call check_fsumrule(iparam%nomega,real(iparam%omega(:)),aimag(res%epsmnlf(:,1)),iparam%omegaplasma)

  else

   write(uout,*) 'check f-sum rule on RPA NLF Macroscopic Epsilon'
   call check_fsumrule(iparam%nomega,real(iparam%omega(:)),aimag(res%epsrpanlf(:,1)),iparam%omegaplasma)
   write(uout,*) 'check f-sum rule on GW NLF Macroscopic Epsilon'
   call check_fsumrule(iparam%nomega,real(iparam%omega(:)),aimag(res%epsgwnlf(:,1)),iparam%omegaplasma)

  endif

  write(uout,*) 'check f-sum rule on Macroscopic Epsilon LF'
  write(uout,*) 'warning: f-sum rule should be checked without LF'
  call check_fsumrule(iparam%nomega,real(iparam%omega(:)),aimag(res%epsmlf(:,1)),iparam%omegaplasma)


   write(uout,*) 

 endif

 if (.not. iparam%exciton) then
   deallocate(res%epsmlf,res%epsmnlf)
 else
   deallocate(res%epsmlf,res%epsrpanlf,res%epsgwnlf)
 endif  

 if(iverbose>2)call uverbose('e','checks')  

#ifdef MPI
 endif
#endif

end subroutine checks
!!***


!{\src2tex{textfont=tt}}
!!****p* checks/check_kramerskronig
!! NAME
!! check_kramerskronig
!!
!! FUNCTION
!! check Kramers Kronig
!! \int_0^\infty d\omega' frac{\omega'}{\omega'^2 - \omega^2} 
!! Im \epsilon(\omega') = Re \epsilon(\omega)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! checks
!!
!! CHILDREN
!! simpson 
!!
!! SOURCE

subroutine check_kramerskronig(n,o,eps)

 use dpdef
 use ggwdef
 implicit none
 integer n
 real o(n)
 complex eps(n)
 complex int(n)
 real e1kk(n)
 integer i, ip
 real omega, omegap, domega
 complex c
 real kk, kkrms, eav
 complex, external :: simpson

!calculate domega step and verify all  
 domega = (o(n) - o(1)) / (n-1)
 do i = 2, n
!  if domega is not constant return
   if(domega-(o(i)-o(i-1)) > 0.001) return
 enddo
!if omega(1) is not 0 return
 if(o(1) > 0.1/HARTREE) return
!if eps(n) is not 0 warn
 if(aimag(eps(n)) > 0.1) then
   write(uout,*) 'warning: Im epsilon for omega = ',o(n)*HARTREE,'eV'
   write(uout,*) '         is not yet zero, epsilon_2 = ', aimag(eps(n))
   write(uout,*) '         Kramers Kronig test could give wrong results'
 endif
 
!perform kramers kronig      
 do i = 1, n
   omega = o(i)
   c = (0.0,0.0)
   do ip = 1, n
     if(ip == i) cycle
     omegap = o(ip)
     c = c + omegap / (omegap**2 - omega**2) * aimag(eps(ip))
   enddo
   e1kk(i) = 1.0 + 2.0/PI * domega * real(c)
 enddo
 
!perform kramers kronig with simpson integration    
 do i = 1, n
   omega = o(i)
   do ip = 1, n
     if(ip == i) cycle
     omegap = o(ip)
     int(ip) = omegap / (omegap**2 - omega**2) * aimag(eps(ip))
   enddo
   c = simpson(n,domega,int)
   e1kk(i) = 1.0 + 2.0/PI * real(c)
 enddo

!werify kramers kronig
 eav = 0.0
 kk = 0.0
 kkrms = 0.0
 do i = 1, n
   kk = kk + abs(real(eps(i)) - e1kk(i))
   kkrms = kkrms +(real(eps(i)) - e1kk(i))*(real(eps(i)) - e1kk(i))
   eav = eav + abs(real(eps(i)))
 enddo
 eav = eav / n
 kk = (kk / n) / eav
 kkrms = (kkrms / n) / (eav*eav)

 kk = abs(real(eps(1)) - e1kk(1)) / real(eps(1))

!write data
 write(uout,'(" the Kramers-Kronig is verified within ",f7.2,"%")') kk*100
 write(uout,*)


!write data      
! open(22,file='out.kk',status='unknown')
! write(22,'("# Kramers Kronig calculation of epsilon1")')
! write(22,'("# omega   epsilon1  epsilon1kk")')
! do i = 1, n
!   write(22,'(f7.3,2e15.7)') o(i)*HARTREE, real(eps(i)), e1kk(i)
! enddo
! close(22)
 
 end
!!***


!{\src2tex{textfont=tt}}
!!****p* checks/check_fsumrule
!! NAME
!! check_fsumrule
!!
!! FUNCTION
!! check f-sum rule
!! \int_0^\infty d\omega \omega Im \epsilon_GG'(q,\omega) =
!!   = \frac{1}{2} \pi \omega_p^2  \frac{\rho(G-G')}{\rho(0)} 
!!     versor(q+G) \dot versor(q+G')
!! for q = G = G' = 0, it reads:
!! \int_0^\infty d\omega \omega Im \epsilon_00(q=0,\omega) = 
!!   = \pi \omega_p^2 / 2
!! calculate only the second one
!! calculate the integral to evaluate an omega_plasma^eff
!! to compare with omega_plasma
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! checks
!!
!! CHILDREN
!! simpson 
!!
!! SOURCE

subroutine check_fsumrule(n,o,e2,omegaplasma)

 use dpdef
 use ggwdef      
 implicit none
 integer n ! number of omega
 real o(n), e2(n) ! omega and imaginary part of epsilon_00
 real omegaplasma
 integer i, ip
 real omegap, domega
 complex int(n)
 real integral
 real omegaplasmaeff, fsumrule
 complex, external :: simpson
       

!calculate domega step and verify      
 domega = (o(n) - o(1)) / (n-1)
 do i = 2, n
   if(domega-(o(i)-o(i-1)) > 0.001) return
 enddo
!if omega(1) is not 0 return
 if(o(1) > 0.1/HARTREE) return
!if eps(n) is not 0 warn
 if(e2(n) > 0.1) then
   write(uout,*) 'warning: Im epsilon for omega = ',o(n)*HARTREE,'eV'
   write(uout,*) '         is not yet zero, epsilon_2 = ', e2(n)
   write(uout,*) '         f-sum rule test could give wrong results'
 endif
 
!integrate to obtain f-sum rule
 integral = 0.0
 do ip = 1, n
   omegap = o(ip)
   integral = integral + omegap * e2(ip)
 enddo
 integral = domega * integral
 
!integrate with simpson to obtain f-sum rule   
 do ip = 1, n
   omegap = o(ip)
   int(ip) = omegap * e2(ip) 
 enddo
 integral = real(simpson(n,domega,int))
 omegaplasmaeff = sqrt(integral*2/PI)

 fsumrule = abs((omegaplasmaeff - omegaplasma)) / omegaplasma

!write data
 write(uout,'(" omega_plasma     = ",f6.2," [eV]")') omegaplasma*HARTREE
 write(uout,'(" omega_plasma^eff = ",f6.2," [eV]")') omegaplasmaeff*HARTREE
 write(uout,'(" the f-sum rule is verified within ",f6.2,"%")') fsumrule*100
 write(uout,*)

end
!!***
      


!{\src2tex{textfont=tt}}
!!****f* checks/simpson
!! NAME
!! simpson
!!
!! FUNCTION
!! Rough area evaluation
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! check_fsumrule, check_kramerskronig
!!
!! SOURCE

 complex function simpson(n,h,f)
 
 implicit none
 integer n
 real h
 complex f(n)
 
 integer i
 complex sumeven, sumodd

!if n even, neglect last value
 if((n/2)*2 == n) n = n - 1

 sumodd = 0.0
 do i = 2, n-1, 2
   sumodd = sumodd + f(i)
 enddo
 
 sumeven = 0.0
 do i = 3, n-2, 2
   sumeven = sumeven + f(i)
 enddo
 
 simpson = h/3 * (f(1) + sumodd*4 + sumeven*2 + f(n))
 
 end
!!***
