!{\src2tex{textfont=tt}}
!!****p* main/creation_chi0
!! NAME
!! creation_chi0
!!
!! FUNCTION
!! Calculate chi0
!!                    |                 <v,k-q| e^-i(q+G)r |c,k><c,k| e^i(q+G')r' |v,k-q> |
!!  chi0 == \Sigma_vck|(f_vk-q - f_ck)  --------------------------------------------------|
!!                    |                        w - (e_ck - e_vk-q) + in                   |
!!
!!  Note: c and v are indices over _all_ bands (not conduction/valence bands)
!!        
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! readchi0, pdtqrc, create_rhotw, create_den, mk_chi0_delta_body, mk_chi0_delta_wings,
!! assemblywings, sort, assemblychi0, dosym, make_hilbert, printcm, writechi0
!!
!! NOTE
!! Case printmatrixelements: the matrix elements are calculated into
!! creation_chi0 and put into res%chi0 and res%u(l)wing. In order to appear
!! in the outlf.mdf file (and so apply the test machinery) these matrix elements
!! are multiplied by Vc and then by the imaginary unit i (so that when the
!! imaginary part of epsilon is taken to produce  outlf.mdf the matrix elements
!! are written down).
!!
!! SOURCE

 subroutine creation_chi0(iparam,bzstruct,bzstructp,tablekp,elecstr,elecstrp,res,latt,Wd,trans,tr,fftmesh,coul)

 use ggwdef
 use dpdef
 use verbose
 use interfaces

 implicit none
 
 type(dpparam) iparam
 type(transition) trans(iparam%nt)
 type(transgen) tr
 type(response) res
 type(bzstructure) bzstruct, bzstructp
 type(polarizationq) tablekp
 type(electronic_structure) elecstr, elecstrp
 type(lattstruct) latt
 type(mesh) fftmesh
 type(coulomb) coul
 
 complex (kind(0.0d0)) :: den(iparam%nomega)
 double precision factocc, Wd
 real weight

 complex rhotwg(iparam%npwmat)
 complex rhotwq1, rhotwq2
 complex, allocatable :: rhotwgq(:)

 real, allocatable :: matrix_elements(:,:,:)

 complex, external :: pdtqrc

 integer ikbz, iinv,  iq, it, iq1, iq2
 integer istat, istat2, idum, iet

 integer isppol
 real, allocatable :: matel(:,:)
 integer itord
 integer, allocatable :: indx(:)
 double precision, allocatable :: ene(:)
 integer startcycle,stopcycle
 integer ntres

 integer ig, igp, io

#ifdef MPI
 include 'mpif.h'
 integer ierr,rang,nprocs
 complex,allocatable :: aux(:,:,:,:),auxchi0(:,:)
 real, allocatable :: aux2(:,:) !,auxmatel(:)
 integer :: iomega
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif

if(iverbose > 2) call uverbose('b','creation_chi0')
 
! first verify if there is not already a chi0 to read from the disk
if(iparam%readchi0) then
 call readchi0(iparam,res)
else ! create the chi0
 
 if(iverbose > 0) then
  headline="Calculating chi0"
  call print_headline()
  if (.not.(iparam%matrixel.or.iparam%JDOS)) then ! for them default is td
    if (iparam%td) call mesg('warning','antiresonant term excluded',idum,idum)
  endif
  if (iparam%hilbert) then
    write(uout,*) 
    write(uout,*) '   Hilbert transform is used for the chi0'
  endif
 endif


! allocate chi0 and wings
 allocate(res%chi0(iparam%npwmat,iparam%npwmat,iparam%nomega,iparam%nsppol),stat=istat)
 if(istat.ne.0) stop 'out of memory, chi0'
 res%chi0(:,:,:,:) = 0.0
 if(iverbose>2) then
  call memstat('(creation_chi0) res%chi0 allocated =','c',iparam%npwmat,iparam%npwmat,iparam%nomega,iparam%nsppol)
 endif
 allocate(res%lwing(iparam%npwmat,iparam%nomega,iparam%nqpol,iparam%nsppol),stat=istat)
 if(istat.ne.0) stop 'out of memory, lwing'
 allocate(res%uwing(iparam%npwmat,iparam%nomega,iparam%nqpol,iparam%nsppol),stat=istat)
 if(istat.ne.0) stop 'out of memory, uwing'
 res%lwing(:,:,:,:) = 0.0
 res%uwing(:,:,:,:) = 0.0
 if(iverbose>2) then
  call memstat('(creation_chi0) wings allocated =','c',2,iparam%npwmat,iparam%nomega,iparam%nqpol,iparam%nsppol)
 endif
 allocate(res%head(iparam%nomega,3,3,iparam%nsppol),stat=istat)
 if(istat.ne.0) stop 'out of memory, head'
 res%head(:,:,:,:) = 0.0
 if(iverbose>2) then
  call memstat('(creation_chi0) head allocated =','c',iparam%nomega,9,iparam%nsppol)
 endif

 if(iparam%matrixel) then
  allocate(matrix_elements(iparam%nomega,iparam%nqpol,iparam%nsppol),stat=istat)
  if(istat.ne.0) stop 'out of memory, matrix_elements'
  matrix_elements(:,:,:)=0
 endif
 if(iparam%matelanalysis) then
   allocate(matel(iparam%nt,iparam%nqpol),stat=istat)
   if(istat.ne.0) stop 'out of memory, matel'
 endif
 if(iparam%matrixel .or. iparam%matelanalysis) allocate(rhotwgq(iparam%nqpol))




!zero chi0 and wings
 res%chi0(:,:,:,:) = 0.0
 res%lwing(:,:,:,:) = 0.0
 res%uwing(:,:,:,:) = 0.0
 res%head(:,:,:,:) = 0.0

!loop over transitions 
#ifdef MPI
  call split_trans(1,iparam%nt,startcycle,stopcycle) 
#else
  startcycle =1; stopcycle = iparam%nt
#endif

  do it=startcycle,stopcycle
  
   if(iverbose > 2) write(uout,*)'it=',it

   if(iparam%JDOS) then
    rhotwg(:) = (1,0)
   else
!   calculate the oscillator strength   
    call create_rhotw(iparam,it,trans,tablekp,elecstr,elecstrp,fftmesh,bzstruct,bzstructp,rhotwg,.false.,iparam%nt)
   endif

!  calculate the denominator of chi0 (or the energy index in case of hilbert transform)
   call create_den(iparam,it,trans,tablekp,elecstr,elecstrp,Wd,iet,den)

!  calculate some quantities ...
   ikbz = trans(it)%k
   iinv=tablekp%iinv(ikbz)
   isppol =  trans(it)%s
   factocc = trans(it)%factocc

!  ...and create the chi0 
   if(iparam%hilbert) then  ! hilbert transform case

    call mk_chi0_delta_body(iet,iparam%nomega,iparam%npwmat,factocc,rhotwg,res%chi0)
    if(iparam%KP_THEORY) then
     do iq = 1, iparam%nqpol
      rhotwg(1) =  tr%rhotwxqpol(it,iq)
      call mk_chi0_delta_wings(iet,iq,iparam%nomega,iparam%npwmat,iparam%nqpol,factocc,rhotwg,res%lwing,res%uwing)
     enddo 
    endif
  
   else ! old 'sum over state' case

    call assemblychi0(iparam%npwmat,iparam%nomega,res%chi0(:,:,:,isppol),rhotwg,den)

    if(.not.iparam%JDOS)then
    if(iparam%KP_THEORY) then
     do iq = 1, iparam%nqpol
      rhotwg(1) =  tr%rhotwxqpol(it,iq)
      if(iparam%matrixel .or. iparam%matelanalysis .and. iparam%ig==1) rhotwgq(iq) = rhotwg(1)
      call assemblywings(iparam%npwmat,iparam%nomega,isppol,res,iq,rhotwg,den)
     enddo
     if (iparam%nqpol /= 1) then ! if the different directions are taken, get for the non diagonal response of the tensor
      do iq1=4,6   ! x,y,z are respectively columns 4,5,6
       do iq2=iq1,6
        rhotwq1=tr%rhotwxqpol(it,iq1)
        rhotwq2=tr%rhotwxqpol(it,iq2)
        res%head(:,iq1-3,iq2-3,isppol) = res%head(:,iq1-3,iq2-3,isppol) + conjg(rhotwq1) * rhotwq2 * den(:)
       enddo
      enddo
     endif
    endif
    endif
    if(iparam%matrixel) then 
      if(iparam%KP_THEORY .and. iparam%ig==1) then 
        matrix_elements(iet,:,isppol)=matrix_elements(iet,:,isppol)+rhotwgq(:)*conjg(rhotwgq(:))
      else
        matrix_elements(iet,1,isppol)=matrix_elements(iet,1,isppol)+rhotwg(iparam%ig)*conjg(rhotwg(iparam%ig))
      endif
    endif  

! ??? should add 1/q^2???
    if(iparam%matelanalysis) then
      if(iparam%KP_THEORY .and. iparam%ig==1) then 
        matel(it,1:iparam%nqpol)=rhotwgq(1:iparam%nqpol)*conjg(rhotwgq(1:iparam%nqpol))*coul%Vc(1,1:iparam%nqpol) 
      else !only one polarization!
        matel(it,1:iparam%nqpol) =rhotwg(iparam%ig)*conjg(rhotwg(iparam%ig))*coul%Vc(1,1:iparam%nqpol)
      endif
    endif !matelanalysis


   endif ! either hilbert or old sum over states if

 enddo ! it


#ifdef MPI
!if(iparam%matelanalysis)  then
! allocate(auxmatel(iparam%nt))
!call mpi_barrier(mpi_comm_world,ierr)
! do iq=1,iparam%nqpol
!  call mpi_gather(matel,iparam%nt,mpi_real, &
!      auxmatel,iparam%nt*iparam%nqpol,mpi_real,&
!      0,mpi_comm_world,ierr)
! if(rang==0)  matel(:,iq)=auxmatel(:)
! enddo
! deallocate(auxmatel)
!endif
if (rang == 0 ) then   
#endif
   if(iparam%matelanalysis) then !analysis done only on resonant transitions
    ntres = sum(iparam%ntblock(1,:))
    allocate(indx(ntres)) 
    allocate(ene(ntres))
     do it=1,ntres
      indx(it) = it
      ene(it) = trans(it)%gwen*HARTREE
     enddo
    call sort(ntres,ene,indx,0.0001) 
    open(44,file='matelanalysis.out')
    write(44,*) '  it is ik1 iki1 ib1 ik2 iki2 ib2  ene   re(matel) im(matel) (iq=1,nq)'
    do itord=1,ntres 
     it=indx(itord)
     write(44,'(i5,i3,6i4,f7.3,6e10.3)') & 
           it, trans(it)%s, & 
           tablekp%ikp(trans(it)%k),tablekp%ikpibz(trans(it)%k), trans(it)%v, &
           tablekp%ik(trans(it)%k),tablekp%ikibz(trans(it)%k), trans(it)%c, &
           ene(itord),matel(it,:)
    enddo
    close(44)

    deallocate(indx,ene)

   endif
#ifdef MPI
endif   
#endif
  
 
 ! deallocate some quantities 
 if (iparam%testrun) return  ! avoid deallocation
 if(.not.iparam%JDOS) then
   if(iparam%KP_THEORY) deallocate(tr%rhotwxqpol)
 endif
   

 if (.not.iparam%JDOS) then
  if (iparam%savememory) then
   if(.not. iparam%wf_distrib) then
      deallocate(elecstr%wfg)
      if (iparam%doublegrid) deallocate(elecstrp%wfg)
   else
      deallocate(elecstr%wfg2,elecstr%wf_count)
      if (iparam%doublegrid) deallocate(elecstrp%wfg2,elecstrp%wf_count)
   endif
  else
   deallocate(elecstr%wfr)
   if (iparam%doublegrid) deallocate(elecstrp%wfr)
  endif
  if(.not.iparam%DRUDE) then
   deallocate(bzstruct%ktabr,fftmesh%igfft)
   if (iparam%doublegrid) deallocate(bzstructp%ktabr)
  endif
 endif

 if (iparam%lifetime) then
   deallocate(elecstr%imsigma)
   if (iparam%doublegrid) deallocate(elecstrp%imsigma)
 endif
 if(.not.iparam%DRUDE) then
  deallocate(tablekp%ikibz,tablekp%iinv,tablekp%iinv2,tablekp%ig01a,tablekp%ig02a,tablekp%ig03a,tablekp%ikp)
  deallocate(tablekp%ikpibz,tablekp%ik)
 endif

 if(iparam%matrixel) then 
   res%chi0(iparam%ig,iparam%ig,:,:)=matrix_elements(:,1,:)
   if(iparam%KP_THEORY .and. iparam%ig==1) then 
     res%lwing(1,:,:,:)=matrix_elements(:,:,:)
     res%uwing(1,:,:,:)=matrix_elements(:,:,:)
   endif
   deallocate(matrix_elements)
 endif 
 if(iparam%matrixel .or. iparam%matelanalysis) deallocate(rhotwgq)
 if(iparam%matelanalysis) deallocate(matel)
 if(iparam%pes) then
  deallocate(elecstr%gwenergy)
  if (iparam%doublegrid)  deallocate(elecstrp%gwenergy)
 endif
   
#ifdef MPI


 allocate(aux(iparam%npwmat,iparam%npwmat,iparam%nomega,iparam%nsppol),stat=istat)
 if (istat /= 0) write(uout,*) 'creation_chi0: aux not allocated (1)' 

 if (istat /= 0) then !try to decrease auxchi0 dimension, but increase number of messages

  allocate(auxchi0(iparam%npwmat,iparam%npwmat),stat=istat2)
  if (istat2 /= 0) then 
   write(uout,*) 'creation_chi0: auxchi0 not allocated' 
   stop
  endif

  call mpi_barrier(mpi_comm_world,ierr)

  do isppol=1,iparam%nsppol
   do iomega=1,iparam%nomega
    auxchi0=cmplx(0.,0.)
    call mpi_reduce(res%chi0(:,:,iomega,isppol),auxchi0,iparam%npwmat*iparam%npwmat,mpi_complex,mpi_sum,0,&
                    mpi_comm_world,ierr)
    res%chi0(:,:,iomega,isppol) = auxchi0  
   enddo
  enddo

  deallocate(auxchi0)

 else ! can allocate aux:

  if(iverbose>2) call memstat('(creation_chi0) aux allocated =')

  call mpi_barrier(mpi_comm_world,ierr)

  call mpi_reduce(res%chi0,aux,iparam%npwmat*iparam%npwmat*iparam%nomega*iparam%nsppol,mpi_complex,mpi_sum,0,&
                  mpi_comm_world,ierr)
  res%chi0 = aux

  deallocate(aux)

 endif

 allocate(aux(iparam%npwmat,iparam%nomega,iparam%nqpol,iparam%nsppol),stat=istat)
 if (istat /= 0) write(uout,*) 'creation_chi0: aux not allocated (2)' 
 
 call mpi_reduce(res%lwing,aux,iparam%npwmat*iparam%nomega*iparam%nqpol*iparam%nsppol,mpi_complex,mpi_sum,0, & 
                 mpi_comm_world,ierr)
 res%lwing = aux
 call mpi_reduce(res%uwing,aux,iparam%npwmat*iparam%nomega*iparam%nqpol*iparam%nsppol,mpi_complex,mpi_sum,0, & 
                 mpi_comm_world,ierr)
 res%uwing = aux
 deallocate(aux)

 allocate(aux(iparam%nomega,3,3,iparam%nsppol), stat=istat) 
 if (istat /= 0) write(uout,*) 'creation_chi0: aux not allocated (3)' 

 call mpi_reduce(res%head,aux,iparam%nomega*9*iparam%nsppol,mpi_complex,mpi_sum,0,mpi_comm_world,ierr)
 res%head = aux
 deallocate(aux)

 if(iverbose>2)  call memstat('(creation_chi0) aux deallocated =')
  

if (rang == 0) then

#endif

 if (iparam%hilbert) call make_hilbert(iparam,res)
      
 if (iparam%nsppol == 1) then
   weight = 2.0 / iparam%nkbz
 elseif (iparam%nsppol == 2) then
   weight = 1.0 / iparam%nkbz
 endif
   
 res%chi0(:,:,:,:) = weight * res%chi0(:,:,:,:) / latt%ucvol
 res%lwing(:,:,:,:) = weight * res%lwing(:,:,:,:) / latt%ucvol
 res%uwing(:,:,:,:) = weight * res%uwing(:,:,:,:) / latt%ucvol
 res%head(:,:,:,:) = weight * res%head(:,:,:,:) / latt%ucvol


 if(iverbose > 1 .and. iparam%npwmat>=9) then
  write(uout,*) 
  write(uout,*) ' chi0(G,G") at the first omega'
   do isppol=1, iparam%nsppol
    if (iparam%nsppol == 2) then
     if (isppol==1) then
      write(uout,*) 'spin component UP UP'
      write(uout,*)
     else
      write(uout,*) 'spin component DOWN DOWN'
      write(uout,*)
     endif      
    endif
    if(iparam%KP_THEORY) then
      res%chi0(:,1,:,:) = res%lwing(:,:,1,:)
      res%chi0(1,:,:,:) = res%uwing(:,:,1,:)
    endif
    call printcm(res%chi0(:,:,1,isppol),iparam%npwmat,iparam%npwmat)
   enddo 
 endif

#ifdef MPI
endif
#endif

endif ! either read or create chi0 


 if(iparam%writechi0) then
! write chi0 down to disk
  call writechi0(iparam,res)
 endif

 if(iverbose > 0) then
  write(uout,*) '                          done'
  call print_footline()
 endif
 
 if(iverbose > 2) call uverbose('e','creation_chi0')

 end
!!***

!{\src2tex{textfont=tt}}
!!****p* creation_chi0/readchi0
!!
!! NAME
!! readchi0
!!
!! FUNCTION
!! Reading chi0 from disk 
!!        
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_chi0
!!
!! CHILDREN
!! readingchi0_tests
!!
!! SOURCE
subroutine readchi0(iparam,res)
use dpdef
use verbose
use ggwdef
implicit none
type(dpparam) iparam
type(response) res
logical chi0exists
integer ig, igp, io, istat, iq, isppol
integer npwmatread, nsppolread, nomegaread, nqpolread, lomoread, nbandsread, nkbzread, npwwfnread
character(len=62) headerchi0

 inquire(file='chi0.dat',exist=chi0exists)
 if (chi0exists) then
  headline="Reading chi0 from file chi0.dat"
  call print_headline()
  open(unit=37,file='chi0.dat',form='unformatted')
  read(37)headerchi0
  read(37)lomoread,nbandsread,nkbzread,npwwfnread
  read(37)npwmatread,nomegaread,nsppolread,nqpolread
  if(iverbose > 0) then
   write(uout,*) headerchi0
   write(uout,*) 'read ->  npwmat,nomega,nsppol,nqpol'
   write(uout,*) npwmatread,nomegaread,nsppolread,nqpolread
   write(uout,*)
   write(uout,*) 'read ->  lomo,nbands,nkbz,npwwfn'
   write(uout,*)lomoread,nbandsread,nkbzread,npwwfnread
  endif
! testing 
  call readingchi0_tests(iparam,npwmatread,nomegaread,nsppolread,nqpolread)
! allocate chi0 and wings
  allocate(res%chi0(iparam%npwmat,iparam%npwmat,iparam%nomega,iparam%nsppol),stat=istat)
  if(istat.ne.0) stop 'out of memory, chi0'
  allocate(res%lwing(iparam%npwmat,iparam%nomega,iparam%nqpol,iparam%nsppol),stat=istat)
  if(istat.ne.0) stop 'out of memory, lwing'
  allocate(res%uwing(iparam%npwmat,iparam%nomega,iparam%nqpol,iparam%nsppol),stat=istat)
  if(istat.ne.0) stop 'out of memory, uwing'
  allocate(res%head(iparam%nomega,3,3,iparam%nsppol),stat=istat)
  if(istat.ne.0) stop 'out of memory, uwing'

! read body
  do ig=1,iparam%npwmat
   do igp=1,iparam%npwmat
    read(37)((res%chi0(ig,igp,io,isppol),io=1,iparam%nomega),isppol=1,iparam%nsppol)
   enddo
   do igp=iparam%npwmat+1,npwmatread
    read(37)
   enddo
  enddo
  do ig=iparam%npwmat+1,npwmatread
   do igp=1,npwmatread
    read(37)
   enddo
  enddo

! read wings and head
  do ig=1,iparam%npwmat
    read(37)(((res%lwing(ig,io,iq,isppol),io=1,iparam%nomega),iq=1,iparam%nqpol),isppol=1,iparam%nsppol)
  enddo
  do ig=iparam%npwmat+1,npwmatread
    read(37)
  enddo
  do ig=1,iparam%npwmat
    read(37)(((res%uwing(ig,io,iq,isppol),io=1,iparam%nomega),iq=1,iparam%nqpol),isppol=1,iparam%nsppol)
  enddo
  do ig=iparam%npwmat+1,npwmatread
    read(37)
  enddo
  read(37)((((res%head(io,ig,igp,isppol),io=1,iparam%nomega),ig=1,3),igp=1,3),isppol=1,iparam%nsppol)
  close(37)
 else
  write(uout,*)'necessary file chi0.dat not existing or not readable'
  stop 'necessary file chi0.dat not existing or not readable'
 endif
end subroutine readchi0
!!***

!{\src2tex{textfont=tt}}
!!****p* creation_chi0/writechi0
!!
!! NAME
!! writechi0
!!
!! FUNCTION
!! Write chi0 on disk
!!        
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_chi0
!!
!! SOURCE
subroutine writechi0(iparam,res)
use dpdef
use verbose
use ggwdef
implicit none
type(dpparam) iparam
type(response) res
integer ig, igp, io, istat, iq, isppol
character(len=62) :: headerchi0

 if(iverbose > 1) then
  write(uout,*)
  write(uout,*)'  writechi0 flag activated - writing chi0 to disk (chi0.dat)'
  write(uout,*)
 endif
 open(unit=37,file='chi0.dat',form='unformatted')
 
 headerchi0='chi0 obtained from DP code, with the following parameters'
!write basic dimensions
 write(37)headerchi0
 write(37)iparam%lomo,iparam%nbnds,iparam%nkbz,iparam%npwwfn
 write(37)iparam%npwmat,iparam%nomega,iparam%nsppol,iparam%nqpol

!write body
 do ig=1,iparam%npwmat 
  do igp=1,iparam%npwmat 
   write(37)((res%chi0(ig,igp,io,isppol),io=1,iparam%nomega),isppol=1,iparam%nsppol) 
  enddo
 enddo

!write wings and head
 do ig=1,iparam%npwmat 
   write(37)(((res%lwing(ig,io,iq,isppol),io=1,iparam%nomega),iq=1,iparam%nqpol),isppol=1,iparam%nsppol) 
 enddo
 do ig=1,iparam%npwmat 
   write(37)(((res%uwing(ig,io,iq,isppol),io=1,iparam%nomega),iq=1,iparam%nqpol),isppol=1,iparam%nsppol) 
 enddo
 write(37)((((res%head(io,ig,igp,isppol),io=1,iparam%nomega),ig=1,3),igp=1,3),isppol=1,iparam%nsppol) 

end subroutine writechi0
!!***



!{\src2tex{textfont=tt}}
!!****p* readchi0/readingchi0_tests
!! NAME
!! readingchi0_tests
!!
!! FUNCTION
!! Testing the compatibility of chi0 dimensions as read in chi0.dat 
!!        
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! readchi0
!!
!! SOURCE
subroutine readingchi0_tests(iparam,npwmatread,nomegaread,nsppolread,nqpolread)
 use dpdef
 use verbose
 implicit none
 type(dpparam) iparam
 integer npwmatread,nomegaread,nsppolread,nqpolread

 if (npwmatread .gt. iparam%npwmat ) then
  write(uout,*)'number of npwmat required is less than the number written in chi0.dat'
  write(uout,*)'assuming expert user, the calculation will continue'
  write(uout,*)'    with npwmat = ',iparam%npwmat
 else if (npwmatread .lt. iparam%npwmat ) then
  write(uout,*)'number of npwmat required (',iparam%npwmat,') is too big'
  write(uout,*)'    chi0.dat only contains ',npwmatread
  stop 'npwmatread < iparam%npwmat'
 endif

 if (nomegaread .ne. iparam%nomega ) stop 'nomegaread /= iparam%nomega'
 if (nsppolread .ne. iparam%nsppol ) stop 'nsppolread /= iparam%nsppol'
 if (nqpolread .ne. iparam%nqpol ) stop 'nqpolread /= iparam%nqpol'
end subroutine readingchi0_tests
!!***
