!{\src2tex{textfont=tt}}
!!****p* main/creation_epsilon
!! NAME
!! creation_epsilon
!!
!! FUNCTION
!! Interface to call the different creation epsilon (RPA, ALDA, MT, etc.) 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! creation_epsrpa,creation_epslda,creation_epsalt, creation_kernelmt
!!
!! SOURCE

 subroutine creation_epsilon(iparam,elecstr,bzstruct,latt,trans,tr,res,coul,lda,mtkernel)

 use dpdef
 use ggwdef
 use verbose

 implicit none
 
 type(dpparam) iparam
 type(electronic_structure) elecstr
 type(bzstructure) bzstruct
 type(lattstruct) latt
 type(transition) trans(iparam%nt)
 type(transgen) tr
 type(response) res 
 type(coulomb) coul
 type(ldakernel) lda
 type(mappingkernel) mtkernel

#ifdef MPI
 include 'mpif.h'
 integer ierr,rang,nprocs
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)

 if (.not.iparam%ONLYDRUDE) then
  call mpi_bcast(res%chi0,iparam%npwmat*iparam%npwmat*iparam%nomega*iparam%nsppol,mpi_complex,0,mpi_comm_world,ierr)
  call mpi_bcast(res%lwing,iparam%npwmat*iparam%nomega*iparam%nqpol*iparam%nsppol,mpi_complex,0,mpi_comm_world,ierr)
  call mpi_bcast(res%uwing,iparam%npwmat*iparam%nomega*iparam%nqpol*iparam%nsppol,mpi_complex,0,mpi_comm_world,ierr)
  call mpi_bcast(res%head,iparam%nomega*9*iparam%nsppol,mpi_complex,0,mpi_comm_world,ierr)
 endif
 call mpi_bcast(coul%Vc,iparam%npwmat*iparam%nqpol,mpi_real,0,mpi_comm_world,ierr)
 if (iparam%ONLYDRUDE.or.iparam%DRUDE) then
  call mpi_bcast(res%chi0drude,iparam%npwmat*iparam%npwmat*iparam%nomega*iparam%nsppol*iparam%nqpol, & 
                 mpi_complex,0,mpi_comm_world,ierr)
 endif
#endif

if(iverbose > 2) call uverbose('b','creation_epsilon')


 if(iparam%NOFXC) then        ! RPA OR GW CALCULATION

    call creation_epsrpa(iparam,res,coul%Vc)
 
 elseif(iparam%LDA) then     ! ALDA CALCULATION

    call creation_epslda(iparam,res,coul%Vc,lda%fxclda)

  else if(iparam%FXCMT) then   ! MAPPING THEORY CALCULATION

!   create mapping theory kernel Tgg''(w)     
    call creation_kernelmt(iparam,elecstr,bzstruct,latt,trans,tr,res,mtkernel)
#ifdef MPI
 if(rang==0)then
#endif
    call creation_epsmt(iparam,res,coul%Vc,coul%qplusG,mtkernel%Tkernel)
    deallocate(res%chi0X,mtkernel%Tkernel)
    deallocate(coul%qplusG)
#ifdef MPI
 endif
#endif    

  else  ! ALTERNATIVE KERNEL, LIKE E.G. ALPHA/Q^2

    call creation_epsalt(iparam,res,coul%Vc) 

  endif

  if (.not.iparam%ONLYDRUDE) then
   deallocate(res%chi0,res%uwing,res%lwing,coul%Vc,res%head)
  endif
  if (iparam%ONLYDRUDE.or.iparam%DRUDE) then
   deallocate(res%chi0drude)
  endif
  if(iparam%LDA) deallocate(lda%fxclda)
 


if(iverbose > 2) call uverbose('e','creation_epsilon')


 end subroutine creation_epsilon
!!***


!{\src2tex{textfont=tt}}
!!****p* creation_epsilon/creation_epsrpa
!! NAME
!! creation_epsrpa
!!
!! FUNCTION
!! Creation of epsilon matrix for rpa or jdos 
!!  rpa : epsilon = 1 - Vc chi0
!!  jdos: epsilon = \delta(w-(Ec-Ev)) (for q=0 it is 1/w^2 \delta(w-(Ec-Ev))) 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! NOTE
!! Case printmatrixelements: the matrix elements are calculated into
!! creation_chi0 and put into res%chi0 and res%u(l)wing. In order to appear
!! in the outlf.mdf file (and so apply the test machinery) these matrix elements
!! are multiplied by Vc and then by the imaginary unit i (so that when the
!! imaginary part of epsilon is taken to produce  outlf.mdf the matrix elements
!! are written down).
!!
!! PARENTS
!! creation_epsilon
!!
!! CHILDREN
!! get_pols, split_trans
!!
!! SOURCE

 subroutine creation_epsrpa(iparam,res,Vc)

 use dpdef
 use ggwdef
 use verbose
#ifdef OMP
!$ use OMP_LIB
#endif

 implicit none
 
 type(dpparam) iparam
 type(response) res

 real :: Vc(iparam%npwmat,iparam%nqpol)
! complex, allocatable :: delta(:), epsinv(:,:)
 complex, allocatable :: epsspin(:,:,:),eps(:,:)
 complex :: epstmp
 integer :: io, ig, istat
 integer isppol 
 real, allocatable :: omegasquare(:)
 integer startcycle,stopcycle,rang
 integer :: irpol, iqpol1, iqpol2, npol_tensor, pol1, pol2, npols

#ifdef MPI
 include 'mpif.h'
 integer  ierr,nprocs
 complex aux(iparam%nomega)
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif

  if(iverbose > 2) call uverbose('b','creation_epsrpa')


! setup the number of different polarization to either 1 or 9 
! 9 stands for the 6 components of the dielectric tensor +
! the 3 response along the reciprocal lattice vectors 
 npols=1
 if(iparam%nqpol>1) npols = 9

 allocate(res%epsmlf(iparam%nomega,npols+2),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 allocate(res%epsmnlf(iparam%nomega,npols+2),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 if (iparam%nsppol == 2) then
  allocate(res%epsmnlfspin(iparam%nomega,npols+2,iparam%nsppol),stat=istat)
  if(istat.ne.0) stop 'out of memory'
 endif

 res%epsmlf = 0.
 res%epsmnlf = 0.
 if (iparam%nsppol == 2) then
   res%epsmnlfspin = 0.
 endif


 headline="Creation Epsilon"
 call print_headline()

 if(iverbose > 0) then
  if (iparam%JDOS) then
   write(uout,*) '                 jdos epsilon function ' 
   if(iparam%KP_THEORY) then
     write(uout,*) '                 epsilon = (1/w^2)*[ 1 - chi0]'
   else
     write(uout,*) '                 epsilon =  1 - Vc chi0'
   endif
   write(uout,*) '                 with chi0 = \sum_vc 1/[w-(Ec-Ev)+ieta]'
  else
   write(uout,*) '                 rpa epsilon function ' 
   write(uout,*) '                 epsilon = 1 - Vc chi0'
  endif
 endif




 if (iparam%JDOS) then
   allocate(omegasquare(iparam%nomega))
   if(iparam%KP_THEORY) then 
    omegasquare(:)=(real(iparam%omega(:))*real(iparam%omega(:))) / & 
                   (real(iparam%omega(iparam%nomega))*real(iparam%omega(iparam%nomega)))
    Vc(:,:)=1
   else 
    omegasquare(:)=1.0
   endif
 endif

 npol_tensor = 1
! in case of q=0 the whole dielectric tensor is evaluated
 if (iparam%nqpol > 1)  npol_tensor = 3
 do iqpol1 = 1, npol_tensor
  do iqpol2 = 1, npol_tensor

!  reduced index spanning over both columns and lines of the dielectric tensor
   call get_pols(iparam%nqpol,iqpol1,iqpol2,pol1,pol2,irpol)

   if ((irpol) >6 .and. (iparam%ONLYDRUDE.or.iparam%DRUDE)) cycle

   if (.not.iparam%ONLYDRUDE) then
   if(iparam%KP_THEORY .and. .not.iparam%JDOS) then
     res%chi0(:,1,:,:) = res%lwing(:,:,pol1,:)
     res%chi0(1,:,:,:) = res%uwing(:,:,pol2,:)
     if (iqpol2 > iqpol1) then
      res%chi0(1,1,:,:) = res%head(:,iqpol1,iqpol2,:)
     endif
   endif
   endif


#ifdef OMP
!$OMP PARALLEL PRIVATE(io,startcycle,stopcycle,rang,epstmp,eps,epsspin) 
#endif

#if defined (MPI) || defined (OMP)
 call split_trans(1,iparam%nomega,startcycle,stopcycle)
#else
 startcycle = 1
 stopcycle = iparam%nomega
#endif
 allocate(epsspin(iparam%npwmat,iparam%npwmat,iparam%nsppol))
 allocate(eps(iparam%npwmat,iparam%npwmat))
!  calculate epsilon = 1 - Vc chi0
   do io = startcycle,stopcycle

     if(iparam%matrixel) then 

      do isppol = 1, iparam%nsppol       
       epstmp = cmplx(0,1) * Vc(iparam%ig,pol2)*res%chi0(iparam%ig,iparam%ig,io,isppol)
       if (iparam%nsppol == 2) res%epsmnlfspin(io,irpol,isppol) = epstmp
! trace over spin
       res%epsmnlf(io,irpol) = res%epsmnlf(io,irpol) + epstmp 
      enddo
      res%epsmlf(io,irpol) = res%epsmnlf(io,irpol)
      if (iparam%nsppol == 2) res%epsmlfspin(io,irpol,:) = res%epsmnlfspin(io,irpol,:) 
 
     else ! normal case 

      eps(:,:) = 0. 
      do ig = 1, iparam%npwmat
       eps(ig,ig) = (1,0)
      enddo
      do isppol = 1, iparam%nsppol
       do ig = 1, iparam%npwmat
       if(iparam%ONLYDRUDE) then
        epsspin(:,ig,isppol) = - Vc(:,pol2)*res%chi0drude(:,ig,io,isppol,irpol)
       endif
       if(iparam%DRUDE) then
        epsspin(:,ig,isppol) = - Vc(:,pol2)*(res%chi0(:,ig,io,isppol)+res%chi0drude(:,ig,io,isppol,irpol))
       endif
       if((.not.iparam%ONLYDRUDE).and.(.not.iparam%DRUDE)) then
        epsspin(:,ig,isppol) = - Vc(:,pol2)*res%chi0(:,ig,io,isppol)
       endif
       enddo
! macroscopic epsilon without local fields spin resolved
       if (iparam%nsppol == 2) res%epsmnlfspin(io,irpol,isppol) = (0.5,0)+epsspin(iparam%ig,iparam%ig,isppol)
! trace over spin
       eps(:,:) = eps(:,:) + epsspin(:,:,isppol) 
      enddo

! macroscopic epsilon without local fields
      res%epsmnlf(io,irpol) = eps(iparam%ig,iparam%ig)
      
!     invert epsilon for only one column
!      call cgle(iparam%npwmat,eps,iparam%npwmat,delta,epsinv)
      call cginv(iparam%npwmat,eps(:,:),iparam%npwmat)
!     calculate macroscopic epsilon with local fields
!      res%epsmlf(io,irpol) = 1.0 / epsinv(iparam%ig) ! case cgle
      res%epsmlf(io,irpol) =   1.0 / eps(iparam%ig,iparam%ig)
     endif

   enddo !io
  deallocate(eps,epsspin)
#ifdef OMP
!$OMP END PARALLEL
#endif

#ifdef MPI
   call mpi_barrier(mpi_comm_world,ierr)
   call mpi_reduce(res%epsmlf(:,irpol),aux,iparam%nomega,mpi_complex,mpi_sum,0,mpi_comm_world,ierr)
   if(rang==0)res%epsmlf(:,irpol)=aux
   call mpi_barrier(mpi_comm_world,ierr)
   call mpi_reduce(res%epsmnlf(:,irpol),aux,iparam%nomega,mpi_complex,mpi_sum,0,mpi_comm_world,ierr)
   if(rang==0)res%epsmnlf(:,irpol)=aux
   call mpi_barrier(mpi_comm_world,ierr)
   if (iparam%nsppol == 2) then
    do isppol = 1, iparam%nsppol  
     call mpi_reduce(res%epsmnlfspin(:,irpol,isppol),aux,iparam%nomega,mpi_complex,mpi_sum,0,mpi_comm_world,ierr)
     if(rang==0) res%epsmnlfspin(:,irpol,isppol)=aux
    enddo
   endif
#endif

   if (iparam%JDOS) then
    res%epsmlf(:,irpol) =  res%epsmlf(:,irpol) /  omegasquare(:)
    if (iparam%nsppol == 2) then
     do isppol = 1, iparam%nsppol
      res%epsmnlfspin(:,irpol,isppol)=res%epsmnlfspin(:,irpol,isppol)/ omegasquare(:) 
     enddo
    endif
   endif

  enddo ! iqpol2 
 enddo ! iqpol1 

! deallocate(delta,epsinv,eps)
 if(iparam%JDOS) deallocate(omegasquare)

 if(iverbose > 0) then
  call print_footline()
 endif

 if(iverbose > 2) call uverbose('e','creation_epsrpa')

 end
!!*** 

!{\src2tex{textfont=tt}}
!!****p* creation_epsilon/creation_epslda
!! NAME
!! creation_epslda
!!
!! FUNCTION
!! Creation of epsilon matrix for alda kernel  
!!  epsilon^-1 = 1 + Vc chi
!!  chi = chi0 + chi0 (Vc + fxclda) chi
!!
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_epsilon
!!
!! CHILDREN
!! get_pols, split_trans
!!
!! SOURCE
 
 subroutine creation_epslda(iparam,res,Vc,fxc)

 use dpdef
 use ggwdef
 use verbose

 implicit none
 
 type(dpparam) iparam
 type(response) res
 

 complex :: fxc(iparam%npwmat,iparam%npwmat,iparam%nsppol,iparam%nsppol)
 real    :: Vc(iparam%npwmat,iparam%nqpol)
! complex, allocatable :: epsinv(:), fxcchi(:)
 complex epsinv,fxcchi 
 complex, allocatable :: mtmp(:,:,:,:), mtmp_te(:,:,:,:), chi(:,:,:)
 complex, allocatable :: chi0tmp(:), chitmp(:)
 complex, external :: cdotu
 integer io, ig, istat
 integer isppol,jsppol
 integer :: irpol, iqpol1, iqpol2, npol_tensor, pol1, pol2, npols
 integer startcycle,stopcycle 

#ifdef MPI
 include 'mpif.h'
 integer  ierr,rang,nprocs
 complex aux(iparam%nomega)
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif

 if(iverbose > 2) call uverbose('b','creation_epslda')


! setup the number of different polarization to either 1 or 9 
! 9 stands for the 6 components of the dielectric tensor +
! the 3 response along the reciprocal lattice vectors 
 npols=1
 if(iparam%nqpol>1) npols = 9

 allocate(res%epsmlf(iparam%nomega,npols+2),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 allocate(res%epsmnlf(iparam%nomega,npols+2),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 if (iparam%nsppol == 2) then
  allocate(res%epsmlfspin(iparam%nomega,npols+2,iparam%nsppol),stat=istat)
  if(istat.ne.0) stop 'out of memory'
  allocate(res%epsmnlfspin(iparam%nomega,npols+2,iparam%nsppol),stat=istat)
  if(istat.ne.0) stop 'out of memory'
 endif

 res%epsmlf = 0.
 res%epsmnlf = 0.

#ifdef MPI
 call split_trans(1,iparam%nomega,startcycle,stopcycle)
#else
 startcycle = 1
 stopcycle = iparam%nomega
#endif

 if(iverbose > 0) then
  headline='Creation Epsilon'
  call print_headline()
  write(uout,*) '                  lda epsilon function ' 
  if(iparam%TESTELECTRON) then
   write(uout,*) ' testelectron:   epsilon^-1 = 1 + (Vc+fxc)* chi'
  else ! iparam%TESTPARTICLE
   write(uout,*) ' testparticle:   epsilon^-1 = 1 + Vc* chi' 
  endif
  write(uout,*) '                  chi = chi0 + chi0 (Vc +fxclda ) chi'
  write(uout,*)
 endif


 allocate(mtmp(iparam%npwmat,iparam%nsppol,iparam%npwmat,iparam%nsppol))
!
 if(iparam%TESTELECTRON) then
  allocate(mtmp_te(iparam%npwmat,iparam%nsppol,iparam%npwmat,iparam%nsppol))
 end if
 allocate(chi(iparam%npwmat,iparam%nomega,iparam%nsppol))

 allocate(chi0tmp(iparam%nsppol*iparam%npwmat))
 allocate(chitmp(iparam%nsppol*iparam%npwmat))


 
! allocate(fxcchi(iparam%nsppol))
! allocate(epsinv(iparam%nsppol))

 npol_tensor = 1
! in case of q=0 the whole dielectric tensor is evaluated
 if (iparam%nqpol > 1)  npol_tensor = 3
 do iqpol1 = 1, npol_tensor
  do iqpol2 = 1, npol_tensor

!  reduced index spanning over both columns and lines of the dielectric tensor
   call get_pols(iparam%nqpol,iqpol1,iqpol2,pol1,pol2,irpol)

   if ((irpol) >6 .and. (iparam%ONLYDRUDE.or.iparam%DRUDE)) cycle

   if (.not.iparam%ONLYDRUDE) then
   if(iparam%KP_THEORY) then
     res%chi0(:,1,:,:) = res%lwing(:,:,pol1,:)
     res%chi0(1,:,:,:) = res%uwing(:,:,pol2,:)
     if (iqpol2 > iqpol1) then
      res%chi0(1,1,:,:) = res%head(:,iqpol1,iqpol2,:)
     endif
   endif
   endif

       if(iparam%ONLYDRUDE) then
        res%chi0(:,:,:,:) = res%chi0drude(:,:,:,:,irpol)
       endif
       if(iparam%DRUDE) then
        res%chi0(:,:,:,:) = res%chi0(:,:,:,:)+res%chi0drude(:,:,:,:,irpol)
       endif
   
!  let us calculate eps = 1 -v chi0
   res%epsmnlf(:,irpol) = (1,0)
   do isppol = 1, iparam%nsppol
    if (iparam%nsppol == 2) then
     res%epsmnlfspin(:,irpol,isppol) = (0.5,0) -  Vc(iparam%ig,pol2) * res%chi0(iparam%ig,iparam%ig,:,isppol)
    endif
    res%epsmnlf(:,irpol) =  res%epsmnlf(:,irpol) - Vc(iparam%ig,pol2) * res%chi0(iparam%ig,iparam%ig,:,isppol)
   enddo

   do io = startcycle,stopcycle


!             || 1 - chi0 v - chi0 f11           - chi0 v - chi0 f12 ||
! calculating ||                                                     ||
!             || - chi0 v - chi0 f21           1 - chi0 v - chi0 f22 ||

!   calculating:  - chi0 * fxc 
!   and - fxc * chi0 if TE case
    do isppol = 1, iparam%nsppol
     do jsppol = 1, iparam%nsppol     
      mtmp(:,isppol,:,jsppol) = - matmul(res%chi0(:,:,io,isppol),fxc(:,:,isppol,jsppol))
      if(iparam%TESTELECTRON) then
       mtmp_te(:,isppol,:,jsppol) = &
       & - matmul(fxc(:,:,isppol,jsppol),res%chi0(:,:,io,isppol))
      end if
     enddo
    enddo

!    calculating 1 - chi0*Vc - chi0*fxc
!   and   1 - Vc*chi0 - fxc * chi0   if TE case
     do isppol = 1, iparam%nsppol
      do jsppol = 1, iparam%nsppol
       do ig = 1, iparam%npwmat
        mtmp(ig,isppol,:,jsppol) = mtmp(ig,isppol,:,jsppol) - res%chi0(ig,:,io,isppol) * Vc(:,pol2) 
        if(iparam%TESTELECTRON) then 
         mtmp_te(ig,isppol,:,jsppol) = &
         & mtmp_te(ig,isppol,:,jsppol) -  Vc(:,pol2) * res%chi0(:,ig,io,isppol)
        end if
       enddo 
      enddo
      do ig = 1, iparam%npwmat
       mtmp(ig,isppol,ig,isppol) = 1.0 + mtmp(ig,isppol,ig,isppol)
        if(iparam%TESTELECTRON) then 
         mtmp_te(ig,isppol,ig,isppol) = 1.0 + mtmp_te(ig,isppol,ig,isppol)
        end if
      enddo
     enddo

    if(iparam%TESTELECTRON) then
     res%epsmnlf(io,irpol) =  mtmp_te(iparam%ig,1,iparam%ig,1)
    endif

!    solving (1 - chi0*Vc - chi0*fxc)*chi=chi0 for chi
     do isppol = 1, iparam%nsppol
      do ig = 1, iparam%npwmat
       chi0tmp((isppol-1)*iparam%npwmat +ig) = res%chi0(ig,iparam%ig,io,isppol)
      enddo
     enddo 

!     call cgle(iparam%nsppol*iparam%npwmat,mtmp,iparam%nsppol*iparam%npwmat, & 
!                  res%chi0(:,iparam%ig,io,iparam%nsppol),chi(:,io,iparam%nsppol))
      call cgle(iparam%nsppol*iparam%npwmat,mtmp,iparam%nsppol*iparam%npwmat, chi0tmp,chitmp)    

     do isppol = 1, iparam%nsppol
      do ig = 1, iparam%npwmat
         chi(ig,io,isppol) = chitmp((isppol-1)*iparam%npwmat +ig)
      enddo
     enddo 

    enddo !io
 

!  calculate epsilon^-1

   if(iparam%TESTELECTRON) then !!! SPIN TODO !!!

!epsilon^-1 = 1+ spinTrace [ Vc* chi + fxc*chi]
    do io = startcycle,stopcycle
! TODO fxcchi spin dependent
       fxcchi = cdotu(iparam%npwmat,fxc(iparam%ig,:,iparam%nsppol,iparam%nsppol),1,chi(:,io,iparam%nsppol),1)
       epsinv = 1.0 * iparam%nsppol + Vc(iparam%ig,pol2)*chi(iparam%ig,io,iparam%nsppol)+fxcchi
       res%epsmlf(io,irpol) = 1.0 / epsinv
    enddo !io


   else ! case iparam%TESTPARTICLE 

!epsilon^-1 =  1 + spinTrace [ Vc* chi]
    res%epsmlf(startcycle:stopcycle,irpol) =  1.0 
    do isppol = 1, iparam%nsppol
      res%epsmlf(startcycle:stopcycle,irpol) = res%epsmlf(startcycle:stopcycle,irpol) &         
      &   + Vc(iparam%ig,pol2)*chi(iparam%ig,startcycle:stopcycle,isppol)
      if (iparam%nsppol == 2) then
       res%epsmlfspin(startcycle:stopcycle,irpol,isppol) =  0.5 + Vc(iparam%ig,pol2)*chi(iparam%ig,startcycle:stopcycle,isppol)  
      endif
    enddo
    res%epsmlf(startcycle:stopcycle,irpol) = 1.0 / res%epsmlf(startcycle:stopcycle,irpol)


   endif  !testelectron

#ifdef MPI
   call mpi_barrier(mpi_comm_world,ierr)
   call mpi_reduce(res%epsmlf(:,irpol),aux,iparam%nomega,mpi_complex,mpi_sum,0,mpi_comm_world,ierr)
   if(rang==0)res%epsmlf(:,irpol)=aux
   if (iparam%nsppol == 2) then
    do isppol = 1, iparam%nsppol  
     call mpi_reduce(res%epsmlfspin(:,irpol,isppol),aux,iparam%nomega,mpi_complex,mpi_sum,0,mpi_comm_world,ierr)
     if(rang==0) res%epsmlfspin(:,irpol,isppol)=aux
    enddo
   endif
#endif

  enddo !iqpol2
 enddo !iqpol1

! deallocate(mtmp,chi,fxcchi,epsinv)
 deallocate(mtmp,chi)
 if(iparam%TESTELECTRON) deallocate(mtmp_te)
 deallocate(chi0tmp,chitmp)    


 if(iverbose > 0) then
  call print_footline()
 endif
 
 if(iverbose > 2) call uverbose('e','creation_epslda')

 end
!!***

 

!{\src2tex{textfont=tt}}
!!****p* creation_epsilon/creation_epsmt
!! NAME
!! creation_epsmt
!!
!! FUNCTION
!! Creation of epsilon matrix for mapping theory kernel
!!  epsilon^-1 = 1 + Vc chi
!!  chi = chi0 + chi0 (Vc + fxcmt) chi
!!
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_epsilon
!!
!! CHILDREN
!! get_pols
!!
!! SOURCE

 subroutine creation_epsmt(iparam,res,Vc,qplusG,Tkernel)

 use dpdef
 use ggwdef
 use verbose
 use interfaces

 implicit none
 
 type(dpparam) iparam
 type(response) res

 

 real    :: Vc(iparam%npwmat,iparam%nqpol), qplusG(iparam%npwmat,iparam%nqpol)
 complex (kind(0.0d0)) :: Tkernel(iparam%npwmat,iparam%npwmat,iparam%nomega)

 complex (kind(0.0d0)), allocatable :: fxcw(:,:,:),mtmpd(:,:,:),mtmpd2(:,:,:),mtmpd3(:,:)
 complex (kind(0.0d0)), allocatable :: epsm1(:,:,:)
 complex (kind(0.0d0)), allocatable :: chi2d(:,:,:),chi_gd(:,:,:),chi(:,:),chi3(:,:)
 
 
 real  :: omegaev
 integer iqpol, io, ig1, ig2, nn, istat

 integer isppol
 
 logical :: inclkernel
 complex, external :: cdotu
 complex aux(iparam%npwmat) 

 if(iverbose > 2) call uverbose('b','creation_epsmt')

 if(iverbose > 0) then
  headline= 'Creation Epsilon'
  call print_headline()
  write(uout,*) '                  MT epsilon function ' 
  write(uout,*) '                 epsilon^-1 = 1 + Vc chi'
  write(uout,*) '             chi = X ( X - chi0 Vc X + T )^-1 chi0'
 endif

 allocate(fxcw(iparam%npwmat,iparam%npwmat,iparam%nomega))
 if(iverbose>2) then
  call memstat('(creation_epsilon) fxcw allocated =') 
 endif                                         
 allocate(mtmpd(iparam%npwmat,iparam%npwmat,iparam%nsppol))
 if(iverbose>2) then
  call memstat('creation_epsilon) mtmpd allocated =') 
 endif                                         
 allocate(epsm1(iparam%npwmat,iparam%npwmat,iparam%nsppol))
 if(iverbose>2) then
  call memstat('(creation_epsilon) epsm1 allocated =') 
 endif                                         
 allocate(chi2d(iparam%npwmat,iparam%npwmat,iparam%nsppol))
 if(iverbose>2) then
  call memstat('(creation_epsilon) chi2d allocated =') 
 endif                                         
 allocate(chi_gd(iparam%npwmat,iparam%nomega,iparam%nsppol))
 if(iverbose>2) then
  call memstat('(creation_epsilon) chi_gd allocated =') 
 endif                                        
 allocate(chi(iparam%nomega,iparam%nsppol))
 if(iverbose>2) then
  call memstat('(creation_epsilon) chi allocated =') 
 endif                                         
 allocate(chi3(iparam%npwmat,iparam%nsppol))
 if(iverbose>2) then
  call memstat('(creation_epsilon) chi3 allocated =') 
 endif                                         
 allocate(mtmpd2(iparam%npwmat,iparam%npwmat,iparam%nsppol))
 if(iverbose>2) then
  call memstat('(creation_epsilon) mtmpd2 allocated =') 
 endif                                         


 allocate(res%epsmlf(iparam%nomega,iparam%nqpol+2),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 allocate(res%epsmnlf(iparam%nomega,iparam%nqpol+2),stat=istat)
 if(istat.ne.0) stop 'out of memory'

 if(iverbose > 0) then
  if(iparam%kernel) then
   write(uout,*)'calculating'
   write(uout,*) 'T(g,g'',w) -->  fxc(g,g'',w)'
   write(uout,*)'chi=(1-chi0*(v+fxc))^-1*chi0'
  else ! i.e. when one wants to use the Tkernel
   if(iparam%TESTPARTICLE)then
     write(uout,*)'calculating '
     write(uout,*)'chi=chi0*(chi0-chi0*v*chi0 +K(g,g'',w))^-1*chi0'
     write(uout,*)'with K(g,g'',w) = chi0(g,g'',w)*W*chi0(g,g'',w)' 
   else  ! TESTELECTRON
     write(uout,*)'calculating '
     write(uout,*)'eps^1=(chi0-chi0*v*chi0 +K(g,g'',w))^-1*chi0'
     write(uout,*)'with K(g,g'',w) = chi0(g,g'',w)*W*chi0(g,g'',w)' 
   endif
  endif
 endif

 do iqpol=1,iparam%nqpol
  if(iparam%KP_THEORY) then
    if(iverbose > 1) write(uout,*) 'q-point number ', iqpol
    res%chi0(:,1,:,:) = res%lwing(:,:,iqpol,:)
    res%chi0(1,:,:,:) = res%uwing(:,:,iqpol,:)
  endif

  if(iparam%kernel) then
   
   fxcw(:,:,:)=(0,0)

   inquire(file='in.fxc',exist=inclkernel)
   if (.not.inclkernel) then
    if(iparam%quick) then


!LC add external loop for nsppol
     do isppol = 1, iparam%nsppol
      do io = 1,iparam%nomega
       aux(1) = res%lwing(1,io,iqpol,isppol) * Vc(1,iqpol)
       aux(2:) = res%lwing(2:,io,iqpol,isppol) 
!       aux(:) = lwing2(:,io,iqpol) ! * Vc(:,iqpol)
      
!       fxcw(1,1,io)= - Vc(1,iqpol) * Tkernel(1,1,io)/ &
!             cdotu(iparam%npwmat,res%uwing(:,io,iqpol),1,aux(:),1)
!       do ig1=1,iparam%npwmat
!        fxcw(ig1,ig1,io)= - Tkernel(1,1,io)/ &
!            cdotu(iparam%npwmat,uwing2(:,io,iqpol),1,aux(:),1)
!       enddo !ig1
       write(uout,*)'w,fxc(w) = ',io,fxcw(1,1,io)     
       chi2d(:,:,isppol)=(0,0)
      enddo!io
     enddo!isppol
!   
!    
    else
!   
! marghe commento questa allocazione di memoria
!     allocate(mtmpd2(iparam%npwmat,iparam%npwmat,iparam%nsppol))
!     if(iverbose>2) then
!      call memstat('(creation_epsilon) mtmpd2 allocated =') 
!     endif                                         
!
!LC add external loop for nsppol
     do isppol = 1, iparam%nsppol
!
      do io = 1, iparam%nomega
       write(uout,*)io
! marghe: I add the case X^-1X
       if(iparam%xx) then
         mtmpd(:,:,isppol) = res%chi0X(:,:,io,isppol)
         call zginv(iparam%npwmat,mtmpd(:,:,isppol),iparam%npwmat)
         mtmpd2(:,:,isppol)= mtmpd(:,:,isppol) 
       else
         mtmpd(:,:,isppol) = res%chi0(:,:,io,isppol)
         mtmpd2(:,:,isppol) = res%chi0X(:,:,io,isppol)
         call zginv(iparam%npwmat,mtmpd(:,:,isppol),iparam%npwmat)
         call zginv(iparam%npwmat,mtmpd2(:,:,isppol),iparam%npwmat)
       endif
       chi2d(:,:,isppol) = matmul(Tkernel(:,:,io),mtmpd2(:,:,isppol))
       fxcw(:,:,io) = -matmul(mtmpd(:,:,isppol),chi2d(:,:,isppol))
      enddo!io
!
     enddo!isppol
!    
     deallocate(mtmpd2)
    endif 
   endif
   if (inclkernel) then
     open(57,file='in.fxc',status='old',form='unformatted')
     open(58,file='param.ng',status='old')
     read(58,*)nn
     do io=1,iparam%nomega
      read(57)((fxcw(ig1,ig2,io),ig1=1,nn),ig2=1,nn)
     enddo
     close(57)
     close(58)
   else
    if(iverbose > 0) write(uout,*)'fxc obtained'
    open(56,file='outf.fxc',status='unknown')
    open(58,file='outf.chi0',status='unknown')
    open(57,file='out.fxc',status='unknown',form='unformatted')
!
!LC add external loop for nsppol
    do isppol = 1, iparam%nsppol 
     do io = 1, iparam%nomega
    
      omegaev = real(iparam%omega(io))*HARTREE
     
      write(57)((fxcw(ig1,ig2,io),ig1=1,iparam%npwmat),ig2=1,iparam%npwmat)
      write(56,*)omegaev,qplusG(1,1)**2*real(fxcw(1,1,io)),&
                 qplusG(1,1)**2*aimag(fxcw(1,1,io))
      write(58,*)omegaev,real(res%chi0(1,1,io,isppol))/(qplusG(1,1)**2),&
                 aimag(res%chi0(1,1,io,isppol))/(qplusG(1,1)**2)
     enddo!io
    enddo!isppol
!   
    close(57)
    close(58)
    close(56)
!    stop 'fxc(g,gp,w) written in out.exc. name it in.fxc to be rewritten'
   endif
   
!LC add external loop for nsppol
   do isppol = 1, iparam%nsppol
!   
    do io = 1, iparam%nomega
     mtmpd(:,:,isppol) = res%chi0(:,:,io,isppol)
     chi2d(:,:,isppol) = fxcw(:,:,io)
      do ig1 = 1,iparam%npwmat
       chi2d(ig1,ig1,isppol) = Vc(ig1,iqpol) + chi2d(ig1,ig1,isppol)
      enddo
       chi2d(:,:,isppol) = - matmul(mtmpd(:,:,isppol),chi2d(:,:,isppol))
      do ig1 = 1, iparam%npwmat
       chi2d(ig1,ig1,isppol) = 1.d0 + chi2d(ig1,ig1,isppol)
      enddo
!      call zginv(iparam%npwmat,chi2d,iparam%npwmat)
!      kernelomega(:,:,io)=matmul(chi2d,mtmpd)
!      solving (1 - chi0*Vc - chi0*fxc) * chi = chi0  for chi
     call zgle(iparam%npwmat,chi2d(:,:,isppol),iparam%npwmat,mtmpd(:,iparam%ig,isppol),chi_gd(:,io,isppol))
    enddo! io
   
   enddo! isppol
!   
!  calculate epsilon-twiddle^-1 = 1 + Vc chi
!  calculate macroscopic epsilon
   
   res%epsmlf(:,iqpol) = 1.  
   do isppol = 1, iparam%nsppol !trace over spin
!   epsmlf(:,iqpol)= 1.0/(1+Vc(iparam%ig,iqpol)*kernelomega(iparam%ig,iparam%ig,:))
    res%epsmlf(:,iqpol) = res%epsmlf(:,iqpol) + Vc(iparam%ig,iqpol)*chi_gd(iparam%ig,:,isppol)
   enddo
   res%epsmlf(:,iqpol) = 1.0 / res%epsmlf(:,iqpol)

!!!    res%epsmnlf(:,iqpol) = res%epsmlf(:,iqpol)

  else ! i.e. when one wants to use the Tkernel

   if(iparam%TESTPARTICLE)then
     do isppol = 1, iparam%nsppol
      chi(:,isppol)=(0.0,0.0)
      do io=1,iparam%nomega

!      aux(1)=Tkernel(1,1,io)
!       Tkernel(:,:,io)=0
!       Tkernel(1,1,io)=aux(1)
!      pay attention: this is chi0*Vc (not the contrary)
        do ig2=1,iparam%npwmat
         do ig1=1,iparam%npwmat
          mtmpd(ig1,ig2,isppol) = res%chi0(ig1,ig2,io,isppol)*Vc(ig2,iqpol) 
         enddo!ig1
        enddo!ig2
       mtmpd2(:,:,isppol) = mtmpd(:,:,isppol)

       mtmpd(:,:,isppol) = matmul(mtmpd2(:,:,isppol),res%chi0X(:,:,io,isppol))
!      questo serve per X-1 X
       allocate(mtmpd3(iparam%npwmat,iparam%npwmat))
       if(iparam%xx) then
         mtmpd2(:,:,isppol) = res%chi0X(:,:,io,isppol)
         call zginv(iparam%npwmat,mtmpd2,iparam%npwmat)
         mtmpd3 = mtmpd2(:,:,isppol)
         mtmpd2(:,:,isppol) = matmul(res%chi0(:,:,io,isppol),mtmpd3)
         mtmpd2(:,:,isppol) = matmul(mtmpd2(:,:,isppol),Tkernel(:,:,io))
         mtmpd(:,:,isppol) = res%chi0X(:,:,io,isppol)- mtmpd(:,:,isppol)+ mtmpd2(:,:,isppol)
       else
         mtmpd(:,:,isppol) = res%chi0X(:,:,io,isppol)- mtmpd(:,:,isppol)+ Tkernel(:,:,io) ! originale
       endif
       deallocate(mtmpd3)
!      end questo serve per X-1 X
       chi3(:,isppol) = res%chi0(:,iparam%ig,io,isppol)
       call zgle(iparam%npwmat,mtmpd,iparam%npwmat,chi3(:,isppol),chi3(:,isppol))
       do ig1=1,iparam%npwmat
         chi(io,isppol) = chi(io,isppol) + res%chi0X(iparam%ig,ig1,io,isppol) * chi3(ig1,isppol)
       enddo
      enddo ! io
     enddo ! isppol

!     calculate epsilon-twiddle^-1 = 1 + Vc chi
!     calculate macroscopic epsilon
     res%epsmlf(:,iqpol) = 1.0
     do isppol =1,iparam%nsppol !trace over spin   
      res%epsmlf(:,iqpol) = res%epsmlf(:,iqpol) + Vc(iparam%ig,iqpol)*chi(:,isppol)
     enddo
     res%epsmlf(:,iqpol) = 1.0/ res%epsmlf(:,iqpol) 
     res%epsmnlf(:,iqpol) = res%epsmlf(:,iqpol)
    
!     
!    
   else  ! TESTELECTRON
!   
    chi(:,:)=(0.0,0.0)
!LC add external loop for nsppol
    allocate(mtmpd3(iparam%npwmat,iparam%npwmat))
    do isppol = 1, iparam%nsppol
     do io = 1, iparam%nomega
       do ig2=1,iparam%npwmat
        do ig1=1,iparam%npwmat
         mtmpd(ig1,ig2,isppol) = res%chi0(ig1,ig2,io,isppol) * Vc(ig2,iqpol)
        enddo!ig1
       enddo!ig2
       mtmpd3=mtmpd(:,:,isppol)
       mtmpd(:,:,isppol) = matmul(mtmpd3,res%chi0(:,:,io,isppol))
       mtmpd(:,:,isppol) = res%chi0(:,:,io,isppol) - mtmpd(:,:,isppol) + Tkernel(:,:,io)
       call zginv(iparam%npwmat,mtmpd,iparam%npwmat)
       epsm1(:,:,isppol) = matmul(mtmpd(:,:,isppol),res%chi0(:,:,io,isppol))
     enddo!io     
    enddo !isppol 
    deallocate(mtmpd3)
     res%epsmlf(:,iqpol) = 0.
     do isppol = 1, iparam%nsppol !trace over spin  
       res%epsmlf(:,iqpol) =  res%epsmlf(:,iqpol)+epsm1(iparam%ig,iparam%ig,isppol)
     enddo
      res%epsmlf(:,iqpol)= 1.0 / res%epsmlf(:,iqpol)
      res%epsmnlf(:,iqpol)=res%epsmlf(:,iqpol)
!     
!    
   endif
  endif
  if(iverbose > 0) write(uout,*) 'epsilon created'
 
 enddo  
 
 deallocate(fxcw,mtmpd,chi2d,chi_gd,chi,chi3)
 if (allocated(mtmpd2))  deallocate(mtmpd2)
 if(iverbose>2) then
  call memstat('(creation_epsilon) fxcw,mtmpd,chi2d DEallocated =') 
  call memstat('(creation_epsilon) chi_gd,chi,chi3,mtmpd2 DEallocated =') 
 endif                                         

 if(iverbose > 0) then
  call print_footline()
 endif
 
 if(iverbose > 2) call uverbose('e','creation_epsmt')
 end
!!***


!{\src2tex{textfont=tt}}
!!****p* creation_epsilon/creation_epsalt
!! NAME
!! creation_epsalt
!!
!! FUNCTION
!! Creation of epsilon matrix for alternative kernels  
!!  epsilon^-1 = 1 + Vc chi
!!  chi = chi0 + chi0 (Vc  - alpha / q^2) chi
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_epsilon
!!
!! CHILDREN
!! get_pols, split_trans
!!
!! SOURCE

 subroutine creation_epsalt(iparam,res,Vc) 
 
 use dpdef
 use ggwdef
 use verbose

 implicit none
 
 type(dpparam) iparam
 type(response) res

 real    :: Vc(iparam%npwmat,iparam%nqpol)
 complex, allocatable :: kernel(:,:,:), epsinv(:,:,:), chi_g(:,:,:), mtmp(:,:,:)
 complex, allocatable :: mtmp_te(:,:,:)
 complex, allocatable :: kernelomega(:,:,:)
 integer :: io, ig, iqpol, istat
 integer isppol
 integer :: irpol, iqpol1, iqpol2, npol_tensor, pol1, pol2, npols
 integer :: startcycle,stopcycle 
 
#ifdef MPI
 include 'mpif.h'
 integer ierr,rang,nprocs
 complex aux(iparam%nomega)
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif
 
 if(iverbose > 2) call uverbose('b','creation_altchi')

 npols=1
 if(iparam%nqpol>1) npols = 9

 allocate(res%epsmlf(iparam%nomega,npols+2),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 allocate(res%epsmnlf(iparam%nomega,npols+2),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 
 if(iverbose > 0) then
  headline= 'Creation Epsilon'
  call print_headline()
  write(uout,*) '                  alpha/q^2 epsilon function ' 
  write(uout,*) '                 epsilon^-1 = 1 + Vc chi'
  write(uout,*) '             chi = chi0 + chi0 (Vc - alpha/q^2 ) chi'
 endif

#ifdef MPI
 res%epsmlf = 0
 res%epsmnlf = 0
 call split_trans(1,iparam%nomega,startcycle,stopcycle)
#else
 startcycle = 1
 stopcycle =iparam%nomega
#endif

 allocate(kernel(iparam%npwmat,iparam%npwmat,iparam%nsppol))
 allocate(kernelomega(iparam%npwmat,iparam%npwmat,iparam%nomega))
 allocate(mtmp(iparam%npwmat,iparam%npwmat,iparam%nsppol))
!
 if(iparam%TESTELECTRON) allocate(mtmp_te(iparam%npwmat,iparam%npwmat,iparam%nsppol))
 allocate(chi_g(iparam%npwmat,iparam%nomega,iparam%nsppol))

 if (iparam%FXCEPS .or. iparam%FXCEPSDIAGO) then
   allocate(epsinv(iparam%npwmat,iparam%npwmat,iparam%nsppol))
   if(iverbose > 0) write(uout,*) 'calculate screening to be used in the kernel'
   iqpol = 1 ! using the first q
   io = 1 ! using the first omega (omega = 0?)
   res%chi0(:,1,:,:) = res%lwing(:,:,iqpol,:)
   res%chi0(1,:,:,:) = res%uwing(:,:,iqpol,:)
!  epsilon = 1 - Vc * chi0
   do isppol = 1, iparam%nsppol
    do ig = 1, iparam%npwmat
     epsinv(:,ig,isppol) = - Vc(:,iqpol)*res%chi0(:,ig,io,isppol)
     epsinv(ig,ig,isppol) = 1 + epsinv(ig,ig,isppol)
    enddo !ig
   enddo !isppol
!  invert epsilon to epsilon^-1
   call cginv(iparam%npwmat,epsinv(:,:,isppol),iparam%npwmat)
   if(iverbose > 0) write(uout,*) 'screening calculated'
 endif 

 if(iverbose > 0) then
   if (iparam%FXCALPHA) write(uout,*)'chi=(1 - chi0(alpha/q^2 + Vc))^-1 chi0'
   if (iparam%FXCALPHABETA) write(uout,*)'chi=(1 - chi0((alpha+beta*omega^2)/q^2 + Vc))^-1 chi0'
   if (iparam%FXCEPS) write(uout,*)'chi=(1 - chi0(eps^-1_gg''*Vc + Vc))^-1 chi0'
   if (iparam%FXCEPSDIAGO) write(uout,*)'chi=(1 - chi0(eps^-1_gg*Vc + Vc))^-1 chi0'
   if (iparam%FXCVBAR) write(uout,*)'chi=(1 - chi0(-\vbar + Vc))^-1 chi0'
 endif
 
 npol_tensor = 1
! in case of q=0 the whole dielectric tensor is evaluated
 if (iparam%nqpol > 1)  npol_tensor = 3
 do iqpol1 = 1, npol_tensor
  do iqpol2 = 1, npol_tensor 

!  reduced index spanning over both columns and lines of the dielectric tensor
   call get_pols(iparam%nqpol,iqpol1,iqpol2,pol1,pol2,irpol)

   if(iparam%KP_THEORY) then
     res%chi0(:,1,:,:) = res%lwing(:,:,pol1,:)
     res%chi0(1,:,:,:) = res%uwing(:,:,pol2,:)
     if (iqpol2 > iqpol1) then
      res%chi0(1,1,:,:) = res%head(:,iqpol1,iqpol2,:)
     endif
   endif

   kernelomega(:,:,:) = (0.0,0.0)

   kernel(:,:,:) = (0.0,0.0)     
  do isppol = 1, iparam%nsppol
!  calculate fxc
   if (iparam%FXCALPHA) then
     kernel(iparam%ig,iparam%ig,isppol)= iparam%alphadiv / (4*pi) * Vc(iparam%ig,pol2)
   else if (iparam%FXCALPHABETA) then
      do io=1, iparam%nomega 
         kernelomega(1,1,io)= (iparam%alphadiv + iparam%beta * iparam%omega(io) * iparam%omega(io)) &
              / (4*pi) * Vc(1,pol2)
      enddo
   else if (iparam%FXCEPS) then
     do ig=1,iparam%npwmat
       kernel(:,ig,isppol)=-epsinv(:,ig,isppol)*Vc(ig,pol2)
     enddo
   else if (iparam%FXCEPSDIAGO) then
     do ig=1,iparam%npwmat
       kernel(ig,ig,isppol)=-epsinv(ig,ig,isppol)*Vc(ig,pol2)
     enddo
   else if (iparam%FXCVBAR) then
     do ig=2,iparam%npwmat
       kernel(ig,ig,isppol)=-Vc(ig,pol2)
     enddo
   endif
!  now (v + fxc)
   do ig = 1,iparam%npwmat
     kernel(ig,ig,isppol)= Vc(ig,pol2) + kernel(ig,ig,isppol)
      if (iparam%FXCALPHABETA) then
         do  io=1, iparam%nomega
            kernelomega(ig,ig,io)= Vc(ig,pol2) + kernelomega(ig,ig,io)
         enddo
      endif
   enddo
  enddo !isppol
  
!  now chi = (1-chi0(fxc + v))^-1 chi0
!  calculating (1 - chi0*Vc - chi0*fxc)
! or (1 - Vc*chi0 - fxc*chi0) if TESTELECTRON
  do isppol = 1, iparam%nsppol
   do io = startcycle,stopcycle
    mtmp(:,:,isppol) = - matmul(res%chi0(:,:,io,isppol),kernel(:,:,isppol))
    if(iparam%TESTELECTRON)  mtmp_te(:,:,isppol) = &
       - matmul(kernel(:,:,isppol),res%chi0(:,:,io,isppol))
    if (iparam%FXCALPHABETA) mtmp(:,:,isppol) = &
         - matmul(res%chi0(:,:,io,isppol),kernelomega(:,:,io))
    do ig = 1, iparam%npwmat
      mtmp(ig,ig,isppol) = 1.0 + mtmp(ig,ig,isppol)
      if(iparam%TESTELECTRON) then
       mtmp_te(ig,ig,isppol) = 1.0 + mtmp_te(ig,ig,isppol)
      end if
    enddo !ig
    call cgle(iparam%npwmat,mtmp(:,:,isppol),iparam%npwmat,res%chi0(:,iparam%ig,io,isppol),chi_g(:,io,isppol))
    ! Test-electron case not correct if q=0 
    ! TODO extend to q=0 case
    if(iparam%TESTELECTRON) then
     res%epsmnlf(io,irpol) = mtmp_te(iparam%ig,iparam%ig,isppol)
    end if
   enddo !io
  enddo !isppol  


! epsilon^-1 = spinTrace[1 + Vc chi]    
! TE: epsilon^-1 = spinTrace[1 + Vc chi + fxc chi]
   res%epsmlf(startcycle:stopcycle,irpol) = 1.0 
   do isppol = 1, iparam%nsppol !over trace spin
    if(iparam%TESTELECTRON) then
     res%epsmlf(startcycle:stopcycle,irpol) =  res%epsmlf(startcycle:stopcycle,irpol) + &  
      & kernel(iparam%ig,iparam%ig,isppol)*chi_g(iparam%ig,startcycle:stopcycle,isppol)
    else
    res%epsmlf(startcycle:stopcycle,irpol) =  res%epsmlf(startcycle:stopcycle,irpol) + &  
      & Vc(iparam%ig,pol2)*chi_g(iparam%ig,startcycle:stopcycle,isppol)
    end if
   enddo
   res%epsmlf(startcycle:stopcycle,irpol) =  1.0 /  res%epsmlf(startcycle:stopcycle,irpol) 
  
#ifdef MPI
   call mpi_barrier(mpi_comm_world,ierr)
   call mpi_reduce(res%epsmlf(:,irpol),aux,iparam%nomega,mpi_complex,mpi_sum,0, &
                   mpi_comm_world,ierr)
   if(rang==0)res%epsmlf(:,irpol)=aux
#endif
   if(iparam%TESTPARTICLE) then
    res%epsmnlf(:,irpol) = res%epsmlf(:,irpol)
   end if
  
  enddo!iqpol2
 enddo!iqpol1

 deallocate(kernel,kernelomega,chi_g,mtmp)
 if(iparam%TESTELECTRON) deallocate(mtmp_te)
 if (iparam%FXCEPS .or. iparam%FXCEPSDIAGO) then
   deallocate(epsinv)
 endif

 if(iverbose > 0) then
  call print_footline()
 endif
 
 if(iverbose > 2) call uverbose('e','creation_altchi')
 end
!!***
