!{\src2tex{textfont=tt}}
!!****p* main/get_initial_states
!! NAME
!! get_initial_states
!!
!! FUNCTION
!! Allows to read separate KSS-files for initial and final states, e.g. to:
!! - go beyond the dipole approximation for q->0 (k.p perturbation theory)
!! - avoid commutator problems for non-local pseudopotentials or self-energies
!! That means we need to call again read_structure and identk.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! NOTES
!!  - USAGE: dp -i <infile> -k <kss_final_states> -k <kss_initial_states>
!!  - there is one bzstructure and electronic_structure type for each kss,
!!    all other parameters should be synchronized 
!!  - tests ok for semiconductors in rpa, alda, so+alpha for small/large q
!!  - key point is the calculation of the oscillator strength
!!
!!   rhotwg(G) = < v,kp | exp(-i*(q+G)*r) | c,k >  
!!   kp = k - q - G0
!!
!!   where |wfn2> = |k ,c> is the final state from the first kss 
!!   and   |wfn1> = |kp,v> is the initial state from the second kss for both,
!!   resonant and antiresonant transitions (v and c are NOT valence/conduction)
!! 
!! BUGS
!!  1. check deallocates
!!
!!  2. check k.p results carefully: 
!!  * rhotwg(1) are completely different, rhotwg(>1) differ sometimes
!!  * the resulting epsilon matrix is rather similar, but the wings have
!!     opposite sign (-q is assumed in k.p)
!!  * check commutator with non-local potential
!!
!!
!! PARENTS 
!! main
!!
!! CHILDREN
!! read_structure, identk, check_iparam
!!
!! SOURCE

  subroutine get_initial_states(klp,kssfilep,iparam,files,symstr,bzstruct,trspace,elecstr,& 
                                fftmesh,rot, bzstructp,elecstrp,chemstrp,iparamp,filesp)

  use ggwdef
  use dpdef
  use verbose
  implicit none
  integer,            intent(in) :: klp
  character(len=klp), intent(in) :: kssfilep
  
  type(dpparam),      intent(inout) :: iparam  ! eventually change nsh
  type(dpfiles),      intent(in) :: files
  type(symmetry),     intent(in) :: symstr
  type(bzstructure),  intent(in) :: bzstruct
  type(transposed_space),     intent(in):: trspace
  type(electronic_structure), intent(in):: elecstr
  type(mesh),         intent(in) :: fftmesh
  type(rot_table),    intent(in) :: rot

  type(bzstructure),  intent(inout) :: bzstructp
  type(electronic_structure), intent(inout):: elecstrp

  type(dpparam) iparamp
  type(dpfiles) filesp
  type(symmetry)  symstrp
  type(chemistry) chemstrp
  type(kb_potential) kbp
  type(transposed_space) trspacep
  type(screening) screenp
  type(lattstruct) lattp
  type(mesh) fftmeshp
  type(rot_table) rotp
  integer idum

  ! return if not a doublegrid calculation
  if (.not.iparam%doublegrid) then
    bzstructp=bzstruct
    elecstrp =elecstr
    return
  endif

  ! fake: read in parameters of the calcuation
  iparamp=iparam; iparamp%wfnsh=-1; iparamp%matsh=-1

  ! reading the second kss file and setting the variables; in case read scr file
  filesp=files; filesp%kl=klp; filesp%kss=kssfilep
  call read_structure(iparamp,filesp,chemstrp,symstrp,bzstructp,trspacep,elecstrp,screenp,lattp,kbp)
  ! number of shells may differ, use minimum of both kss
  iparam%nsh  = min(iparam%nsh,iparamp%nsh)
  iparamp%nsh = min(iparam%nsh,iparamp%nsh)

  ! skip setmesh and surrot - get following structures from previous mesh 
  symstrp=symstr; fftmeshp=fftmesh; rotp=rot

  ! identify k-points
  call identk(iparamp, bzstructp,fftmeshp,rotp,symstrp)



  ! CHECKS:
  !  - compatibility of the two kss files
  call check_iparam(iparam,iparamp)
  if (any(trspace%gvec(:,:).ne.trspacep%gvec(:,:))) &
    call mesg('error','trspace%gvec(:,:) differ unexpectedly',idum,idum)

  !  - debug: be sure not to have symmetries
  if ((iparam%nkbz  .ne. iparam%nkibz)  .or. (iparamp%nkbz .ne. iparamp%nkibz))     &
    call mesg('error','Symmetries have been taken into account: nkibz /= nkbz',idum,idum)

  end subroutine get_initial_states
!!***


!{\src2tex{textfont=tt}}
!!****p* main/correct_initial_states
!! NAME
!! correct_initial_states
!!
!! FUNCTION
!! recalculate GW energy and correct the Fermi level 
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! quasiparticles, Fermi, check_iparam
!!
!! SOURCE

 subroutine correct_initial_states(glp,gwfilep,qlp,qpsfilep,iparam,files,bzstruct,elecstr, bzstructp,elecstrp)

  use ggwdef
  use dpdef
  use verbose
  implicit none
   
  integer, intent(in) :: glp,qlp
  character(len=glp),intent(in) :: gwfilep
  character(len=qlp),intent(in) :: qpsfilep

  type(dpparam),      intent(in) :: iparam
  type(dpfiles),      intent(in) :: files
  type(bzstructure),  intent(in) :: bzstruct
  type(electronic_structure), intent(in):: elecstr
  type(bzstructure),  intent(inout) :: bzstructp
  type(electronic_structure), intent(inout):: elecstrp


  type(dpfiles) filesp
  type(dpparam) iparamp
  integer idum

  ! return if not a doublegrid calculation
  if (.not.iparam%doublegrid) then
    bzstructp=bzstruct
    elecstrp =elecstr
    return
  endif

  ! get the quasiparticle wfn and energies or the so energy for final states
  ! TODO: read quasiparticle energies for second kss here
  iparamp=iparam
  ! reading the second gw and qp files (in case)
  filesp=files
  filesp%gl=glp; filesp%gw=gwfilep
  filesp%ql=qlp; filesp%qps=qpsfilep
  call quasiparticles(iparamp,filesp,bzstructp,elecstrp)

  ! calculate the Fermi level for the second kss (is it ok to be different from the other?)
  call  Fermi(iparamp,elecstrp,bzstructp)
  if (elecstr%fermie .ne. elecstrp%fermie) &
    call mesg('warning','Fermi energies of the two kss files differ!',idum,idum)



  ! CHECKS:
  ! - TODO: Fermi level
  write(uout,*) "Comparison of the two kss-files:  final states (kss1) vs.  initial states (kss2)" 
  write(uout,*) "   Fermi Energies: ", elecstr%fermie, elecstrp%fermie
  write(uout,*) "   Etotal        : ", elecstr%etotal, elecstrp%etotal
  write(uout,*) "   eff. Cutoff   : ", elecstr%ecut_eff, elecstrp%ecut_eff
  write(uout,*) "   XC functional : ", elecstr%xc,     elecstrp%xc

  ! - debug: consistency between the two kss files
  call check_iparam(iparam,iparamp)
  
 end subroutine correct_initial_states
!!***


!{\src2tex{textfont=tt}}
!!****p* main/fft_initial_states
!! NAME
!! fft_initial_states
!!
!! FUNCTION
!! calculate quasiparticle wavefunctions and perform the
!! fourier transform for the second kss
!! 
!!
!! PARENTS
!! main
!!
!!
!! SOURCE

 subroutine fft_initial_states(iparam,fftmesh,elecstr, elecstrp)

  use ggwdef
  use dpdef
  use verbose
  implicit none
   
  type(dpparam),      intent(in) :: iparam
  type(mesh),         intent(in) :: fftmesh
  type(electronic_structure), intent(in):: elecstr
  type(electronic_structure), intent(inout):: elecstrp

  type(dpparam) iparamp
 
  ! return if not a doublegrid calculation
  if (.not.iparam%doublegrid) then
    elecstrp =elecstr
    return
  endif

  ! in case: calculate quasiparticle wavefunctions (and deallocate KS wfns)
  ! TODO: also ok with quasiparticles ?
  iparamp=iparam
  if (iparam%qps) call calc_wf_qp(iparamp,elecstrp)
 
  ! calculate wavefunctions in real space using FFT
  if(.not.iparam%JDOS .and. .not.iparam%savememory) then
    call fftwfn(iparamp,fftmesh,elecstrp)
  endif
  if(iparam%JDOS) deallocate(elecstrp%wfg)


  ! CHECKS:
  ! debug: consistency between the two kss files
  call check_iparam(iparam,iparamp)

 end subroutine fft_initial_states
!!***

!{\src2tex{textfont=tt}}
!!****p* get_initial_states/check_iparam
!! NAME
!! check_iparam
!!
!! FUNCTION
!! checking that the input parameters chosen are ok to handle both kss files
!!
!! PARENTS
!! get_initial_states, correct_initial_states
!!
!! SOURCE

 subroutine check_iparam(p1,p2)

  use dpdef
  implicit none  
  type(dpparam),      intent(in) :: p1, p2
  integer idum

  if (p1%npwmat .ne. p2%npwmat) call mesg('error','iparam%npwmat differ unexpectedly',idum,idum) 
  if (p1%npwwfn .ne. p2%npwwfn) call mesg('error','iparam%npwwfn differ unexpectedly',idum,idum)
  if (p1%npwvec .ne. p2%npwvec) call mesg('error','iparam%npwvec differ unexpectedly',idum,idum)
  if (p1%lomo   .ne. p2%lomo  ) call mesg('error','iparam%lomo   differ unexpectedly',idum,idum)
  if (p1%homo   .ne. p2%homo  ) call mesg('error','iparam%homo   differ unexpectedly',idum,idum)
  if (p1%lumo   .ne. p2%lumo  ) call mesg('error','iparam%lumo   differ unexpectedly',idum,idum)
  if (p1%nbnds  .ne. p2%nbnds ) call mesg('error','iparam%nbnds  differ unexpectedly',idum,idum)
  if (p1%nkibz  .ne. p2%nkibz ) call mesg('error','iparam%nkibz  differ unexpectedly',idum,idum)
  if (p1%nkbz   .ne. p2%nkbz  ) call mesg('error','iparam%nkbz   differ unexpectedly',idum,idum)
  if (p1%nsppol .ne. p2%nsppol) call mesg('error','iparam%nsppol differ unexpectedly',idum,idum)
  if (p1%nsh    .ne. p2%nsh   ) call mesg('error','iparam%nsh    differ unexpectedly',idum,idum)
  if (p1%vkb  .neqv. p2%vkb   ) call mesg('error','iparam%vkb    differ unexpectedly',idum,idum)

end subroutine check_iparam
!!***

!{\src2tex{textfont=tt}}
!!****p* qpol_and_q/check_qstruct
!! NAME
!! check_qstruct
!!
!! FUNCTION
!! checking that the input parameters chosen are ok to handle both kss files
!!
!! PARENTS
!! qpol_and_q
!!
!! SOURCE

  subroutine check_qstruct(nqibz,qstruct,qstructp)
  
  use dpdef
  use ggwdef
  use verbose
  implicit none
  
  type(qstructure) qstruct,qstructp
  integer :: nqibz

  integer :: iq 
  logical, external :: lcompv
  
  do iq=1,nqibz
    if(.not. lcompv(qstruct%qibz(:,iq),qstructp%qibz(:,iq))) then
     write(uout,*) 'doublegrid warning: qibz grids differ', iq, qstruct%qibz(:,iq),qstructp%qibz(:,iq)
    endif
  enddo
!  do iq=1,qstruct%nqbz
!    if(.not. lcompv(qstruct%qbz(:,iq),qstructp%qbz(:,iq))) then
!     write(uout,*) 'doublegrid warning: qbz grids differ', iq, qstruct%qbz(:,iq),qstructp%qbz(:,iq)
!    endif
!  enddo


  end 
!!***
