!{\src2tex{textfont=tt}}
!!****p* DP/main
!! NAME
!! dp
!!
!! FUNCTION
!! Fortran Main for the dp-code 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! NOTES
!! DP (Dielectric Properties)
!! DP is a TDDFT code, working in reciprocal space on a 
!! plane-waves basis, and in frequency domain
!! DP needs some input files in order to work, namely:
!! * parameter input file
!! * a kohn-sham structure file (_KSS file from the abinit package)
!! * gw corrections file in case of a GW-RPA calculation
!! * screening file (_SCR file from the abinit package) in case of MT
!! * a quasiparticle structure file (_QPS file from the abinit package) in case of  
!!   a calculation with quasiparticle wavefunctions  
!! * a second KSS file (and in case also GW and QPS files) shifted by qpol with respect to 
!!    the previous one in case of a doublegrid calculation
!! DP output is the log file (dp.log) plus the macroscopic dielectric 
!! function file (*.mdf)
!!
!! CHILDREN
!! header, rddpin, read_structure, setmesh, surot,
!! identk, get_initial_states, lattice,qpol_and_q, creation_tablekp,creation_tableantikp, quasiparticles, 
!! Fermi, correct_initial_states, get_dim_trans, prep_trans, prep_paral_distr,calccomm, calc_wf_qp,fftwfn, 
!! fft_initial_states,cden, ckxclda, createVc, haydock_scheme,
!! creation_exciton, solve_exciton, create_eps_exc, creation_chi0, creation_epsilon, 
!! wreps, checks
!!
!! SOURCE

!====================================================================
  integer function dp__(inputfile,il,kssfile,kl,kssfilep,klp,gwfile,gl,gwfilep,glp,qpsfile,ql,qpsfilep,qlp,scrfile,sl,&
                        vkbfile,vl,lambdafile,ll,root,rl)
!====================================================================
! phantom function to avoid underscore problem in linking C-Fortran
! call the function dp without underscore
!-------------------------------------------------------------------
  implicit none
  integer il, kl, klp, gl, glp, ql, qlp, sl, vl, ll, rl
  character (len=il) :: inputfile
  character (len=kl) :: kssfile
  character (len=klp):: kssfilep
  character (len=gl) :: gwfile
  character (len=glp):: gwfilep
  character (len=ql) :: qpsfile
  character (len=qlp):: qpsfilep
  character (len=sl) :: scrfile
  character (len=vl) :: vkbfile
  character (len=ll) :: lambdafile
  character (len=rl) :: root
  integer, external :: dp
  dp__ = dp(inputfile,il,kssfile,kl,kssfilep,klp,gwfile,gl,gwfilep,glp,qpsfile,ql,qpsfilep,qlp,scrfile,sl,vkbfile,vl,&
           lambdafile,ll,root,rl)
  end

!===================================================================
  integer function dp_(inputfile,il,kssfile,kl,kssfilep,klp,gwfile,gl,gwfilep,glp,qpsfile,ql,qpsfilep,qlp,scrfile,sl,& 
                       vkbfile,vl,lambdafile,ll,root,rl)
!===================================================================
! phantom function to avoid underscore problem in linking C-Fortran
! call the function dp without underscore
!-------------------------------------------------------------------
  implicit none
  integer il, kl, klp, gl, glp, ql, qlp, sl, vl, ll, rl
  character (len=il) :: inputfile
  character (len=kl) :: kssfile
  character (len=klp):: kssfilep
  character (len=gl) :: gwfile
  character (len=glp):: gwfilep
  character (len=ql) :: qpsfile
  character (len=qlp):: qpsfilep
  character (len=sl) :: scrfile
  character (len=vl) :: vkbfile
  character (len=ll) :: lambdafile
  character (len=rl) :: root
  integer, external :: dp
  dp_ = dp(inputfile,il,kssfile,kl,kssfilep,klp,gwfile,gl,gwfilep,glp,qpsfile,ql,qpsfilep,qlp,scrfile,sl,vkbfile,vl, &
           lambdafile,ll,root,rl)
  end

!===================================================================
  integer function dp(inputfile,il,kssfile,kl,kssfilep,klp,gwfile,gl,gwfilep,glp,qpsfile,ql,qpsfilep,qlp,scrfile,sl,& 
                      vkbfile,vl,lambdafile,ll,root,rl)
!===================================================================
! main fortran program
!-------------------------------------------------------------------
  use ggwdef
  use dpdef
  use verbose
  use interfaces
  implicit none
  integer il, kl, klp, gl, glp, ql, qlp, sl, vl, ll, rl

  character (len=il) :: inputfile
  character (len=kl) :: kssfile
  character (len=klp):: kssfilep
  character (len=gl) :: gwfile
  character (len=glp):: gwfilep
  character (len=ql) :: qpsfile
  character (len=qlp):: qpsfilep
  character (len=sl) :: scrfile
  character (len=vl) :: vkbfile
  character (len=ll) :: lambdafile
  character (len=rl) :: root

  type(dpparam) iparam, iparamp
  type(dpfiles) files, filesp
  type(symmetry) symstr
  type(chemistry) chemstr, chemstrp
  type(kb_potential) kb
  type(bzstructure) bzstruct, bzstructp        ! final and initial states
  type(qstructure)  qstruct
  type(polarizationq) tablekp,tableantikp
  type(transposed_space) trspace
  type(electronic_structure) elecstr, elecstrp ! final and initial states
  type(screening) screen
  type(lattstruct) latt
  type(rot_table) rot
  type(mesh) fftmesh
  type(ldakernel) lda
  type(transition),allocatable, dimension(:) :: trans, drude_trans
  type(transition),allocatable, dimension(:,:) :: transdrude
  type(transgen) tr, drude_tr
  type(response) res 
  type(coulomb) coul
  type(mappingkernel) mtkernel 

  character(len=10) :: code
 
!exciton
  integer, allocatable :: transtab(:,:,:,:)
  double precision :: gwgap
!end exciton

#ifdef MPI
  include 'mpif.h'
  integer ierr,rang,nprocs
  call mpi_comm_size(mpi_comm_world,nprocs,ierr)
  call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif


! manage names/unit of log/tree file (especially for MPI)
  call logfile(inputfile,il)
  if(iverbose>2) call uverbose('b','dp')  

! starting message and timing
  call header(code,inputfile,il)
  call pclock2(0) 

! group name of files in a single structure
  call filenames(files,il,kl,gl,ql,sl,vl,klp,glp,qlp,ll,rl,inputfile,kssfile,gwfile,qpsfile,&
                 scrfile,vkbfile,kssfilep,gwfilep,qpsfilep,lambdafile,root)
  
! read in parameters of the calcuation
  call rddpin(inputfile,il,iparam)
  call pclock2(10) 

! reading final states from the kss file and setting the variables; in case read scr file
  call read_structure(iparam,files,chemstr,symstr,bzstruct,trspace,elecstr,screen,latt,kb)
  call pclock2(20) 

! create the real space Mesh for fft 
  if(.not.iparam%JDOS) call setmesh(iparam,trspace,fftmesh)

! setup the r- and g- rotation tables
  if(.not.iparam%JDOS) call surot(iparam,symstr,fftmesh,trspace,rot)

! identify k-points
  call identk(iparam,bzstruct,fftmesh,rot,symstr)
  call pclock2(30) 
  
! read second kss-file (in case)
  call get_initial_states(klp,kssfilep,iparam,files,symstr,bzstruct,trspace,elecstr,& 
                          fftmesh,rot, bzstructp,elecstrp,chemstrp,iparamp,filesp)
  call pclock2(40) 

! lattice transformation matrix 
  call lattice(iparam,latt) 
! everything connected to the polarization vector qpol and grid of q points
  call qpol_and_q(iparam,latt,trspace,bzstruct,bzstructp,qstruct,kb,symstr,screen) 
  call pclock2(50) 

  if(iparam%JDOS .or. .not.iparam%KP_THEORY) deallocate(chemstr%typeat)

! create k - k' and g0 table  
  call creation_tablekp(iparam,bzstruct,bzstructp,tablekp)
  if (iparam%exciton .and. .not. iparam%td) then
   call creation_tableantikp(iparam,bzstruct,tableantikp)
  endif

! get the quasiparticle wfn and energies or the so energy, if any
  call quasiparticles(iparam,files,bzstruct,elecstr)
  call pclock2(60) 

! calculate the Fermi level and occupation numbers
  call  Fermi(iparam,elecstr,bzstruct)
  call pclock2(70) 

! correct energies and occupations for second kss-file (in case)
  call correct_initial_states(glp,gwfilep,qlp,qpsfilep,iparam,files,bzstruct,elecstr, &
                              bzstructp,elecstrp)
  call pclock2(80) 


! prepare and allocate transition object (non-linear case not ready)
  if(.not.iparam%ONLYDRUDE) then
    call get_dim_trans(iparam,elecstr,elecstrp,tablekp)
    allocate(trans(iparam%nt))
    allocate(transtab(iparam%nkbz,iparam%lomo:iparam%nbnds,iparam%lomo:iparam%nbnds,iparam%nsppol))
    call prep_trans(iparam,elecstr,elecstrp,trans,tablekp,tableantikp,transtab,gwgap)
    call prep_paral_distr(iparam,iparamp,trans,tablekp,elecstr,elecstrp,kb)  
  endif
 
! now read the wave functions
  if(iparam%oldqplda) then
   call leggi_wfg_qplda(iparam,files,elecstr)
  else 
   call leggi_wfg(iparam,files,chemstr,elecstr)
   if(iparam%doublegrid)  call leggi_wfg(iparamp,filesp,chemstrp,elecstrp)
  endif
  call pclock2(90) 
  call memstat('after leggi_wfg =','c',iparam%npwvec,elecstr%wfdim)

! prepare drude transitions
 if (iparam%DRUDE .or. iparam%ONLYDRUDE ) then
    iparam%nt=iparam%nkbz*(iparam%nbnds-iparam%lomo+1)*elecstr%nsppol
    allocate(drude_trans(iparam%nt))
    call prep_matel_drude(iparam,drude_trans,elecstr)
    call calccomm_drude(iparam,drude_tr,trspace,bzstruct,fftmesh,elecstr,latt,symstr,&
                       kb,chemstr,drude_trans)
    call get_drude_dim_trans(iparam,elecstr,bzstruct,trspace,latt,drude_tr,drude_trans,tablekp)
    deallocate(drude_trans)
    allocate(transdrude(iparam%ndtmax,iparam%nqpol))
    allocate(iparam%ndrudet(iparam%nqpol))
    call prep_drude_trans(iparam,elecstr,transdrude,bzstruct,trspace,latt,drude_tr,tablekp)
 endif

! calculate the commutator (in case)
  if(.not.iparam%JDOS .and. iparam%KP_THEORY .and. .not. (iparam%plot_exciton .or. iparam%analysis_exciton) & 
      .and. .not. iparam%ONLYDRUDE) then
    call calccomm(iparam,tr,trspace,bzstruct,elecstr,latt,symstr,&
                       kb,chemstr,trans,transtab)
  endif
  deallocate(elecstr%energy)
  call pclock2(100) 

! in case: calculate quasiparticle wavefunctions (and deallocate KS wfns)
  if (iparam%qps) call calc_wf_qp(iparam,elecstr) 
  call pclock2(110) 

! calculate wavefunctions in real space using FFT
! TODO: eventually allocate FFT+calc_wf_qp in correct_final_states ?
  if(.not.iparam%JDOS .and. .not.iparam%savememory) then
    call fftwfn(iparam,fftmesh,elecstr)
  endif
  if(iparam%JDOS) then
   if(iparam%savememory) deallocate(elecstr%wfg2)
   if(.not. iparam%savememory) deallocate(elecstr%wfg)
  endif
! fft for (quasiparticle) wavefunctions of second kss (in case)
  call fft_initial_states(iparam,fftmesh,elecstr,elecstrp)
  call pclock2(120) 

! for a tdlda calculation
  if(iparam%LDA) then
!   calculate electron density
    call cden(iparam,fftmesh,rot,latt,bzstruct,elecstr,lda)
!   calculate exchange-correlation kernel fxc lda
    call ckxclda(iparam,fftmesh,lda,trspace)
  endif
  call pclock2(130) 

! calculate Coulomb potential
  call createVc(iparam,latt,trspace,coul)
  call pclock2(140) 

! create Wtt in transition space and \tilde{rho}_t for the Mapping Theory kernel
  if(iparam%FXCMT) then
    call creation_Wtt(iparam,bzstruct,elecstr,latt,mtkernel,symstr,trspace,rot,fftmesh)
    call prep_rhotw(iparam,bzstruct,tablekp,fftmesh,elecstr,trans,tr)
  endif
  call pclock2(150) 

!=============================
!= Dyson linear TDDFT or PES =
!=============================

 if(iparam%linear .or. iparam%pes) then

! create chi0
    if(iparam%testrun) then  
      call estimate_time_chi0(iparam,bzstruct,bzstructp,tablekp,elecstr,elecstrp,res,&
                               latt,mtkernel%Wd,trans,tr,fftmesh,coul)
    else
      if (.not.iparam%ONLYDRUDE) then
       call creation_chi0(iparam,bzstruct,bzstructp,tablekp,elecstr,elecstrp,res,&
                           latt,mtkernel%Wd,trans,tr,fftmesh,coul)
       call pclock2(160) 
      endif
      if (iparam%DRUDE .or. iparam%ONLYDRUDE ) then
        call creation_chi0drude(transdrude,iparam,bzstruct,tablekp,elecstr,res,&
                                latt%ucvol,fftmesh)
        deallocate(transdrude)
       call pclock2(170) 
      endif

    endif
! create epsilon
    call creation_epsilon(iparam,elecstr,bzstruct,latt,trans,tr,res,coul,lda,mtkernel)
    call pclock2(180) 

! write out dielectric macroscopic functions (with and without
! local fields), refraction functions and dynamic structure factors
    call wreps(iparam,latt,res,files)

 endif


!===========
!= Exciton =
!===========

  if(iparam%exciton) then

! calculate excitonic hamiltonian
    call creation_exciton(iparam,files,latt,bzstruct,bzstructp,rot,fftmesh,qstruct,screen,trans, &
                          transtab,tablekp, tableantikp,elecstr,elecstrp,tr,coul,lda)
    call pclock2(160) 
    deallocate(transtab)
    
    if (iparam%haydock) then

      call haydock_scheme(iparam,files,tr,trans,latt,coul,res)
      call pclock2(170) 

    else

      call solve_exciton(iparam,files,trans,gwgap)
      call pclock2(170) 

      call create_eps_exc(iparam,latt,files,trans,coul,tr,res)
      call pclock2(180) 

    endif

  endif


! checks
   if(.not. iparam%JDOS .and. (iparam%linear .or. iparam%exciton )) call checks(iparam,res)


!===============
!= Plotexciton =
!===============

  if(iparam%plot_exciton .or. iparam%analysis_exciton) then
   call plexc(iparam,files,trans,fftmesh,elecstr,bzstruct,latt,chemstr)
  endif


!deallocate some quantities
   if(allocated(trans)) deallocate(trans)
   deallocate(iparam%omega)
  
  if(iverbose>2) then 
   call memstat('end of dp =')
   close(78)
  endif


  call pclock2(200) 
!date and time
  call pclock2(1000) 

  if(iverbose>2)call uverbose('e','dp')  
  if(iverbose>2)close(77)

  dp = 1
  write(uout,*) trim(code)//' ended'

! flush 
  close(uout)

  end

!!***
