!!****p* main/Fermi
!! NAME
!! Fermi level calculation
!!
!! FUNCTION
!! Calculation of the fermi level 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main, correct_initial_states
!!
!! CHILDREN
!! unify_spin, degenerescence, EFermi_occ_T0,occ_nb_tot_r,EFermi_occ_FD, FermiDirac 
!!
!! TODO
!! possible problems in a gw rpa or qp energy calculation: 
!! mix between lda and gw/qp energies can produce eigenvalues
!! that are not ordered in increasing order
!!
!! problem: in exciton lda-rpa has wrong occupations because 
!! they are calculated in gwrpa
!!
!! SOURCE

 subroutine Fermi(iparam,elecstr,bzstruct)
   
   use dpdef
   use ggwdef
   use verbose
   
   implicit none
   type(dpparam) iparam
   type(electronic_structure) elecstr
   type(bzstructure) bzstruct
   
   
   real*8 efermi,kBT2
   
   ! Internal Variables
   real*8 tab_Emax_k(iparam%nkibz),wtk_tot
   integer iband
   integer ikpt,ibnd  
   integer degg(iparam%nbnds*iparam%nsppol,iparam%nkibz)
 
   integer isppol
 
   ! Double precision variables
   
   !real*8 Eigen(iparam%nbnds,iparam%nkibz,iparam%nsppol)
   real*8 Eigen(iparam%nbnds*iparam%nsppol,iparam%nkibz)
   real*8 occ_tot(iparam%nbnds*iparam%nsppol,iparam%nkibz)
   real*8 wtk2(iparam%nkibz),unswtk(iparam%nkibz)

   type(tabellaspin), allocatable, dimension(:,:) :: ibntab(:,:)
   double precision, allocatable :: Eigennew(:,:)

   real*8, external :: FermiDirac

   
#ifdef MPI
   include 'mpif.h'
   logical dofermi
   integer ierr,rang,nprocs

   call mpi_comm_size(mpi_comm_world,nprocs,ierr)
   call mpi_comm_rank(mpi_comm_world,rang,ierr)

  if(.not.(iparam%mpilocal.and.rang/=0)) then

#endif

   if(iverbose > 2) call uverbose('b','fermi')

   if(iverbose > 0) then
    headline="Fermi level analysis"
    call print_headline()
   endif


 ! particular case in which we must keep the KS occupations
   if (iparam%occopt.eq. 99) then 
    write(uout,*) 'Warning: Using occupations from KSS file'
     do isppol = 1, iparam%nsppol
      do ikpt = 1, iparam%nkibz
        write(uout,'(i4,5x,8f8.4/60(9x,8f8.4/))') &
&         ikpt, (elecstr%oc(ikpt,ibnd,isppol),ibnd=1,iparam%nbnds)
      enddo
     enddo
   else  ! else do the normal fermi subroutine 

    elecstr%oc(:,:,:)=0.0

    allocate(ibntab(iparam%nkibz,iparam%nbnds*iparam%nsppol))
    allocate(Eigennew(iparam%nkibz,iparam%nbnds*iparam%nsppol))
    call unify_spin(iparam%nsppol,iparam%nbnds,iparam%nkibz,ibntab,elecstr%gwenergy,Eigennew)
     do ikpt=1, iparam%nkibz
       do ibnd =1, iparam%nbnds*iparam%nsppol
          Eigen(ibnd,ikpt) = Eigennew(ikpt,ibnd) 
       enddo
       wtk2(ikpt)=dble(bzstruct%wtk(ikpt))
     enddo
   
    deallocate(Eigennew)
    kBT2=dble(iparam%kBT)
    
   
!    we have the energy table and the weights      
    
    wtk_tot =0.d0
    do ikpt =1, iparam%nkibz
       unswtk(ikpt)=1.0d0/wtk2(ikpt)
       if((unswtk(ikpt)-int(unswtk(ikpt))).lt.0.5d0) wtk2(ikpt)=1.0d0/int(unswtk(ikpt))
       if((unswtk(ikpt)-int(unswtk(ikpt))).gt.0.5d0) wtk2(ikpt)=1.0d0/(int(unswtk(ikpt))+1)
       wtk_tot = wtk_tot + wtk2(ikpt)
    enddo
   
!   we have dealt with the single precision problems of the weights and things
!   like 'sum of weights' ==> 0.666666667 + 0.333333333 = 1. and not 0.999999992
  
    write(uout,*) ' Calculation of the Fermi Energy and occupation numbers'
    
    if (iparam%occopt.eq.0) then    !  case T=0 zero temperature
   
       write(uout,*) ' for T=0 '
       write(uout,*)
   
!       let's calculate the degeneracy     
        if(iverbose > 2) write(uout,*) 'degeneracy:'
        do ikpt = 1, iparam%nkibz
          call degenerescence(Eigen(:,ikpt),iparam%nbnds*iparam%nsppol,degg(:,ikpt))
          if(iverbose > 2) then
            write(uout,'(i4,5x,20i3/30(9x,20i3/))') &
&            ikpt, (degg(ibnd,ikpt),ibnd=1,iparam%nbnds)
          endif
        enddo
      
       call EFermi_occ_T0(iparam%nkibz,iparam%nbnds*iparam%nsppol,Eigen,&
            &               iparam%nel,iparam%nsppol,degg,efermi,&
            &               tab_Emax_k,wtk_tot,wtk2)
     
       ! On calcule le remplissage de chacun des etats avec le niveau 
       ! de Fermi final
   
       call occ_nb_tot_r(iparam%nel,wtk_tot,efermi,iparam%nsppol,Eigen,degg,wtk2,&
            &            iparam%nkibz,iparam%nbnds*iparam%nsppol,occ_tot(:,:))
       
        do ikpt = 1, iparam%nkibz
          do ibnd = 1, iparam%nbnds*iparam%nsppol
             iband = ibntab(ikpt,ibnd)%ibn
             isppol = ibntab(ikpt,ibnd)%is
             elecstr%oc(ikpt,iband,isppol)=occ_tot(ibnd,ikpt)
          enddo
        enddo
   
    else
   
       write(uout,*) ' for metallic occupation of levels (Fermi-Dirac smearing)'
       write(uout,*)
       call EFermi_occ_FD(iparam%nkibz,iparam%nbnds*iparam%nsppol,Eigen(:,:),&
            &            iparam%nel,iparam%nsppol,kBT2,efermi,&
            &            tab_Emax_k,wtk_tot,wtk2)
       
       ! On calcule le remplissage de chacun des etats avec le niveau 
       ! de Fermi final
        do ikpt =1, iparam%nkibz
          do ibnd = 1, iparam%nbnds*iparam%nsppol
             iband = ibntab(ikpt,ibnd)%ibn
             isppol = ibntab(ikpt,ibnd)%is
             occ_tot(ibnd,ikpt)=FermiDirac(Eigen(ibnd,ikpt),iparam%nsppol,efermi,kBT2)
             elecstr%oc(ikpt,iband,isppol)=occ_tot(ibnd,ikpt)
          enddo
        enddo
   
    endif !occopt
   
    if(iverbose>2) then 
       write(uout,*)
       do isppol = 1, iparam%nsppol
        do ikpt = 1, iparam%nkibz
          write(uout,'(" ik = ",i4,"  wtk =",f8.4,&
               &" Emax_k =",f8.4," eV  EFermi =",f8.4," eV")')&
               &     ikpt,wtk2(ikpt),tab_Emax_k(ikpt)*HARTREE, efermi*HARTREE       
        enddo
       enddo!isppol 
    endif
  
    write(uout,*)
    write(uout,'(" Fermi energy = ",f8.4," [eV]",3x,f8.5," [Ha]")') efermi*HARTREE, efermi
    if (iparam%occopt.ne.0) write(uout,'(" kBT          = ",f8.4," [eV]")') iparam%kBT*HARTREE
    write(uout,*)      

   elecstr%fermie = efermi

    write(uout,*) 'Occupation numbers after recalculation:'   
!       
    do isppol = 1, iparam%nsppol
     do ikpt = 1, iparam%nkibz
      write(uout,'(i4,5x,8f8.4/60(9x,8f8.4/))') &
&       ikpt, (elecstr%oc(ikpt,ibnd,isppol),ibnd=1,iparam%nbnds)
     enddo
    enddo 
   
    deallocate(ibntab)

   endif


   if(iverbose > 0) then
    call print_footline()
   endif
   if(iverbose > 2) call uverbose('e','fermi')



#ifdef MPI
 endif
 call mpi_barrier(mpi_comm_world,ierr)
 call mpi_bcast(elecstr%oc,iparam%nkibz*iparam%nbnds*iparam%nsppol,mpi_real,0,mpi_comm_world,ierr)
 call mpi_bcast(elecstr%fermie,1,mpi_double_precision,0,mpi_comm_world,ierr)
#endif

 if(.not.iparam%lda) deallocate(bzstruct%wtk)
  
 end subroutine Fermi

!!***


!{\src2tex{textfont=tt}}
!!****p* Fermi/EFermi_occ_FD
!! NAME
!! Fermi level and occupation calculation at T /= 0
!!
!! FUNCTION
!!
!! Calcul pour les energies de tous les points k l'energie qui remplit les etats   
!! a T donne (Equivalent de EFermi si un seul point k)
!! 1)  On cherche le niveau de Fermi                                                
!! 1a) On cherche l'energie qui permet de remplir pour chaque point k              
!! toutes les bandes avec le nombre d'electrons Nel par point k : Emax_k        
!! 1b) On cherche la valeur minimale et maximale de Emax_k                          
!! 1c) On cherche le niveau de Fermi entre ces deux valeurs minimale et maximale     
!! comme etant la valeur qui permet de remplir simultanement toutes les bandes 
!! pour tous les points k avec Nel * nkpt                                         
!! 
!! calcul de l'occupation des niveaux a partir                                       
!! du niveau de Fermi definitif                                                        
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! Fermi
!!
!! CHILDREN
!! Emax_occ_FD,max_tab,min_tab, FermiDirac
!!
!! SOURCE
!!
   subroutine EFermi_occ_FD(nkpt,nbnd,Ener,&
&            Nel,nsppol,kBT,EFermi,&
&            tab_Emax,wtk_tot,wtk)

  use verbose   
  use ggwdef
  use dpdef

  implicit none

  integer nbnd, Nel
  integer nkpt, ikpt, nsppol
  real*8 Ener(nbnd,nkpt),EFermi, Nelc 
  real*8 kBT, EFmin, EFmax,tolN
  integer itN,itNmax
  real*8 EFc, pasEFermi,  wtk_tot
  parameter (itNmax = 8,tolN = 1.0d-7) 
  
  integer ibnd
  
  real*8 Nelc_E,Nelb
  real*8 wtk(nkpt) 
  real*8, external ::  FermiDirac
  
  real*8 Emax_k, tab_Emax(nkpt)


  if(iverbose > 2) call uverbose('b','EFermi_occ_FD')

  !
  ! On calcule pour chaque point k la valeur de l'energie qui permet
  ! de remplir l'etat avec Nel electron
  do ikpt =1, nkpt
     call Emax_occ_FD(Ener(:,ikpt),nbnd,Nel,nsppol,&
          &                  kBT,Emax_k)
     tab_Emax(ikpt)=Emax_k
  enddo
  ! On cherche le niveau de Fermi  entre la plus petite et la plus grande 
  ! des energies qui conduisent au remplissage a Nel
  ! du point ikpt
  call max_tab(tab_Emax,nkpt,EFmax)
  call min_tab(tab_Emax,nkpt,EFmin)
  !
    if (iverbose > 2)  &
&       write(uout,'(" EFmin,EFmax  [eV]= ",2f8.4)') EFmin*HARTREE,EFmax*HARTREE 

  itN = 0
  pasEFermi = 0.1d0
  do while (pasEFermi.ge.(EFmax-EFmin))
     pasEFermi = pasEFErmi/10.0d0
     itN = itN + 1
  enddo

  !  EFc niveau de Fermi courant
  EFc = EFmin - pasEFermi
  do while (EFc.le.EFmax) 
     EFc = EFc+ pasEFermi 
     
     Nelc=0.0d0
     do ikpt = 1, nkpt
        Nelb = 0.0d0
        do ibnd =1,nbnd
           Nelc_E = FermiDirac(Ener(ibnd,ikpt),nsppol,EFc,kBT)
           Nelb = Nelb + Nelc_E
        enddo
        Nelc = Nelc + Nelb * wtk(ikpt)
     enddo
     if ((Nel*wtk_tot-Nelc).le.tolN) then
        if(itN.lt.itNmax) then
           itN = itN + 1
           EFc = EFc - pasEFermi
           pasEFermi = pasEFermi / 10.0d0
           EFc = EFc - pasEFermi
        else
           if ((Nelc-(Nel*wtk_tot)).gt.1.0d0) then
              write(uout,*)'WARNING : EFermi not found in subroutine EFermi_occ_FD'
              write(uout,*)'Nel too large, Nel=', Nelc
           else
              EFermi = EFc
              write(uout,fmt='(" Number of electrons = ",f11.4)') Nelc/wtk_tot
              if (iverbose >2 )  write(uout,fmt='(" Nelc x wtk_tot = ",f11.6)') Nelc    
           endif
           exit 
        end if
     end if
  enddo
  if(abs(EFc-EFmax).le.1.0d-3) then
    write(uout,*)'WARNING : EFermi not found in subroutine EFermi_occ_FD'
    write(uout,*)'EFmax too small, Efmax=', EFmax
  end if

  if(iverbose > 2) call uverbose('e','EFermi_occ_FD')

  end subroutine EFermi_occ_FD

!!***

!{\src2tex{textfont=tt}}
!!****p* EFermi_occ_FD/Emax_occ_FD
!! NAME
!! 
!!
!! FUNCTION
!!
!! Calcul pour les energies d'un point k l'energie qui remplit les etats
!! a T donne (Equivalent de EFermi si un seul point k)
!! 1) On cherche le niveau de Fermi
!! 1a) On fixe le niveau de Fermi EFermi a l'energie de la derniere bande occupee
!! 1b) On augmente le niveau de Fermi EFc de EFermi +/- 5 kBT
!!     jusqu'a ce que le nombre total d'electrons occupant ces niveaux pour EFc
!!     soit egal au nombre d'electrons total
!! 
!! calcul de l'occupation des niveaux a partir
!! du niveau de Fermi definitif 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! EFermi_occ_FD
!!
!! CHILDREN
!! EFermiguess , FermiDirac
!!
!! SOURCE

 subroutine Emax_occ_FD(Ener,nbnd,Nel,nsppol,kBT,&
      &                 EFermi)

   use verbose
   use ggwdef 
   use dpdef

   implicit none
   integer nbnd, Nel, nsppol
   real*8 Ener(nbnd),EFermi, Nelc,tolN 
   real*8 kBT, DeltakBT, EFmin, EFmax
   integer itN,itNmax
   real*8 EFc, pasEFermi 
   parameter (itNmax = 8,tolN = 1.0d-8) 
   
   integer ibnd
   
   real*8 Nelc_E
   real*8, external :: FermiDirac


   if(iverbose > 2) call uverbose('b','Emax_occ_FD')
  
 
   DeltakBT = 5.0d0
   !
   !  Definition du niveau de Fermi initial  a la valeur de la derniere bande occupee
   !

   call EFermiguess(Nel,nsppol,nbnd,Ener,EFermi)
   if (iverbose > 2)  &
&       write(uout,'(" EFermi guess value [eV]= ",f8.4)') EFermi*HARTREE
   EFmin = EFermi - DeltakBT * kBT
   EFmax = EFermi + DeltakBT * kBT
   if (EFmin.lt.Ener(1)) EFmin = Ener(1)
   if (EFmax.gt.Ener(nbnd)) EFmax = Ener(nbnd)
   
   pasEFermi = 0.1d0
   itN = 0
   do while (pasEFermi.ge.(EFmax-EFmin))
     pasEFermi=pasEFErmi/10.0d0
     itN=itN+1
   enddo

   !  EFc niveau de Fermi courant
   EFc = EFmin - pasEFermi
   loop: do while (EFc.le.EFmax) 
      EFc = EFc+ pasEFermi 
      
      Nelc=0.0d0
      do ibnd =1,nbnd
!         write(uout,*) 'Emax_occ :ibnd',ibnd,Ener(ibnd),EFc,kBT
         Nelc_E = FermiDirac(Ener(ibnd),nsppol,EFc,kBT)
!         write(uout,*) 'Emax_occ Nelc_E',Nelc_E
         Nelc = Nelc + Nelc_E
!         write(uout,*) 'Emax_occ Nelc',Nelc
      enddo
      if (((Nel*1.0d0)-Nelc).le.tolN) then
         if(itN.lt.itNmax) then
            itN = itN + 1
            EFc = EFc - pasEFermi
            pasEFermi = pasEFermi / 10.0d0
            EFc = EFc - pasEFermi
         else
            if ((Nelc-(Nel*1.0d0)).gt.0.4d0) then
               write(uout,*)'WARNING Emax_k not found in subroutine Emax_occ_FD'
               write(uout,*)'Nel for k too large : Nel = ',Nelc
               exit loop
            else
               EFermi = EFc
               !           write(uout,fmt='("Emax_k final = ",F10.6," eV")') EFc
               !           write(uout,fmt='("Nelectron = ",F10.6)') Nelc
               !           write(uout,fmt='("Emax_k final = ",f11.7," Ha  &
               !   &               Nelectron = ",f10.6)') EFc,Nelc
            endif
            exit loop 
         end if
      else
         if(EFc.ge.EFmax) then
            !         write(uout,*)'WARNING : DeltakBT trop petit '
            !         write(uout,'("Nelectron = ",f8.4)') Nelc
            !         write(uout,*)' Nelectron = ', Nelc
            DeltakBT = 2 * DeltakBT
            !         write(uout,'("new DeltakBT =",f8.2)')DeltakBT
            EFmin = EFermi - DeltakBT * kBT
            EFmax = EFermi + DeltakBT * kBT
            if (EFmin.lt.Ener(1)) EFmin = Ener(1)
            if (EFmax.gt.Ener(nbnd)) EFmax = Ener(nbnd)
            EFc = EFmin - pasEFermi
         end if
      end if
   enddo loop
   if(EFc.gt.EFmax) then
      write(uout,*)'WARNING : Emax_k not found in subroutine Emax_occ_FD'
      write(uout,*)'EFmax too small, Efmax=', EFmax
   end if

   if(iverbose > 2) call uverbose('e','Emax_occ_FD')

   end subroutine Emax_occ_FD

!!***

!{\src2tex{textfont=tt}}
!!****p* Fermi/EFermi_occ_T0
!! NAME
!! Fermi level and occupation calculation at T = 0
!!
!! FUNCTION
!!
!! Calcul pour les energies de tous les points k l'energie qui remplit les etats   
!! a T donne (Equivalent de EFermi si un seul point k)
!! 1)  On cherche le niveau de Fermi                                                
!! 1a) On cherche l'energie qui permet de remplir pour chaque point k              
!! toutes les bandes avec le nombre d'electrons Nel par point k : Emax_k        
!! 1b) On cherche la valeur minimale et maximale de Emax_k                          
!! 1c) On cherche le niveau de Fermi entre ces deux valeurs minimale et maximale     
!! comme etant la valeur qui permet de remplir simultanement toutes les bandes 
!! pour tous les points k avec Nel * nkpt                                         
!! 
!! calcul de l'occupation des niveaux a partir                                       
!! du niveau de Fermi definitif                                                        
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! Fermi
!!
!! CHILDREN
!! Emax_occ_T0,max_tab,min_tab,occ_nb_tot_EF 
!!
!! SOURCE
!!

 subroutine EFermi_occ_T0(nkpt,nbnd,Ener,&
     &            Nel,nsppol,degg,EFermi,&
     &            tab_Emax,wtk_tot,wtk)

  use verbose
  use ggwdef
  use dpdef

  implicit none
  integer nbnd, Nel, nsppol
  integer nkpt, ikpt
  integer degg(nbnd,nkpt)
  real*8 Ener(nbnd,nkpt),EFermi, Nelc,tolN 
  real*8 EFmin, EFmax
  integer itN,itNmax
  real*8 EFc, pasEFermi,  wtk_tot
  parameter (itNmax = 8,tolN=1.0d-7) 

  real*8 wtk(nkpt) 
  
  real*8 Emax_k, tab_Emax(nkpt)

  if(iverbose > 2) call uverbose('b','EFermi_occ_T0')

  !
  ! On calcule pour chaque point k la valeur de l'energie qui permet
  ! de remplir l'etat avec Nel electron
  do ikpt =1, nkpt
     call Emax_occ_T0(Ener(:,ikpt),nbnd,Nel,nsppol,degg(:,ikpt),Emax_k)
     tab_Emax(ikpt)=Emax_k
  enddo
  ! On cherche le niveau de Fermi  entre la plus petite et la plus grande 
  ! des energies qui conduisent au remplissage a Nel
  ! du point ikpt
  call max_tab(tab_Emax,nkpt,EFmax)
  call min_tab(tab_Emax,nkpt,EFmin)
  if (iverbose > 2)  &
&       write(uout,'(" EFmin,EFmax  [eV]= ",2f8.4)') EFmin*HARTREE,EFmax*HARTREE 


  
  if (EFmax==EFmin) then  ! this excludes the case 'only 1 k-point' 
  
! matteo: it is OK, but with 1 k point one would rather define Efermi = (lumo+homo) / 2
! here it is Efermi ~ homo

   EFermi = EFmax
  
  else 
   
   itN = 0
   pasEFermi = 0.1d0
   do while (pasEFermi.ge.(EFmax-EFmin))
      pasEFermi = pasEFErmi/10.0d0
      itN = itN + 1
   enddo
   
   !  EFc niveau de Fermi courant
   EFc = EFmin - pasEFermi
   do while (EFc.le.EFmax) 
      EFc = EFc+ pasEFermi 
      !
      Nelc=0.0d0
      call occ_nb_tot_EF(Nel,wtk_tot,EFc,nsppol,Ener,degg,&
           &                wtk,nkpt,nbnd,Nelc)
      if ((Nel*wtk_tot-Nelc).le.tolN) then
         !     if (Nelc.ge.(Nel*wtk_tot)) then
         if(itN.lt.itNmax) then
            itN = itN + 1
            EFc = EFc - pasEFermi
            pasEFermi = pasEFermi / 10.0d0
            EFc = EFc - pasEFermi
         else
            if (abs(Nelc-(Nel*wtk_tot)).gt.0.4d0) then
               write(uout,*)'WARNING : EFermi not found in subroutine EFermi_occ_T0'
               write(uout,*)'WARNING : EFc =', EFc, 'Ha'
               EFermi = EFc
               write(uout,*)'Nel too large'
               exit
            else
               EFermi = EFc
               write (uout, fmt='(" Number of  electrons", f11.6)') Nelc / wtk_tot 
               if (iverbose >2 )  write(uout,fmt='(" Nelc x wtk_tot = ",f11.6)') Nelc          
             endif
            exit 
         end if
      end if
   enddo
   if(abs(EFc-EFmax).le.1.0d-3) then
      write(uout,*)'WARNING : EFermi not found in subroutine EFermi_occ_T0'
      write(uout,*)'EFmax too small, Efmax=', EFmax
   end if

  endif ! if 1 k-point
  
  if(iverbose > 2) call uverbose('e','EFermi_occ_T0')


  end subroutine EFermi_occ_T0

!!***

!{\src2tex{textfont=tt}}
!!****p* EFermi_occ_T0/Emax_occ_T0
!! NAME
!! Emax_occ_T0
!!
!! FUNCTION
!!
!! Calcul pour les energies d'un point k l'energie qui remplit les etats
!! a T donne (Equivalent de EFermi si un seul point k)
!! 1) On cherche le niveau de Fermi
!! 1a) On fixe le niveau de Fermi EFermi a l'energie de la derniere bande occupee
!! 1b) On augmente le niveau de Fermi EFc de EFermi +/- 5 kBT
!!     jusqu'a ce que le nombre total d'electrons occupant ces niveaux pour EFc
!!     soit egal au nombre d'electrons total
!! 
!! calcul de l'occupation des niveaux a partir
!! du niveau de Fermi definitif 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! EFermi_occ_T0
!!
!! CHILDREN
!! EFermiguess,occ_nb 
!!
!! SOURCE

 subroutine Emax_occ_T0(Ener,nbnd,Nel,nsppol,deg,EFermi)

  use verbose   
  use ggwdef
  use dpdef

  implicit none
  integer nbnd, Nel, nsppol
  real*8 Ener(nbnd),EFermi, Nelc,tolN 
  real*8 Delta,EFmin, EFmax,occ_k(nbnd)
  integer itN,itNmax
  real*8 EFc, pasEFermi 
  parameter (itNmax = 8,tolN = 1.0d-8) 
  
  
  integer ibnd,deg(nbnd)
  
  if(iverbose > 2) call uverbose('b','Emax_occ_T0')


  Delta = 0.5d0
  !
  !  Definition du niveau de Fermi initial  a la valeur de la derniere bande occupee
  !

  call EFermiguess(Nel,nsppol,nbnd,Ener,EFermi)
   if (iverbose > 2)  &
&       write(uout,'(" EFermi guess value [eV]= ",f8.4)') EFermi*HARTREE
  EFmin = EFermi - abs(EFermi) * Delta 
  EFmax = EFermi + abs(EFermi) * Delta
  if (EFmin.lt.Ener(1)) EFmin = Ener(1)
  if (EFmax.gt.Ener(nbnd)) EFmax = Ener(nbnd)
  
  itN = 0
  pasEFermi = 0.1d0
  do while (pasEFermi.ge.(EFmax-EFmin))
     pasEFermi = pasEFErmi/10.0d0
     itN = itN + 1
  enddo

  !  EFc niveau de Fermi courant
  EFc = EFmin - pasEFermi

  do while (EFc.le.EFmax) 
     EFc = EFc+ pasEFermi 
     Nelc=0.0d0
     occ_k(:) = 0.0d0
     call occ_nb(Nel,EFc,nsppol,Ener,deg,nbnd,occ_k)
     do ibnd =1,nbnd
        Nelc = Nelc + occ_k(ibnd)
     enddo
     if (((Nel*1.0d0)-Nelc).le.tolN) then
        if(itN.lt.itNmax) then
           itN = itN + 1
           EFc = EFc - pasEFermi
           pasEFermi = pasEFermi / 10.0d0
           EFc = EFc - pasEFermi
        else
           if ((Nelc-(Nel*1.0d0)).gt.0.4d0) then
              write(uout,*)'WARNING : Emax_k not found in subroutine Emax_occ_T0'
              write(uout,*)'N. electrons for k too large : Nel = ',Nelc
              exit
           else
              EFermi = EFc
               !          write(uout,fmt='("Emax_k final = ",f11.7," Ha  &
               !  &               Nelectron = ",f10.6)') EFc,Nelc
           endif
           exit 
        end if
     else
        if(EFc.ge.EFmax) then
           !         write(uout,*)'WARNING : DeltakBT trop petit '
           !         write(uout,'("Nelectron = ",f8.4)') Nelc
           !         write(uout,*)' Nelectron = ', Nelc
           Delta = 2 * Delta
           !         write(uout,'("new DeltakBT =",f8.2)')DeltakBT
           EFmin = EFermi - EFermi * Delta
           EFmax = EFermi + EFermi * Delta
           if (EFmin.lt.Ener(1)) EFmin = Ener(1)
           if (EFmax.gt.Ener(nbnd)) EFmax = Ener(nbnd)
           EFc = EFmin - pasEFermi
        end if
     end if
  enddo
  if(EFc.gt.EFmax) then
     write(uout,*)'WARNING : Emax_k not found in subroutine Emax_occ_T0'
     write(uout,*)'EFmax too small'
  end if

  if(iverbose > 2) call uverbose('e','Emax_occ_T0')

  end subroutine Emax_occ_T0

!!***



!{\src2tex{textfont=tt}}
!!****p* Fermi/max_tab
!! NAME
!! max_tab
!!
!! FUNCTION
!! Calculate the maximum value in a table 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! EFermi_occ_FD,EFermi_occ_T0
!!
!!
!! TODO 
!! A library function should be used for this role
!!
!! SOURCE

subroutine max_tab(tab,npt,max)

  implicit none
  integer i, npt
  real*8 tab(npt),max
  
  max = tab(1)
  do i = 1 ,npt
     if(max.lt.tab(i)) max = tab(i)
  enddo
end subroutine max_tab
!!***

!{\src2tex{textfont=tt}}
!!****p* Fermi/min_tab
!! NAME
!! min_tab
!!
!! FUNCTION
!! Calculate the minimum value in a table 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! EFermi_occ_FD,EFermi_occ_T0
!!
!!
!! TODO 
!! A library function should be used for this role
!!
!! SOURCE

subroutine min_tab(tab,npt,min)

  implicit none
  integer i,npt
  real*8 tab(npt),min
  
  min = tab(1)
  do i = 1 ,npt
     if(min.gt.tab(i)) min = tab(i)
  enddo
end subroutine min_tab
!!***

!{\src2tex{textfont=tt}}
!!****p* Fermi/FermiDirac
!! NAME
!! FermiDirac
!!
!! FUNCTION
!! Calculate the Fermi-Dirac distribution
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! Fermi,EFermi_occ_FD,Emax_occ_FD
!!
!! SOURCE

double precision function FermiDirac(Eigen,nsppol,EFermi,kBT)

  implicit none
  real*8 Eigen,EFermi,kBT
  real*8 E
  integer nsppol
  double precision elelevel
  if (nsppol==1) elelevel=2.
  if (nsppol==2) elelevel=1.
  
  E = (Eigen-EFermi)/kBT
!  write(uout,*) 'FermiDirac:Eigen,EFermi,kBT,E',Eigen,EFermi,kBT,E
  if (abs(E).gt.100.0d0) then
     if (E.gt.100.0d0) FermiDirac = 0.0d0
     if (E.lt.(-100.0d0)) FermiDirac = elelevel
  else
     FermiDirac = elelevel/(1.0d0 + exp(E))    ! fonction de Fermi Dirac normee a nsppol
  endif

!  write(uout,*) 'FermiDirac',E
!  write(uout,*) 'FermiDirac',FermiDirac
end function FermiDirac
!!***


!{\src2tex{textfont=tt}}
!!****p* Emax_occ_T0/occ_nb
!! NAME
!! occ_nb
!!
!! FUNCTION
!! Fill the energy levels with 0. and nsppol. 
!! Fractional numbers are used if degeneracies occur.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! Emax_occ_T0
!!
!! SOURCE

subroutine occ_nb(Nel,EFermi,nsppol,ener,deg,nbnd,occ)

  implicit none
  integer nbnd, Nel, nsppol
  integer ib, ibb,deg(nbnd)
  real*8 ener(nbnd),EFermi,occ(nbnd), Nelc, tolE
  parameter(tolE = 1.0d-9)
  double precision elelevel
  if (nsppol==1) elelevel=2.
  if (nsppol==2) elelevel=1.
  
  occ(:) = 0.0d0
  Nelc = 0.0d0
  ib = 1

  do while (ener(ib).le.EFermi)
     if ((EFermi-ener(ib)).gt.tolE) then
        do ibb = ib, ib + deg(ib) - 1
           occ(ibb) = elelevel
           Nelc = Nelc + occ(ibb)
        enddo
     else
        do ibb = ib, ib + deg(ib) - 1
           occ(ibb) = (Nel*1.0d0-Nelc)/deg(ib)
           if (occ(ibb).gt.elelevel) occ(ibb)=elelevel
        enddo
        do ibb = ib, ib + deg(ib) - 1
           Nelc = Nelc + occ(ibb)
        enddo
     endif
     ib = ib + deg(ib)
  enddo
end subroutine occ_nb
!!***



!{\src2tex{textfont=tt}}
!!****p* Fermi/degenerescence
!! NAME
!! degenerescence
!!
!! FUNCTION
!! Calculation of the degeneracy of the energies ener(nbnd)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! Fermi
!!
!! SOURCE

subroutine degenerescence(tab,nbnd,deg)

  implicit none
  integer ib,ibb,ibbb,nbnd,cnt
  integer deg(nbnd)
  real*8  tab(nbnd),test, tolE
  
  tolE = 1.0d-9
  deg(:) = 1
  ib = 1
  do while (ib.lt.nbnd) 
     test = tab(ib)
     cnt = 1
     ibb = ib+1
     do while (ibb.le.nbnd)
        if (abs(tab(ibb)-test).le.tolE) then
           cnt = cnt + 1
           ibb = ibb + 1
        else
           ibb = nbnd+1
        endif
     enddo
     deg(ib) = cnt
     if (cnt.gt.1) then
        do ibbb = ib+1,ib+cnt-1 
           deg(ibbb)=0
        enddo
     endif
     ib = ib + cnt
  enddo
end subroutine degenerescence

!!***


!{\src2tex{textfont=tt}}
!!****p* Fermi/occ_nb_tot_r
!!
!! NAME
!! occ_nb_tot_r
!!
!! FUNCTION
!! Cette routine remplit les niveaux d'energie avec 0. et 2.
!! et occupation fractionnaire au niveau de Fermi si degenere
!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! Fermi
!!
!!
!! SOURCE

subroutine occ_nb_tot_r(Nel,wtk_tot,EFermi,nsppol,ener,degg,wtk,nkpt,nbnd,occ)
   
  implicit none
  integer nkpt,nbnd, Nel,ik, nsppol
  integer ib, ibb,degg(nbnd,nkpt),Indege(nkpt),degeEF
  real*8 ener(nbnd,nkpt),EFermi,occ(nbnd,nkpt), Nelc, tolE
  real*8 wtk(nkpt),wtk_tot
  parameter(tolE = 1.0d-9)
  double precision elelevel
  if (nsppol==1) elelevel=2.
  if (nsppol==2) elelevel=1.
  
  occ(:,:) = 0.0d0
  Nelc = 0.0d0
  Indege(:) = 0
  degeEF = 0
  do ik = 1,nkpt
     ib = 1
     do while (ener(ib,ik).le.EFermi)
        if ((EFermi-ener(ib,ik)).gt.tolE) then
           do ibb = ib, ib + degg(ib,ik) - 1
              occ(ibb,ik) = elelevel
              Nelc = Nelc + occ(ibb,ik)*wtk(ik)
           enddo
        else
           Indege(ik) = ib 
           degeEF = degeEF + degg(ib,ik)
        endif
        ib = ib + degg(ib,ik)
     enddo
  enddo
  do ik = 1,nkpt
     if (Indege(ik).ne.0) then
        do ibb = Indege(ik), Indege(ik) + degg(Indege(ik),ik) - 1
           occ(ibb,ik) = (Nel*wtk_tot-Nelc)/(degeEF*1.0d0)/wtk(ik)
           if (occ(ibb,ik).gt.elelevel) occ(ibb,ik)=elelevel
        enddo
     endif
  enddo
  do ik = 1,nkpt
     if (Indege(ik).ne.0) then
        do ibb = Indege(ik), Indege(ik) + degg(Indege(ik),ik) - 1
           Nelc=Nelc+occ(ibb,ik)*wtk(ik)
        enddo
     endif
  enddo
end subroutine occ_nb_tot_r

!!***


!{\src2tex{textfont=tt}}
!!****p* EFermi_occ_T0/occ_nb_tot_EF
!! NAME
!! occ_nb_tot_EF
!!
!! FUNCTION
!! Fill the energy levels with 0. and 2.
!! In case of degeneray at the Fermi level, fractional occupation are used
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! EFermi_occ_T0
!!
!!
!! SOURCE

subroutine occ_nb_tot_EF(Nel,wtk_tot,EFermi,nsppol,ener,degg,wtk,nkpt,nbnd,Nelc)

  implicit none
  integer nkpt,nbnd, Nel,ik, nsppol
  integer ib, ibb,degg(nbnd,nkpt),Indege(nkpt),degeEF
  real*8 ener(nbnd,nkpt),EFermi,occ(nbnd,nkpt), Nelc, tolE
  real*8 wtk(nkpt),wtk_tot
  parameter(tolE = 1.0d-9)
  double precision elelevel
  if (nsppol==1) elelevel=2.
  if (nsppol==2) elelevel=1.
  
  occ(:,:) = 0.0d0
  Nelc = 0.0d0
  Indege(:) = 0
  degeEF = 0
  do ik = 1,nkpt
     ib = 1
     do while (ener(ib,ik).le.EFermi)
        if ((EFermi-ener(ib,ik)).gt.tolE) then
           do ibb = ib, ib + degg(ib,ik) - 1
              occ(ibb,ik) = elelevel
              Nelc = Nelc + occ(ibb,ik)*wtk(ik)
           enddo
        else
           Indege(ik) = ib 
           degeEF = degeEF + degg(ib,ik)
        endif
        ib = ib + degg(ib,ik)
     enddo
  enddo
  !   write(uout,*) Nelc
  do ik = 1,nkpt
     if (Indege(ik).ne.0) then
        do ibb = Indege(ik), Indege(ik) + degg(Indege(ik),ik) - 1
           occ(ibb,ik) = (Nel*wtk_tot-Nelc)/(degeEF*1.0d0)/wtk(ik)
           if (occ(ibb,ik).gt.elelevel) occ(ibb,ik)=elelevel
        enddo
     endif
  enddo
  do ik = 1,nkpt
     if (Indege(ik).ne.0) then
        do ibb = Indege(ik), Indege(ik) + degg(Indege(ik),ik) - 1
           Nelc=Nelc+occ(ibb,ik)*wtk(ik)
        enddo
     endif
  enddo
end subroutine occ_nb_tot_EF
!!***

!{\src2tex{textfont=tt}}
!!****p* Fermi/unify_spin
!! NAME
!! unify_spin
!!
!! FUNCTION
!! Put together the two spin channels, in order to find the Fermi level also
!! taking into account the spin degree of freedom
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! Fermi
!!
!! SOURCE

subroutine unify_spin(nsppol,nbnds,nkbz,ibntab,eneinput,eneoutput)
 use ggwdef
 implicit none
 
 integer nsppol, nbnds, nkbz, ikibz, iup, idown, i
 double precision eneup,enedown
 double precision eneinput(nkbz,nbnds,nsppol), eneoutput(nkbz,nbnds*nsppol)
 type(tabellaspin) ibntab(nkbz,nsppol*nbnds)

 if (nsppol == 1 ) then
  eneoutput(:,:)=eneinput(:,:,1)
  do i=1,nbnds
   ibntab(:,i)%ibn=i
   ibntab(:,i)%is=1
  enddo
 else 
  do ikibz=1,nkbz
   iup=1
   idown=1
   eneup=eneinput(ikibz,iup,1)
   enedown=eneinput(ikibz,idown,2)
   do i=1,nsppol*nbnds
     if(eneup<=enedown)then
      eneoutput(ikibz,i)=eneup
      ibntab(ikibz,i)%ibn=iup
      ibntab(ikibz,i)%is=1
      if(iup<nbnds) then
       iup = iup+1
       eneup=eneinput(ikibz,iup,1)
      else
       eneup=999999999999999.
      endif
     else
      eneoutput(ikibz,i)=enedown
      ibntab(ikibz,i)%ibn=idown
      ibntab(ikibz,i)%is=2
      if(idown<nbnds) then
       idown = idown+1
       enedown=eneinput(ikibz,idown,2)
      else
       enedown=999999999999999.
      endif
     endif
    enddo
   enddo
 endif 


end subroutine unify_spin

!!***



!{\src2tex{textfont=tt}}
!!****p* Fermi/EFermiguess
!! NAME
!! EFermiguess
!!
!! FUNCTION
!! First guess of the Fermi level. Useful for further refinement (metals,
!! temperature, degeneracies)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! Emax_occ_T0,Emax_occ_FD
!!
!! SOURCE
subroutine EFermiguess(Nel,nsppol,nbnd,Ener,EFermi)
implicit none
integer Nel, Nels2, nbnd, nsppol
double precision EFermi
double precision, dimension(nbnd) :: Ener
logical PAIR

if (nsppol==1) then
  PAIR= .FALSE.
  Nels2 = int(Nel/2)
  if ((Nel-2*Nels2).eq.0) PAIR = .TRUE.
 
  if (PAIR) then
     EFermi = Ener(Nels2)
  else 
     EFermi = Ener(Nels2+1)
  endif
else
  EFermi = Ener(Nel)
endif

end subroutine EFermiguess
!!***
   
