!{\src2tex{textfont=tt}}
!!****h* Mod/ggwdef
!! NAME
!! ggwdef
!!
!! FUNCTION
!! ggwdef module contains all the types related to dp (few exceptions)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!***
module ggwdef
use params
implicit none

!{\src2tex{textfont=tt}}
!!****t* ggwdef/crystal
!! NAME
!! crystal
!!
!! FUNCTION
!! crystal structure (cell parameter, cell volume, angles, primitive vectors, etc.)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
type crystal
  integer :: spgroup, pgroup                 ! space, point group
  integer :: bravaislatt, crystsys           ! bravais lattice, crystal system
  double precision :: rprim(3,3)             ! dimensionless
  double precision :: gprim(3,3)             ! dimensionless
  double precision :: acell(3)               ! cell dimension in Bohr
  double precision :: angdeg(3)              ! angle degrees among rprim
  double precision :: rprimd(3,3)            ! in Bohr
  double precision :: gprimd(3,3)            ! a.u. dimensioned primitives
  double precision :: ucvol, bzvol           ! unit cell, brillouin zone volume
  double precision :: gprimdm1(3,3)          ! transf from cc to rl
end type
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/symmetry_operation
!! NAME
!! symmetry_operation
!!
!! FUNCTION
!! symmetry_operation structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
type symmetry_operation                      ! single symmetry operation
  integer :: r(3,3)                          ! rotation
  double precision :: t(3)                   ! translation
  integer :: tr                              ! time reversal (-1 = time rev)
end type
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/symmetry
!! NAME
!! symmetry
!!
!! FUNCTION
!! symmetry
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
type symmetry                                ! symmetry structure of the system
  integer :: nsym                            ! maximum symmetries 96
  type(symmetry_operation) :: sym(96)        ! direct symmetries
  type(symmetry_operation) :: isym(96)       ! inverse symmetries
  type(symmetry_operation) :: itsym(96)      ! inverse transpose symmetries for k-points
  real, pointer :: op(:,:,:)
  integer, pointer :: symafm(:)
end type
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/pseudoatom
!! NAME
!! pseudoatom
!!
!! FUNCTION
!! pseudoatom structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
type pseudoatom                              ! single pseudoatom
  double precision :: znucl                  ! atom Z
  double precision :: zion                   ! pseudoatom Z
  integer :: lmax                            ! maximum angular momentum compon.
  integer :: lloc                            ! local angular momentum compon.
  integer :: mmax                            ! mesh dimension
  integer :: pspso,pspdat,pspcod,pspxc       ! spin orbit ....
  character :: title(132)
end type
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/chemistry
!! NAME
!! chemistry
!!
!! FUNCTION
!! chemistry structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
type chemistry                               ! chemistry structure of the system
  integer :: ntype                           ! number of atom types
  double precision, pointer :: znucl(:)      ! atoms Z (redundant)
  integer :: natom                           ! number of atoms
  integer, pointer :: typeat(:)              ! type of the atoms
  double precision, pointer :: xred(:,:)     ! atom reduced coordinates
  double precision, pointer :: xcart(:,:)    ! atom cartesian coordinates
  integer :: npsa                            ! number of pseudoatoms ( = ntype)
  type(pseudoatom), pointer :: psa(:)        ! pseudoatoms
  integer :: nel                             ! number of electrons in the unit cell
end type
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/mesh
!! NAME
!! mesh
!!
!! FUNCTION
!! mesh structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
type mesh
  integer ng                                 ! number of G's (of planewaves)
  integer nr                                 ! number of r's (real space mesh)
  integer ngg(3)                             ! reciprocal space mesh
  integer nrr(3)                             ! real space mesh
  integer, pointer :: igfft(:,:,:,:)         ! indexing for nr -> npwmat operation 
  integer(ILONG) :: plan, menoplan
end type
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/rot_table
!! NAME
!! rot_table
!!
!! FUNCTION
!! rot_table structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
type rot_table
  integer, pointer :: irottb(:,:)            ! contains the index no. of r**-1 r [note inverse!]
  integer, pointer :: grottb(:,:,:)          ! contains the index no. of r i g
end type
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/transposed_space
!! NAME
!! transposed_space
!!
!! FUNCTION
!! transposed_space structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
type transposed_space                        ! transposed space structure
!  integer ng                                 ! number of G's (redundant)
!  integer nsh                                ! number of shells (to add: number of stars)
  integer nk                                 ! number of k-points in IBZ
  integer, pointer :: gvec(:,:)              ! G-vectors
!  integer, pointer :: sh(:)                  ! shells limits
  double precision, pointer :: k(:,:)        ! k-points in IBZ (may be better a pointer to ibzvector object)
end type
!!***

type ibzvector                               ! IBZ vector
  double precision :: rc(3)                  ! reduced coordinates
  double precision :: w                      ! weight
end type

type bzvector                                ! BZ vector
  double precision :: rc(3)                  ! reduced coordinates
  integer :: ibz                             ! corresponding IBZ-vector (better a pointer to ibzvector object)
  integer :: isym                            ! IBZ -> BZ originating symmetry (better a pointer to symmetry_operation object)
end type
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/bzstructure
!! NAME
!! bzstructure
!!
!! FUNCTION
!! bzstructure structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
type bzstructure                             ! BZ structure of the system
  integer nibz                               ! number of IBZ k-points
  integer nbz                                ! number of BZ k-points
  type(ibzvector), pointer :: ibz(:)         ! array of IBZ k-points
  type(bzvector), pointer :: bz(:)           ! array of BZ k-points
  real, pointer :: kbz(:,:)
  real, pointer :: kibz(:,:)
  real, pointer :: wtk(:)
  integer, pointer :: ktab(:), ktabr(:,:), ktabi(:),ktabo(:)
end type
!!***

!!****t* ggwdef/qstructure
!! NAME
!! qstructure
!!
!! FUNCTION
!! qstructure contains the qs in the BZ, IBZ and also the tabs to have one in
!! terms of the other
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
type qstructure
 integer nqbz
 real, pointer :: qibz(:,:) ! should be passed to type
 real, pointer :: qbz(:,:) ! should be passed to type
 integer, pointer :: qtab(:), qtabi(:), qtabo(:)
 real, pointer :: qpg(:,:)
 integer :: qdenom, iqbz_0
 integer, pointer :: kmkptab(:,:,:)
end type
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/polarizationq
!! NAME
!! polarizationq
!!
!! FUNCTION
!! polarizationq structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
type polarizationq
  integer, pointer :: ik(:)                 !  index of the vector k (BZ)    
  integer, pointer :: ikp(:)                !  index of the vector kmq = k - q (BZ)
  integer, pointer :: ikibz(:)              !  index of the vector k (IBZ)
  integer, pointer :: ikpibz(:)             !  index of the vector kmq = k - q (IBZ) 
  integer, pointer :: ig01a(:)              !  index of g0 for the fft mesh 
  integer, pointer :: ig02a(:)              !  index of g0 for the fft mesh 
  integer, pointer :: ig03a(:)              !  index of g0 for the fft mesh 
  integer, pointer :: iinv(:)               !  inversion for k
  integer, pointer :: iinv2(:)              !  inversion for k - q
end type
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/electronic_structure
!! NAME
!! electronic_structure
!!
!! FUNCTION
!! electronic_structure structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
type electronic_structure                         ! electronic structure of the system
  integer nb                                      ! number of bands
  integer nk                                      ! number of k-points (IBZ)
  integer ng                                      ! number of planewaves (of G's)
  integer nr                                      ! number of r-points
  integer :: wfdim
  integer :: nsppol                               ! number of spin polarizations
  real, pointer :: oc(:,:,:)                        ! occupations oc(n,k,spin) Bloch order
  real, pointer :: occ(:,:,:)                       ! occupations oc(n,k,spin) Bloch order
  double precision, pointer :: energy(:,:,:)                    ! energies en(n,k,spin) Bloch order
  double precision, pointer :: energytmp(:,:,:)                 ! energies en(n,k,spin) Bloch order
  double precision, pointer :: gwenergy(:,:,:)                  ! QP energies en(n,k,spin) Bloch order
  double precision, pointer :: imsigma(:,:,:)                   ! imaginary part of self energy - lifetime
  ! complex (kind(0.0d0)), pointer :: qps(:,:)      ! qps(n,k) QP shift
  integer, pointer :: wf_count(:,:,:)
  complex, pointer :: wfg(:,:,:,:)                  ! wavefunctions in rec. space wfg(g,n,k) order
  complex, pointer :: wfg2(:,:)                  ! wavefunctions in rec. space wfg(g,n,k) order
  complex, pointer :: wfr(:,:,:,:)                  ! wavefunctions in real space wfr(r,n,k,spin) order
  complex, pointer :: wfgqp(:,:,:,:)                ! quasiparticle wfns in rec. space (from file QPS)
  complex, pointer :: m_lda_to_qp(:,:,:,:)        ! matrix representing qp wfns on a kohn-sham basis
  double precision :: fermie                      ! Fermi energy
  double precision :: etotal, residm, ecut_eff    ! 
  integer :: xc                                   ! xc functional
end type
!!***


!{\src2tex{textfont=tt}}
!!****t* ggwdef/screening
!! NAME
!! screening
!!
!! FUNCTION
!! screening structure: it contains the dynamically screened coulomb interaction
!! for the exciton code
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! TODO 
!! implement the use of this object also for the mapping theory kernel
!!
!! SOURCE
type screening
  integer :: nqibz
  real,pointer :: qibz(:,:)
  complex,pointer :: epsm1q(:,:,:)
  complex,pointer :: Wibz(:,:,:)
  complex, pointer :: Wstar(:,:,:)
  complex, pointer :: Wstardiag(:,:)
  complex,pointer :: bigomegatwsq(:,:,:)
  complex,pointer :: omegatw(:,:,:)
  integer, pointer :: iq(:) !iq(iqr): iq point in IBz corresponding to the iqr point read from scr file  
  integer  nomega
  complex  omega(1)
end type
!!***


!!****t* ggwdef/kmkpq
!! NAME
!! kmkpq
!!
!! FUNCTION
!! k minus kp structure: it contains the information about the point q = k -kp
!! for the exciton code (it is the q point appearing in the coulomb term)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! TODO
!! implement the use of this object also for the mapping theory kernel
!!
!! SOURCE
type kmkpq
  integer, pointer :: ig01(:,:)
  integer, pointer :: ig02(:,:)
  integer, pointer :: ig03(:,:)
  integer, pointer :: iqkkp(:,:)
end type
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/kb_potential
!! NAME
!! kb_potential
!!
!! FUNCTION
!! kb_potential structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! TODO
!! remove the DRUDE variable
!!
!! SOURCE
type kb_potential
  integer :: ntype
  integer :: natom
  integer :: mpsang
  integer :: ng
  integer :: nk
  integer :: kdim
  integer, pointer :: k_count(:)
  real, pointer :: vkbsign(:,:)
  real, pointer :: vkb(:,:,:,:), vkbd(:,:,:,:)
  complex, pointer :: gradvnl(:,:,:,:)
  complex, pointer :: fnl(:,:,:,:), fnld(:,:,:,:,:)
  logical inclvkb
  logical exists
  logical DRUDE
end type        
!!***


!      type rlv
!        integer rlu(3)                          ! gvec
!        integer jgfft                           ! igfft(G0=0)
!        integer igfft(-3:3,-3:3,-3:3)           ! igfft
!        type(rlv), pointer :: grottb(:,:)       ! grottb(ninv,nop)
!      endtype

!      type ibzpoint
!        real rlu(3)                             ! kibz, qibz
!        real w                                  ! weight
!        real, pointer :: en(:)                  ! energy
!      endtype
      
!      type bzpoint
!        real rlu(3)                             ! kbz, qbz
!        type(ibzpoint), pointer :: ibz          ! ktab, qtab
!        type(symmop), pointer :: o              ! ktabo, qtabo
!        logical :: i                            ! ktabi, qtabi
!      endtype

!      type dlv
!        integer, pointer :: irottb(:)           ! irottb(nop)
!      endtype
!     ktabr(nr,nkbz) -> ktabr(:,kbz) = irottb(:,iop)
      
!{\src2tex{textfont=tt}}
!!****t* ggwdef/ldakernel
!! NAME
!! ldakernel
!!
!! FUNCTION
!! ldakernel structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
   type ldakernel
     real, pointer :: rho2(:,:)              !  density (2 spin components)
     complex, pointer :: fxclda(:,:,:,:)     !  lda kernel in reciprocal space
                                             !  (nspin X nspin  matrix)
     real, pointer :: fxcldar(:,:,:)         !  lda kernel in real space
                                             !  (nspin X nspin matrix)
   endtype
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/mappingkernel
!! NAME
!! mappingkernel
!!
!! FUNCTION
!! mappingkernel structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
   type mappingkernel
      real (kind(0.0d0)) :: Wd=0
      complex (kind(0.0d0)), pointer :: hd(:,:)
      complex (kind(0.0d0)), pointer :: hdc(:,:)
      complex (kind(0.0d0)), pointer :: Tkernel(:,:,:)
   endtype
!!***
   
!{\src2tex{textfont=tt}}
!!****t* ggwdef/coulomb
!! NAME
!! coulomb
!!
!! FUNCTION
!! coulomb structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
   type coulomb
     real, pointer :: qplusG(:,:)               !  q+G
     real, pointer :: Vc(:,:)                   ! 4pi/(q+G)^2
   endtype
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/lattstruct
!! NAME
!! lattstruct
!!
!! FUNCTION
!! lattstruct structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
      type lattstruct                           ! deprecated: use crystal
        integer lattype
        real acell(3)                           ! cell dimension in Bohr
        real rprim(3,3)                         ! dimensionless
        real r(3,3)                             ! in Bohr
        real a1(3), a2(3), a3(3)
        real b1(3), b2(3), b3(3)
        real ucvol, bzvol
        real brl2cc(3,3)
        real bcc2rl(3,3)
        real tpula
      endtype
!!***
      
!{\src2tex{textfont=tt}}
!!****t* ggwdef/transition
!! NAME
!! transition
!!
!! FUNCTION
!! transition structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
      type transition
!       temporary an integer but bzpoint in the future
        integer k
        integer v
        integer c
        integer s
        real (kind(0.0d0)) :: en
        real (kind(0.0d0)) :: gwen
        real factocc
      endtype
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/transgen
!! NAME
!! transgen
!!
!! FUNCTION
!! transgen structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
      type transgen
       complex, pointer :: rhotwgen(:,:)
       complex, pointer :: rhotwxqpol(:,:)
       complex, pointer :: aux(:,:)
       complex, pointer :: aux2(:,:)
       complex, pointer :: rhotwepsexc(:)
      endtype
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/response
!! NAME
!! response
!!
!! FUNCTION
!! response structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
      type response
        complex, pointer :: chi0(:,:,:,:)  !diagonal in spin space
        complex, pointer :: lwing(:,:,:,:) !diagonal in spin space
        complex, pointer :: uwing(:,:,:,:) !diagonal in spin space
        complex, pointer :: head(:,:,:,:) !diagonal in spin space
        complex, pointer :: chi0drude(:,:,:,:,:)
        complex, pointer :: epsmlf(:,:)    !resulting from trace over spin (so no spin dependence)
        complex, pointer :: epsmnlf(:,:)   !resulting from trace over spin (so no spin dependence)
        complex, pointer :: epsmnlfspin(:,:,:) !spin diagonal <= spin resolved  
        complex, pointer :: epsmlfspin(:,:,:) !spin diagonal <= spin resolved  
        complex, pointer :: chi0X(:,:,:,:) !SPIN TODO

        complex, pointer :: epsgwnlf(:,:)  !!SPI TODO
        complex, pointer :: epsrpanlf(:,:) !!SPI TODO

      endtype
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/momentum
!! NAME
!! momentum
!!
!! FUNCTION
!! momentum structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
      type momentum
        real qplusg(3)
        real qpplusg(3)
        real qplusqpplusg(3)
        real qcc(3)
        real qpcc(3)
        real q(3)
        integer gq(3)
        integer iqg
        real qp(3)
        integer gqp(3)
        integer iqpg
        real qplusqp(3)
        integer gqplusqp(3)
        integer iqplusqpg
      endtype
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/tabellaspin
!! NAME
!! tabellaspin
!!
!! FUNCTION
!! tabellaspin structure 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
      type tabellaspin
        integer ibn
        integer is
      end type
!!***

!{\src2tex{textfont=tt}}
!!****t* ggwdef/Wmatrixel
!! NAME
!! Wmatrixel
!!
!! FUNCTION
!! indeced for matrixelements of W 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
      type Wmatrixel
        integer ikibz1,ikibz2,ikibz3,ikibz4
        integer ikbz1,ikbz2,ikbz3,ikbz4
        integer iik1,iik2,iik3,iik4
        integer iq,right_ig01,right_ig02,right_ig03,left_ig01,left_ig02,left_ig03  
        integer ibini1,ibend1,ibini2,ibend2,ibini3,ibend3,ibini4,ibend4
       

      end type
!!***


!{\src2tex{textfont=tt}}
!!****t* ggwdef/wfnmemory
!! NAME
!! wfnmemory
!!
!! FUNCTION
!! Wave functions memory object
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
  type wfnmemory
   integer wfnmem
   integer, pointer :: wfnmemobj(:,:,:)
   integer, pointer :: wfntab(:,:,:)
  endtype
!!***

 type exceigen
  complex, pointer :: acvk(:,:,:,:)
  integer :: lomo, nkbz, nbndo, nbnds
  integer :: ntrans, nholes
  integer, pointer :: irhole(:)
  real, pointer :: poshole(:,:)
  integer :: nr
  integer :: nexciton
  integer, pointer :: nlambdadeg(:)
 end type


 type bond
  integer ::  first, second
 end type





end module

module paral

 integer ierr,rang,nprocs
 integer :: nprow, npcol, myprow, mypcol
 integer, external :: blacs_pnum, numroc
 integer :: contxt, icaller, nout,nzout
 integer, allocatable :: desca(:),descz(:)
 integer, allocatable :: index1(:)
 integer, allocatable :: index2(:)
 integer :: nb,mb,info, lda,ldb, hisrow, hiscol

end module 
