!{\src2tex{textfont=tt}}
!!****p* main/esslinterfaces
!!
!!***

!{\src2tex{textfont=tt}}
!!****p* esslinterfaces/chevc
!! NAME
!! chevc
!!
!! FUNCTION
!! Compute the eigenvalues and the eigenvectors of a complex Hermitian matrix
!! only the upper triangle of a is used

!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!
!! SOURCE

 subroutine chevc(n,a,lda,w,z,ldz)
 use dpdef
 integer n, lda, ldz
 complex a(lda,n)
 complex z(ldz,n)
 real w(n)
 
 integer iopt
!upper-packed storage mode and eigenvectors (iopt=21)
!lower-packed storage mode and eigenvectors (iopt=1)
 parameter(iopt=21)
 
 integer np
 complex, allocatable :: ap(:)
 integer naux
 real, allocatable :: aux(:)

 integer istat
 
 np = n*(n+1)/2
 allocate(ap(np),stat=istat)
 if(istat.ne.0) then
   write(uout,*) 'out of memory: memory-space requested', np,'sizeof(complex)'
   stop 'chevc 1: out of memory!'
 endif
 
 naux = 4*n
 allocate(aux(naux),stat=istat)
 if(istat.ne.0) stop 'chevc 2: out of memory!'

!upper-packed storage mode    
 do i = 1, n
   do j = i, n
     ap(i+j*(j-1)/2) = a(i,j)
   enddo
 enddo

!lower-packed storage mode    
!do i = 1, n
!  do j = 1, i
!    ap(i+(2*n-j)*(j-1)/2) = a(i,j)
!  enddo
!enddo
 
 call chpev(iopt,ap,w,z,ldz,n,aux,naux)
 
 deallocate(ap)
 deallocate(aux)
 return
 end
!!***


!{\src2tex{textfont=tt}}
!!****p* esslinterfaces/zhevc
!! NAME
!! zhevc
!!
!! FUNCTION
!! Compute the eigenvalues and the eigenvectors of a double complex Hermitian matrix
!! only the upper triangle of a is used

!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!
!! SOURCE

 subroutine zhevc(n,a,lda,w,z,ldz)
 
 integer n, lda, ldz
 complex*16 a(lda,n)
 complex*16 z(ldz,n)
 real*8 w(n)
 
 integer iopt
!upper-packed storage mode and eigenvectors (iopt=21)
!lower-packed storage mode and eigenvectors (iopt=1)
 parameter(iopt=21)
 
 integer np
 complex*16, allocatable :: ap(:)
 integer naux
 real*8, allocatable :: aux(:)

 integer istat
 
 np = n*(n+1)/2
 allocate(ap(np),stat=istat)
 if(istat.ne.0) stop 'zhevc 1: out of memory!'
 
 naux = 4*n
 allocate(aux(naux),stat=istat)
 if(istat.ne.0) stop 'zhevc 2: out of memory!'

!upper-packed storage mode    
 do i = 1, n
   do j = i, n
     ap(i+j*(j-1)/2) = a(i,j)
   enddo
 enddo

!lower-packed storage mode    
!do i = 1, n
!  do j = 1, i
!    ap(i+(2*n-j)*(j-1)/2) = a(i,j)
!  enddo
!enddo
 
 call zhpev(iopt,ap,w,z,ldz,n,aux,naux)
 
 deallocate(ap)
 deallocate(aux)
 return
 end
!!***


!{\src2tex{textfont=tt}}
!!****p* esslinterfaces/cgevc
!! NAME
!! cgevc
!!
!! FUNCTION
!! Compute the eigenvalues and the (right) eigenvectors of a complex 
!! general matrix 
!! a and z must have no common elements
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!
!! SOURCE
 subroutine cgevc(n,a,lda,w,z,ldz)
 
 integer n, lda, ldz
 complex a(lda,n)
 complex z(ldz,n)
 complex w(n)
 
 integer, parameter :: iopt = 1
 logical select(n)
 integer naux
 real, allocatable :: aux(:)

 integer istat
       
 naux = 3*n
 allocate(aux(naux),stat=istat)
 if(istat.ne.0) stop 'cgevc 1: out of memory!'

 call cgeev(iopt,a,lda,w,z,ldz,select,n,aux,naux)
 
 deallocate(aux)
 return
 end
!!***


!{\src2tex{textfont=tt}}
!!****p* esslinterfaces/rginv
!! NAME
!! rginv
!!
!! FUNCTION
!! invert a real general matrix
!! SGETRF computes an LU factorization of a general M-by-N matrix A
!! using partial pivoting with row interchanges.
!! SGETRI computes the inverse of a matrix using the LU factorization
!! computed by SGETRF.

!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! calccomm, latt_and_q
!!
!! SOURCE

 subroutine rginv(n,a,lda)
 
 integer n, lda
 real a(lda,n)
 integer, parameter :: iopt = 0
 real rcond, det(2)
 integer naux
 real, allocatable :: aux(:)
 
 naux = 200*n
 allocate(aux(naux))
 call sgeicd(a,lda,n,iopt,rcond,det,aux,naux)
 deallocate(aux)
 return
 end
!!***

!{\src2tex{textfont=tt}}
!!****p* esslinterfaces/cginv
!! NAME
!! cginv
!!
!! FUNCTION
!! invert a complex general matrix
!! CGETRF computes an LU factorization of a general M-by-N matrix A
!! using partial pivoting with row interchanges.
!! CGETRI computes the inverse of a matrix using the LU factorization
!! computed by CGETRF.

!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_altchi, creation_epsrpa
!!
!! SOURCE

 subroutine cginv(n,a,lda)

 use dpdef
 integer n, lda
 complex  a(lda,n)

 integer ipiv(n)
 complex  work(n)
 integer info

 call cgetrf(n,n,a,lda,ipiv,info)
 if(info.ne.0) then
   write(uout,*) info
   stop 'cgetrf failed'
 endif
 call cgetri(n,a,n,ipiv,work,n,info)
 if(info.ne.0) then
   write(uout,*) info
   stop 'cgetri failed'
 endif

 return
 end
!!***


!{\src2tex{textfont=tt}}
!!****p* esslinterfaces/zginv
!! NAME
!! zginv
!!
!! FUNCTION
!! invert a double precision complex general matrix
!! ZGETRF computes an LU factorization of a general M-by-N matrix A
!! using partial pivoting with row interchanges.
!! ZGETRI computes the inverse of a matrix using the LU factorization
!! computed by ZGETRF.

!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_epsmt
!!
!! SOURCE

 subroutine zginv(n,a,lda)

 use dpdef
 integer n, lda
 complex (kind(0.0d0)) :: a(lda,n)

 integer ipiv(n)
 complex (kind(0.0d0)) :: work(n)
 integer info

 call zgetrf(n,n,a,lda,ipiv,info)
 if(info.ne.0) then
   write(uout,*) info
   stop 'cgetrf failed'
 endif
 call zgetri(n,a,n,ipiv,work,n,info)
 if(info.ne.0) then
   write(uout,*) info
   stop 'cgetri failed'
 endif

 return
 end
!!***

!{\src2tex{textfont=tt}}
!!****p* esslinterfaces/zgle
!! NAME
!! zgle
!!
!! FUNCTION
!! solve the general double presicion complex problem A * X = B
!! where A is a complex matrix N x N and X, B are complex vectors


!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_epsmt
!!
!! SOURCE

 subroutine zgle(n,a,lda,b,x)
 implicit none
 integer n, lda
 complex*16 a(lda,n), x(n), b(n)

 integer iopt
 integer ipvt(n)
 complex*16 bx(n)

 bx(:) = b(:)
 iopt = 0
 call zgef(a,lda,n,ipvt)
 call zges(a,lda,n,ipvt,bx,iopt)
 x(:) = bx(:)

 return
 end
!!*** 

!!****p* esslinterfaces/cgle
!! NAME
!! cgle
!!
!! FUNCTION
!! solve the general complex problem A * X = B
!! where A is a complex matrix N x N and X, B are complex vectors


!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_epsrpa, creation_epslda, creation_altchi 
!!
!! SOURCE

 subroutine cgle(n,a,lda,b,x)
 implicit none
 integer n, lda
 complex a(lda,n), x(n), b(n)
 
 integer iopt
 integer ipvt(n)
 complex bx(n)
 
 bx(:) = b(:)
 iopt = 0
 call cgef(a,lda,n,ipvt)
 call cges(a,lda,n,ipvt,bx,iopt)
 x(:) = bx(:)
 
 return
 end
!!***



!{\src2tex{textfont=tt}}
!!****f* esslinterfaces/allowed_fftsize
!! NAME
!! allowed_fftsize
!!
!! FUNCTION
!!  calculate the size of the smallest allowed FFT size that will encompass n
!!  in FFTW all is allowed
!!  this routine is written for the ESSL routine scft, in which the
!!  allowable sizes of FFT are those whose prime factors are
!!  n <= 37748736
!!  n = 2^h * 3^i * 5^j * 7^k * 11^m  h=1,...,25  i=0,1,2  j,k,m=0,1
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! setmesh
!!
!! SOURCE

 integer function allowed_fftsize(n)
 use dpdef
 implicit none
 integer n
 integer, parameter :: nn = 56
 integer nallwd(nn)
 data nallwd/&
   2,   4,   6,   8,  10,  12,  14,  16,  18,  20,&
  22,  24,  28,  30,  32,  36,  40,  42,  44,  48,&
  56,  60,  64,  66,  70,  72,  80,  84,  88,  90,&
  96, 110, 112, 120, 126, 128, 132, 140, 144, 154,&
 160, 168, 176, 180, 192, 198, 210, 220, 224, 240,&
 252, 256, 264, 280, 288, 308/                    
 integer i
 
 if(n > nallwd(nn) .or. n < 2) then
   write(uout,*) '**ERROR allowed_fftsize'
   stop 'error allowed fft_size'
 endif
 do i = nn, 1, -1
   if(nallwd(i) >= n) allowed_fftsize = nallwd(i)
 enddo
 return
 end
!!***

!{\src2tex{textfont=tt}}
!!****p* esslinterfaces/c3dfft
!! NAME
!! c3dfft
!!
!! FUNCTION
!! computes the forward fourier transform of a complex
!! function c. the fourier transform is
!! returned in c in output (the input c is overwritten)
!! single precision fourier transform
!! isig <0  inverse
!! isig >0  direct
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! fftwfn,create_rhotw,cden,rho_tw_g,ckxclda, create_rhotw
!!
!! SOURCE

 subroutine c3dfft(a,lda,mda,hda,n1,n2,n3,isign,plan)
 use params
 integer n1, n2, n3
 integer lda, mda, hda
 complex a(lda,mda,*)
 integer isign
 
 real, parameter :: scale = 1.0
 integer, save :: naux = 60000
 double precision, allocatable :: aux(:)
 integer(ILONG):: plan

 
!the next lines are not required if n1<=8192, n2,n3<252
! if(n3 < 252) then
!   if(n2 < 252) then
!     if(n1 <= 8192) then
!       naux = 60000
!     else
!       naux = 60000 + 2.28 * n1
!     endif
!   else
!     if(n1 <= 8192) then
!       naux = 60000 + (n2+256) * (min(64,n1) + 2.28)
!     else
!       naux = 60000 + 2.28 * n1 + (n2+256) * (min(64,n1) + 2.28)
!     endif
!   endif
! else
!   if(n2 < 252) then
!     if(n1 <= 8192) then
!       naux = 60000 + (n3+256) * (min(64,n1*n2) + 2.28)
!     else
!       naux = 60000 + 2.28 * n1 + (n3+256) * (min(64,n1*n2) + 2.28)
!     endif
!   else
!     if(n1 <= 8192) then
!       naux = max( 60000 + (n2+256) * (min(64,n1) + 2.28) ,
!&                  60000 + (n3+256) * (min(64,n1*n2) + 2.28) )
!     else
!       naux = max( 60000+2.28*n1 + (n2+256) * (min(64,n1) + 2.28) ,
!&           60000 + 2.28 * n1 + (n3+256) * (min(64,n1*n2) + 2.28) )
!     endif
!   endif
! endif


 allocate(aux(naux),stat=istat)
 if(istat.ne.0) stop 'cfft out of memory'
 
 call scft3(a,lda,lda*mda,a,lda,lda*mda,n1,n2,n3,isign,scale,aux,naux)
 
 deallocate(aux)
 
 return
 end
!!***

!{\src2tex{textfont=tt}}
!!****p* esslinterfaces/cmatxmat
!! NAME
!! cmatxmat
!!
!! FUNCTION
!! Performs a complex matrix-matrix multiplication
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! calc_wf_qp
!!
!! SOURCE
 subroutine cmatxmat(m,n,k,a,lda,b,ldb,c,ldc)

 implicit none

 Integer  m, n, k, lda, ldb, ldc
 complex  a(lda,k), b(ldb,n), c(ldc,n)


 call cgemm('n','n',m,n,k,(1.,0.),a,lda,b,ldb,(0.,0.),c,ldc)

 end
!!*** 


subroutine c3dfft_plan(nl,nm,nn,plan,menoplan)

  use params
  implicit none
  integer nl, nm, nn
  integer(ILONG) :: plan
  integer(ILONG) :: menoplan

end

