!{\src2tex{textfont=tt}}
!!****p* quasiparticles/rdgw
!! NAME
!! Read GW file (quasiparticle energies)
!!!
!! FUNCTION
!! Read GW file 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! quasiparticles
!!
!! CHILDREN
!!  
!! NOTES
!! Since Abinit version 5.*.* in file _GW:
!! for each k point (and spin component) list of: 
!! ib, reenqp, gw, imenqp 
!! ib = band index
!! reenqp = real part of quasiparticle energy 
!! gw = correction with respect to previous iteration in self-consistent QP calculation
!!      (gw correction in standard perturbative GW)
!! imenqp = imaginary part of quasiparticle energy
!!
!! previous versions of Abinit:
!! - only gw (GW corrections) were reliable (and only pert. GW possible)
!! - imenqp were not written 
!!
!! the choice between the two is hardcoded in routine quasiparticles 
!! see flag filegwver
!!
!! TODO
!! Imaginary parts of quasiparticle energy are only read, not used.
!!
!! SOURCE

  subroutine rdgw(gwfile,gl,nkibz,nkbz,nbnds,lomo,kbz,ktab,gwen,imsigma,nspin,filegwver,mpilocal)

  use verbose
  use dpdef
  implicit none
  integer gl, ib, j, i, jk, nb, jb
  character (len=gl) :: gwfile
  integer nkibz, nkbz, nbnds, lomo, nspin
  integer ktab(nkbz)
  real kbz(3,nkbz)
  double precision gwen(nkibz,nbnds,nspin)
  double precision imsigma(nkibz,nbnds,nspin)
  integer filegwver  

  real reenqp
  real gw
  real imenqp  

  integer ik, ikibz, is
  integer ibcheck, ibcheckok
  
  integer nk
  real k(3), k2bz(3)
  integer g1, g2, g3
  logical mpilocal

#ifdef MPI
include 'mpif.h'
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)
if(mpilocal .and. rang/=0) return
#endif


  if(iverbose > 2) call uverbose('b','rdgw')

  if(iverbose > 0) then
   headline="Reading QP energies from GW file"
   call print_headline()
  endif

  write(uout,*)
  write(uout,*) 'GW file version ', filegwver, ' (hopefully)'
  
  ibcheckok = 0
  do ib = lomo, nbnds
    ibcheckok = ibcheckok + ib
  enddo

! opening file
  open(13,file=gwfile,status='old')

! reading number of k-points
  read(13,*) nk
  if(iverbose > 0) then
  write(uout,*) 'number of k-points found ', nk
  write(uout,*)
  endif
  
  if(nk.lt.nkibz) then
    write(uout,*) 'warning: the number of k-points found is less than ', nkibz
  endif

  do j = 1, nk

!  loop on spin
!  note: nspin components not written in GW file (should agree with dp input)
   do is=1,nspin 

    if(iverbose > 2)  write(uout,*) 'spin component', is
 

    read(13,*) (k(i),i=1,3)
 
!   determine the index number in bz
    ik = 0
    do jk = 1, nkbz
      if(all(abs(k-kbz(1:3,jk)) < 1.e-4)) ik = jk
    enddo
    if(ik==0) then
      write(uout,1023) (k(i),i=1,3)
1023      format ('trying to find k =',3f14.6,' in other BZ zones')
      do g1 = -1, 1
      do g2 = -1, 1
      do g3 = -1, 1
        k2bz(1) = k(1) + g1
        k2bz(2) = k(2) + g2
        k2bz(3) = k(3) + g3
        do jk = 1, nkbz
          if(all(abs(k2bz-kbz(1:3,jk)) < 1.e-4)) ik = jk
        enddo
      enddo
      enddo
      enddo
    endif
    if(ik .eq. 0) then
      write(uout,1071) (k(i),i=1,3)
1071      format ('**ERROR, rdgw: k-point ',3f14.6,' is not in set of kBZ!')
      stop '**ERROR, rdgw: k-point not in the set'
    endif

    ikibz = ktab(ik)

    if(iverbose > 2) then
    write(uout,100) ik, ikibz, kbz(:,ik)
100     format('  k-point, BZ number ',i3,'  IBZ number ',i3,'  r.l.c. ',3f6.3)
    endif

    read(13,*) nb
    if(iverbose > 2) then
    write(uout,*) '  number of bands found ', nb
    endif

    if(nb < (nbnds-lomo+1)) then
      write(uout,*) 'warning: the number of bands found is less than ', nbnds-lomo+1
    endif

!    if(filegwver == 5) then
!    write(uout,150)
!150     format(3x,'band',3x,'Re(E_qp)',4x,'Im(E_qp)')
!    endif


    ibcheck = 0  
    do jb = 1, nb
! new Abinit versions: imenp are now written in file GW
      if (filegwver == 5) read(13,*) ib, reenqp, gw, imenqp
! old Abinit versions: imenqp were not written in file GW
      if (filegwver == 4) read(13,*) ib, reenqp, gw  
      if(ib <= nbnds) then
! for new Abinit versions: gwen are qp energies
        if (filegwver == 5) then
          gwen(ikibz,ib,is) = dble(reenqp) /HARTREE
          imsigma(ikibz,ib,is) = dble(imenqp) /HARTREE
        endif  
! for back compatibility wrt old Abinit versions: in this case 
! gwen contains gw perturbative corrections, not qp energies
        if (filegwver == 4) gwen(ikibz,ib,is) = dble(gw) / HARTREE 
        if (filegwver == 5) then
          write(uout,200) ib, reenqp, imenqp  
200         format(4x,i3,6x,f6.2,6x,f6.2)
        endif
        if(ib >= lomo) ibcheck = ibcheck+ib
      endif
    enddo ! jb
    if(ibcheck .ne. ibcheckok) &
         write(uout,*) 'warning: some of the gw corrections are missing'
 
   enddo ! is (spin)
  enddo ! j  (k point)

  close(13)

!  if (filegwver == 4 .and. iverbose > 2) then
!
!   write(uout,*)
!   do is =1,nspin
!   write(uout,*) 'spin component ',is
!   write(uout,*) ' k       Re GW shift [eV]'
!   do ik = 1, nkibz
!     write(uout,'(i3,7x,10f7.2/50(10x,10f7.2/))') &
!&         ik, (HARTREE*gwen(ik,ib,is),ib=1,nbnds)
!   enddo
!   enddo
!   write(uout,*)
!  endif 
  
  if(iverbose > 0) then
   call print_footline()
  endif

  if(iverbose > 2) call uverbose('e','rdgw')
  end
!!***
