!{\src2tex{textfont=tt}}
!!****p* main/identk
!! NAME
!! identk
!!
!! FUNCTION
!! identify k-points in whole bz and also the q-points by which they differ
!! 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! TODO 
!! Rearrange INPUT and OUTPUT considering the objects
!! input:
!!   kibz: k-points in ibz
!!   nop: number of symmetry operations
!!   op: symmetry operation matrices
!!   irottb: rotation table for r-points
!!   ninv: = 2 inversion is considered; = 1 not considered
!! 
!! output:
!!   kbz: k-points in whole bz, and
!!   ktab: a table giving for each the k-points
!!         in the ibz to which it is related, and
!!   ktabr: the r-point to look at for each required r (unless nr=0,
!!          in which case ktabr is untouched)
!!   ktabi: whether the wfn has to be complex-conjugated (-1 if so, 1 if not)
!!   ktabo: the symmetry operation that takes kibz to each kbz
!!   nkbz: no. of such k-points
!!   wtk: weight for each k-point in ibz for symmetric quantities:
!!        no. of distinct k's in whole bz / (ninv*nop)                        
!!
!! PARENTS
!! main, get_initial_states
!!
!! CHILDREN
!! icompv_fraction, dosym
!!
!! SOURCE
!=======================================================================
  subroutine identk(iparam,bzstruct,fftmesh,rot,symstr)
!=======================================================================

 use dpdef
 use ggwdef
 use verbose
 use interfaces
 implicit none
 type(dpparam) iparam
 type(bzstructure) bzstruct
 type(mesh)fftmesh
 type(rot_table) rot
 type(symmetry) symstr

 integer nkbzx

 integer istat,iinv,ikibz,iop,iold,ikbz,i,ik,ir
 real knew(3)
 integer, external :: icompv_fraction
#ifdef MPI
 include 'mpif.h'
 integer ierr,rang,nprocs
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif

 if(iverbose > 2) call uverbose('b','identk')

 nkbzx = iparam%nkibz * iparam%nop * 2
 allocate(bzstruct%kbz(3,nkbzx),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 allocate(bzstruct%wtk(iparam%nkibz),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 allocate(bzstruct%ktab(nkbzx),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 allocate(bzstruct%ktabi(nkbzx),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 allocate(bzstruct%ktabo(nkbzx),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 if(.not.iparam%JDOS) then
  allocate(bzstruct%ktabr(fftmesh%nr,nkbzx),stat=istat)
  if(istat.ne.0) stop 'out of memory'
 endif

 if(iverbose > 2) call memstat('ktabr allocated =','i',fftmesh%nr,nkbzx)
#ifdef MPI
 if (.not.(iparam%mpilocal .and. rang /= 0))  then
#endif

!zero no. of k-points found
 iparam%nkbz = 0

!loop over k-points in ibz
 do ikibz = 1, iparam%nkibz
   bzstruct%wtk(ikibz) = 0.0

!  loop over symmetry operations r and inversions i
   do iinv = 1, iparam%ninv
     do iop = 1, iparam%nop
       if(symstr%symafm(iop)==-1) cycle 
!      form rik
       call dosym(symstr%op(1,1,iop), iinv, bzstruct%kibz(1,ikibz), knew(1))
!      check whether it has already been found (to within a rl vector)
       iold = 0
       do ikbz = 1, iparam%nkbz
         iold = iold + icompv_fraction(knew,bzstruct%kbz(1,ikbz))
       enddo
       if (iold.eq.0) then
         iparam%nkbz = iparam%nkbz + 1
         bzstruct%wtk(ikibz) = bzstruct%wtk(ikibz) + 1.0
         if (iparam%nkbz.gt.nkbzx) then
           write(uout,*) '**error identk 1: nkbzx too small;'
           write(uout,*) '                  nkbzx = ', nkbzx
           write(uout,*) '                  increase nkbzx!'
           stop
         endif
         bzstruct%kbz(1,iparam%nkbz) = knew(1)
         bzstruct%kbz(2,iparam%nkbz) = knew(2)
         bzstruct%kbz(3,iparam%nkbz) = knew(3)
         bzstruct%ktab(iparam%nkbz) = ikibz
         bzstruct%ktabo(iparam%nkbz) = iop
         bzstruct%ktabi(iparam%nkbz) = 3 - 2 * iinv
         if(.not.iparam%JDOS) then
          do ir = 1, fftmesh%nr
!          we want u(r**-1 r).  irottb contains the r-point-no. of r**-1 r.
           bzstruct%ktabr(ir,iparam%nkbz) = rot%irottb(ir,iop)
          enddo 
         endif
       endif
     enddo 
   enddo 
   bzstruct%wtk(ikibz) = bzstruct%wtk(ikibz) / (iparam%ninv*iparam%nop)
 enddo 


! print out results
 if(iverbose > 0)then 
   headline="About k points"
   call print_headline()
   write(uout,*) 'identifying k-points'
   write(uout,'("  number of k-points in the ibz ",i5)') iparam%nkibz
   write(uout,*) ' k-points [reduced coordinates]    weights'
   write(uout,'(3(1x,f10.6),2x,f6.3)') ((bzstruct%kibz(i,ik),i=1,3),bzstruct%wtk(ik),ik=1,iparam%nkibz)
   write(uout,*)
   write(uout,'("  number of k-points in the full bz ",i5)') iparam%nkbz
   write(uout,*) ' k-points [reduced coordinates]:'
   write(uout,'(3(1x,f10.6))') ((bzstruct%kbz(i,ik),i=1,3),ik=1,iparam%nkbz)
   call print_footline()
 endif

 if(.not.iparam%LDA .and. .not.iparam%JDOS .and. .not. iparam%doublegrid) deallocate(rot%irottb)
 
 if(iverbose > 2) call uverbose('e','identk')

#ifdef MPI
 endif
if(iparam%mpilocal) then
 call mpi_barrier(mpi_comm_world,ierr)
 call mpi_bcast(iparam%nkbz,1,mpi_integer,0,mpi_comm_world,ierr)
 call mpi_bcast(bzstruct%ktabo,iparam%nkbz,mpi_integer,0,mpi_comm_world,ierr)
 if(.not.iparam%JDOS) call mpi_bcast(bzstruct%ktabr,fftmesh%nr*iparam%nkbz,mpi_integer,0,mpi_comm_world,ierr)
 call mpi_bcast(bzstruct%ktabi,iparam%nkbz,mpi_integer,0,mpi_comm_world,ierr)
 call mpi_bcast(bzstruct%ktab,iparam%nkbz,mpi_integer,0,mpi_comm_world,ierr)
 call mpi_bcast(bzstruct%kbz,3*iparam%nkbz,mpi_real,0,mpi_comm_world,ierr)
endif
#endif

 end
!!***


!{\src2tex{textfont=tt}}
!!****p* main/lattice
!! NAME
!! lattice
!!
!! FUNCTION
!! Calculate reciprocal lattice vectors and cell volumes
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! rginv
!!
!! SOURCE

 subroutine lattice(iparam,latt)

 use ggwdef
 use dpdef
 use verbose
 implicit none

 type(lattstruct) latt
 type(dpparam) iparam 

 integer lattype,i
 real ula

!a1, a2, a3: real-space lattice vectors in au
!b1, b2, b3: reciprocal-space lattice vectors in au**-1
!ucvol: unit cell volume in au**3
!bzvol: reciprocal cell volume

 real aa(3,3)
 real a

 if(iverbose > 2) call uverbose('b','lattice')

! PI = 4.0 * atan(1.0)

!calculate brillouin zone and unit cell volumes
 latt%ucvol = latt%a1(1)* (latt%a2(2)*latt%a3(3)-latt%a2(3)*latt%a3(2)) &
       + latt%a1(2)* (latt%a2(3)*latt%a3(1)-latt%a2(1)*latt%a3(3)) &
       + latt%a1(3)* (latt%a2(1)*latt%a3(2)-latt%a2(2)*latt%a3(1))
 latt%bzvol = 8*PI*PI*PI / latt%ucvol

!calculate reciprocal-space lattice vectors b1-b3
 latt%b1(1) = 2.0 * PI * (latt%a2(2)*latt%a3(3)-latt%a2(3)*latt%a3(2)) / latt%ucvol
 latt%b1(2) = 2.0 * PI * (latt%a2(3)*latt%a3(1)-latt%a2(1)*latt%a3(3)) / latt%ucvol
 latt%b1(3) = 2.0 * PI * (latt%a2(1)*latt%a3(2)-latt%a2(2)*latt%a3(1)) / latt%ucvol
 latt%b2(1) = 2.0 * PI * (latt%a3(2)*latt%a1(3)-latt%a3(3)*latt%a1(2)) / latt%ucvol
 latt%b2(2) = 2.0 * PI * (latt%a3(3)*latt%a1(1)-latt%a3(1)*latt%a1(3)) / latt%ucvol
 latt%b2(3) = 2.0 * PI * (latt%a3(1)*latt%a1(2)-latt%a3(2)*latt%a1(1)) / latt%ucvol
 latt%b3(1) = 2.0 * PI * (latt%a1(2)*latt%a2(3)-latt%a1(3)*latt%a2(2)) / latt%ucvol
 latt%b3(2) = 2.0 * PI * (latt%a1(3)*latt%a2(1)-latt%a1(1)*latt%a2(3)) / latt%ucvol
 latt%b3(3) = 2.0 * PI * (latt%a1(1)*latt%a2(2)-latt%a1(2)*latt%a2(1)) / latt%ucvol

!find bravais lattice type
  lattype = 1

!fcc
 a = (latt%ucvol*4.0)**(1.0/3.0)
 do i = 1, 3
   aa(1,i) = latt%a1(i) / (a/2)
   aa(2,i) = latt%a2(i) / (a/2)
   aa(3,i) = latt%a3(i) / (a/2)
   if(aa(1,i)<=0 .and. aa(2,i)<=0 .and. aa(3,i)<=0) then
     aa(:,i) = - aa(:,i)
   endif
 enddo
 if(aa(1,1)==1 .and. aa(1,2)==0 .and. aa(1,3)==1 .and.&
        aa(2,1)==0 .and. aa(2,2)==1 .and. aa(2,3)==1 .and.&
        aa(3,1)==1 .and. aa(3,2)==1 .and. aa(3,3)==0) then
   lattype = 1
 endif
 if(iverbose > 1) then
 if(lattype == 1) then
   headline="Lattice parameters"
   call print_headline(headline)
   write(uout,50) a, a * 0.529177
50      format(' a = ',f5.2,' [au]   ',f5.2,' angstrom')
    write(uout,*)
  endif

  write(uout,100) latt%a1, latt%a2, latt%a3
100   format(' real-space lattice vectors in cartesians [au]:'&
           /('     ',3f8.2))
  write(uout,200) latt%b1, latt%b2, latt%b3
200   format(/' calculated reciprocal lattice vectors', &
              ' in cartesians [au]:'/&
             ('     ',3f8.4))
  write(uout,300) latt%bzvol, latt%ucvol
300   format (/' brillouin zone volume = ', f8.4, ' [au]'/&
               ' unit cell volume      = ', f8.2, ' [au]')
  call print_footline()
 endif


!transformation matrices rl -> cc and cc -> rl
 latt%brl2cc(:,1) = latt%b1(:) 
 latt%brl2cc(:,2) = latt%b2(:) 
 latt%brl2cc(:,3) = latt%b3(:)
 latt%bcc2rl(:,:) = latt%brl2cc(:,:)
 call rginv(3,latt%bcc2rl,3)
!note that ULA is not guaranteed to be the same as that used by
!RJN; it is chosen to be sqrt(2) * |a1|
 ula = sqrt(2.0) * sqrt(latt%a1(1)**2+latt%a1(2)**2+latt%a1(3)**2)
!ula should be the lattice parameter, and tpula = 2pi/a
 latt%tpula = 2.0 * PI / ula

!calculate omegaplasma
 iparam%omegaplasma = sqrt(4.0 * (4.0*atan(1.0)) * iparam%nel / latt%ucvol)


 if(iverbose > 2) call uverbose('e','lattice')
 end
!!***


!{\src2tex{textfont=tt}}
!!****p* Multi/reduceto1bz
!! NAME
!! reduceto1bz
!!
!! FUNCTION
!! given a k+g-point not necessarily in the 1st Brillouin Zone
!! find k and G where k is in the first BZ and G
!! a RL vector such that kpg=k+G.
!! k, G and kpg are expressed in RL coords.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! latt_and_q
!!
!! CHILDREN
!! vlngth
!!
!! SOURCE

  subroutine reduceto1bz(kpg,k,g,latt)

  use verbose
  use ggwdef
  implicit none
  type(lattstruct) latt
  real kpg(3), k(3)
  integer g(3)

  real kt(3)
  integer gt(3)
  real xmin, x, y, ymin
  integer gt1, gt2, gt3
  real, external :: vlngth

!  if(iverbose>2) call uverbose('b','reduceto1bz')

  xmin = 99999999.99
  do gt1 = -10, 10
    gt(1) = gt1
    do gt2 = -10, 10
      gt(2) = gt2
      do gt3 = -10, 10
        gt(3) = gt3
        kt(:) = kpg(:) - gt(:)
        x = vlngth(kt,latt%b1,latt%b2,latt%b3)
!       Heuristic method for choosing the right G when two Gs will do
!       (avoid the G such that kBZ has all components with same sign
!       otherwise k1BZ+k2BZ may need a G0 with components other than
!       -1,0,1 in RL coordinates to translate it back to the 1st BZ)
!       so add a small "penalty" to X if all 3 components of G have
!       the same sign
        x = x + 0.001 * abs(sign(1.,kt(1))+sign(1.,kt(2))+sign(1.,kt(3)))
!       this test has been added in order to (really) discriminate between 
!       two equal cases. in fact what can happen is that x and xmin are equal
!       and the choice between the two is compiler dependent, or worse (ifort) 
!       compiler flag dependent. of course the two results should be completely 
!       equivalent, but it's not the case for q/=0 and shikted kpoints where
!       the time reversal is broken and eps(q) and eps(-q) are not equal anymore
        if (abs(x-xmin)>0.000001)then  
         if(x < xmin) then
          g(:) = gt(:)
          k(:) = kt(:)
          xmin = x
         endif
        else
         ymin=vlngth(float(g),latt%b1,latt%b2,latt%b3)
         y=vlngth(float(gt),latt%b1,latt%b2,latt%b3)
         if(y<ymin) then
          g(:) = gt(:)
          k(:) = kt(:)
          xmin = x
         endif
        endif
      enddo
    enddo
  enddo

  ! this may be an alternative ?

  ! k is allowed to be in the BZ _including_ its border
  !  g(:) = nint( ceiling(abs(kpg(:))-0.5)*sign(1.,kpg(:)) ); ! [-0.5,0.5] -> 0
  !  g(:) = nint( ceiling(kpg(:)-0.5)*1.0 );                  ! (-0.5,0.5] -> 0
!  g(:) = nint( floor(kpg(:)+0.5)*1.0 );                     ! [-0.5,0.5) -> 0
!  k(:) = kpg(:) - g(:);

  ! if (any(kt(:).ne.k(:)) .or. any(gt(:).ne.g(:))) then
  !   write(uout,*) kpg(:), k(:), g(:), kt(:), gt(:)
  !   stop
  ! endif

!  if(iverbose>2) call uverbose('e','reduceto1bz')
  end
!!***

!{\src2tex{textfont=tt}}
!!****p* latt_and_q/findshells
!! NAME
!! findshells
!!
!! FUNCTION
!! Find the g-vector numbers that close a shell
!! check if ngs closes a shell
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! latt_and_q
!!
!! CHILDREN
!! vlngth
!!
!! SOURCE

 subroutine findshells(iparam,latt,trspace)

 use verbose
 use ggwdef
 use dpdef
 implicit none
 type(dpparam) iparam
 type(lattstruct) latt
 type(transposed_space) trspace

 real g(3)
 integer s, ig
 real l, lold
 logical :: shellclosed = .false.
 real, external :: vlngth
 integer newnpwmat
 
 if(iverbose>2) call uverbose('b','findshells')

 if(iverbose > 1) then
 write(uout,*) 'shell  g-vector     radius [au]'
 endif
 s = 0
 g(:) = trspace%gvec(:,1)
 lold = vlngth(g,latt%b1,latt%b2,latt%b3)
 do ig = 2, iparam%npwvec
   g(:) = trspace%gvec(:,ig)
   l = vlngth(g,latt%b1,latt%b2,latt%b3)
   if((l-lold).gt.0.000001) then
     if(iverbose > 1) then
     write(uout,10) s, ig-1, lold
10        format(i6,i10,f11.6)
     endif
     lold = l
     s = s + 1
     if(iparam%npwmat == ig-1) then
       shellclosed = .true.
     else
      if(iparam%npwmat < ig-1) then
        newnpwmat = ig-1
        exit
      endif
     endif
   endif
 enddo
 if(shellclosed) then
   if(iverbose > 1)write(uout,*)'shell closed at ', iparam%npwmat
 else if(iparam%npwmat == iparam%npwvec) then
   if(iverbose > 1)write(uout,*)'shell probably closed at ', iparam%npwmat
 else
   write(uout,*) 'WARNING, findshell: shell not closed'
   write(uout,*) 'npwmat = ', iparam%npwmat,' probably coming from a wrong kss or from a qplda'
   write(uout,*) 'calculation will continue with npwmat =',newnpwmat
!   stop 'shell not closed'
   iparam%npwmat = newnpwmat
 endif
 write(uout,*)

 if(iverbose>2) call uverbose('e','findshells')
 end
!!***



!{\src2tex{textfont=tt}}
!!****f* Multi/vlngth
!! NAME
!! vlngth
!!
!! FUNCTION
!! calculate the length of a vector expressed in terms of the basis vectors b1-b3
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main, createVc, reduceto1bz, findshells, clcqpg, create_eps_exc, wreps
!!
!! SOURCE

 real function vlngth(v,b1,b2,b3)

 implicit none
 real v(3), b1(3), b2(3), b3(3)
 integer i

 vlngth = 0.0
 do i = 1, 3
   vlngth = vlngth + (v(1)*b1(i)+v(2)*b2(i)+v(3)*b3(i))**2
 enddo
 vlngth = sqrt(vlngth)
 end
!!***


!{\src2tex{textfont=tt}}
!!****f* findkp/lcompv
!! NAME
!! lcompv
!!
!! FUNCTION
!! compare two 3d vectors, and return .true. if same, .false. if diff
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! findkp, latt_and_q
!!
!! CHILDREN
!! is_zero
!!
!! SOURCE
 logical function lcompv(v1,v2)
 use dpdef
 implicit none
 real v1(3), v2(3)
 logical, external :: is_zero

 lcompv = is_zero(abs(v1(:)-v2(:)))
 end
!!***

!{\src2tex{textfont=tt}}
!!****f* Multi/icompv_fraction
!! NAME
!! icompv_fraction
!!
!! FUNCTION
!! compare fractional part of two vectors.  return 1 if same, 0 if different. 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! checkq, identk, identq, findnq, build_qibz
!!
!! CHILDREN
!! is_zero
!!
!! SOURCE
!=======================================================================
 integer function icompv_fraction(v1,v2)
!=======================================================================
 use dpdef
 implicit none

 real v1(3), v2(3)
 logical, external :: is_zero

 icompv_fraction=0;
 if(is_zero(abs((v1(:)-v2(:))-nint(v1(:)-v2(:))))) icompv_fraction=1;
   ! fraction() may return -0.9999999, use nint() instead 
 end function icompv_fraction
!!***


!{\src2tex{textfont=tt}}
!!****f* checkq/is_zero
!! NAME
!! litst0
!!
!! FUNCTION
!! Check whether a vector is nominally zero (returns .true. if so)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!!  lcompv, icompv_fraction, latt_and_q
!!
!! SOURCE
!=======================================================================
 logical function is_zero(v)
!=======================================================================
 use dpdef
 implicit none

 real v(3)
 is_zero = all(abs(v(:))<q_precision) 
 end function is_zero

!!***

!{\src2tex{textfont=tt}}
!!****f* checkq/is_small
!! NAME
!! litst0
!!
!! FUNCTION
!! Check whether a q-vector is small enough to apply k.p perturbation
!! theory (returns .true. if so)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! checkq, qtabg0, build_qibz, identq, latt_and_q, rddpin
!!
!! SOURCE
!=======================================================================
 logical function is_small(q)
!=======================================================================
 use dpdef
 implicit none

 real q(3) 
 is_small= all(abs(q(:))<kp_limit) 

 end function is_small
!!***



!{\src2tex{textfont=tt}}
!!****f* Multi/modx
!! NAME
!! modx
!!
!! FUNCTION
!! function similar to mod.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! NOTE
!! calculate a function similar to mod:
!!   modx(i,j) = mod (i,j)             i>=0                              
!!             = mod ( mod(i,j)+j , j) i<0
!!   thus modx(3,2)=1; modx(-1,2)=1.
!!   this is different from mod(-1,2)=-1 for i negative.
!!   rewritten by nathalie vast on august the 28th 2000
!!   to allow vectoriz. (inline function)
!!   checked on november 14th 2000
!!    if (i.ge.0) then
!!      modx = mod(i,j)
!!    else
!!      modx = mod( mod(i,j) + j , j)
!!    endif                                                
!!   -----------------------------------------------
!!    if (i.ge.0) then
!!      modx = mod(i,j)
!!    else
!!      modx = mod( mod(i,j) + j , j)
!!    endif                                                             
!! 
!!
!! PARENTS
!! surot, setmesh, setup_rrotational_table
!!
!! SOURCE

 function modx(i,j)

 implicit none
 integer i, j, modx
 modx = mod( mod(i,j) + ((1-sign(1,i))/2)*j , j)
 end
!!***


!{\src2tex{textfont=tt}}
!!****f* Multi/findkp
!! NAME
!! findkp
!!
!! FUNCTION
!! check if the k-point is in the set of the kbz
!! (subtracting the reciprocal vector g0)
!! and return the index number and the reciprocal vector g0
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_tablekp, create_tablek2
!!
!! CHILDREN
!! lcompv
!!
!! SOURCE

 logical function findkp(nkbz,kbz,ik,k,g0)

 implicit none
 integer nkbz
 real kbz(3,nkbz)
 real k(3)
 integer ik
 integer g0(3)
 integer ikp
 real kp(3), kpg0(3)
 integer ig1, ig2, ig3
 logical, external :: lcompv
 integer g(5)

 findkp=.false.
 
 g(1) =  0
 g(2) =  1
 g(3) = -1
 g(4) =  2
 g(5) = -2

 do ikp = 1, nkbz
   kp(:) = kbz(:,ikp)
!  form k' + g0
   do ig1 = 1,5
     kpg0(1) = kp(1) + g(ig1)
     do ig2 = 1,5
       kpg0(2) = kp(2) + g(ig2)
       do ig3 = 1,5
         kpg0(3) = kp(3) + g(ig3)
!        compare k' + g0 with k given
         if(lcompv(kpg0,k)) then
           ik = ikp
           g0(1) = g(ig1)
           g0(2) = g(ig2)
           g0(3) = g(ig3)
           findkp=.true.
           return
         endif
       enddo
     enddo
   enddo
 enddo
! stop 'findkp did not find kp!!'

 end
!!***

!{\src2tex{textfont=tt}}
!!****f* main/pdtqrc
!! NAME
!! pdtqrc
!!
!! FUNCTION
!! Calculate the dot product of a real vector with a complex
!! vector, where each is in terms of b1-b3
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! calccomm, creation_chi0, create_eps_exc, creation_chi0drude
!!
!! SOURCE

 complex function pdtqrc(r,c,b1,b2,b3)

 implicit none
 complex c(3)
 real r(3), b1(3), b2(3), b3(3)
 integer i

 pdtqrc = 0.0
 do i = 1, 3
   pdtqrc = pdtqrc + (r(1)*b1(i)+r(2)*b2(i)+r(3)*b3(i)) * &
                     (c(1)*b1(i)+c(2)*b2(i)+c(3)*b3(i))
 enddo
 end
!!***

!{\src2tex{textfont=tt}}
!!****p* latt_and_q/identifyg
!! NAME
!! identifyg
!!
!! FUNCTION
!! Find ig, index of gvec, corresponding to g
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! latt_and_q
!!
!! SOURCE

 subroutine identifyg(g,ig,gvec,npwvec)

 implicit none
 integer npwvec, ig
 integer g(3), gvec(3,npwvec)
 ig = 1
 do while(any(g(:)/=gvec(:,ig)))
   ig = ig+1
   if(ig > npwvec) exit
 enddo
 end
!!***


!{\src2tex{textfont=tt}}
!!****p* qpol_and_q/clcqpg
!! NAME
!! clcqpg
!!
!! FUNCTION
!! calculate |q+g| for each q and g
!! input:
!!   gvec: g-vectors
!!   q: q-points
!!   b1,b2,b3: rl vectors
!! output:
!!   qpg: |q+g| for each q and g
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS 
!! qpol_and_q
!!
!! CHILDREN
!! vlngth
!!
!! SOURCE
!=======================================================================
 subroutine clcqpg(iparam,gvec,q,b1,b2,b3,qpg)
!==================================================================

 use verbose
 use dpdef
 implicit none
 type(dpparam) iparam

 integer :: ig, iq
 real :: qpgtemp
 real qpg(iparam%npwvec,iparam%nqibz), q(3,iparam%nqibz), b1(3),b2(3),b3(3)
 integer gvec(3,iparam%npwvec)

 real gpq(3)
 real, external :: vlngth

 if(iverbose > 2) call uverbose('b','clcqpg')
    

 do iq = 1, iparam%nqibz
  do ig = 1, iparam%npwvec
   gpq(1) = gvec(1,ig) + q(1,iq)
   gpq(2) = gvec(2,ig) + q(2,iq)
   gpq(3) = gvec(3,ig) + q(3,iq)
   qpg(ig,iq) = vlngth(gpq,b1,b2,b3)
  enddo
 enddo

 if(iparam%coulcutoff) then
  write(uout,'("cut-off on the coulombian interaction ",f6.2)') iparam%rlcut
  do iq = 1, iparam%nqibz
   qpg(1,iq) = sqrt(2.0)/iparam%rlcut
   do ig = 2, iparam%npwvec
     qpgtemp = qpg(ig,iq)
     qpg(ig,iq) = qpgtemp * sqrt(1.0/(1.0-cos(qpgtemp*iparam%rlcut)))
   enddo
  enddo
 endif

 if(iverbose > 2) call uverbose('e','clcqpg')
 end
!!***

!{\src2tex{textfont=tt}}
!!****p* qpol_and_q/qtabg0
!! NAME
!! qtabg0
!!
!! FUNCTION
!! find the denominator of the q-vectors qdenom
!! and create the table which relates
!! q = (k - k') - g0 = (q1, q2, q3) - g0
!! to the q in the qbz and the rec. lattice vector g0 by
!! 
!!  ind = kmkptab(q1*qdenom, q2*qdenom, q3*qdenom) 
!!
!!  iqbz = ind / 1000 ; ind = ind - 1000*iqbz
!!  ig01 = ind / 100 ; ind = ind - 100*ig01 ; ig01 = ig01 - 2
!!  ig02 = ind / 10  ; ind = ind - 10*ig02  ; ig02 = ig02 - 2
!!  ig03 = ind - 2
!!
!! input:
!!  nqbz, qbz, qdenommx
!! ouput:
!!  qdenom, kmkptab
!!
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS 
!! qpol_and_q
!!
!! CHILDREN
!! is_small,mnint
!!
!! SOURCE
!=======================================================================
 subroutine qtabg0 (nqbz, qbz, qdenom, kmkptab)
!=======================================================================

 use verbose
 use dpdef
 use ggwdef

 implicit none
 integer, intent(in)  :: nqbz 
 real, dimension(3,nqbz), intent(in)  :: qbz
 integer, intent(out) :: qdenom  
 integer, dimension(-3*qdenommx:3*qdenommx,-3*qdenommx:3*qdenommx,&
                    -3*qdenommx:3*qdenommx), intent(out) :: kmkptab

 integer :: i, iq, itemp, ind, ig01, ig02, ig03
 real    :: rtemp
 real, dimension(3) :: qpg0
 logical, external  :: is_small
 integer, external :: mnint

 if(iverbose > 2) call uverbose('b','qtabg0')

!find the denominator of the q-vectors qdenom
 qdenom = 1
 iq_loop: do iq = 1, nqbz
   if (is_small(qbz(1,iq))) cycle  ! why ?
   do i = 1, 3
     rtemp = qbz(i,iq)
     if (abs(rtemp) < 1e-3) cycle
     itemp = nint(1. / rtemp)
     if (itemp > qdenom) qdenom = itemp
   enddo
 enddo iq_loop
 if (qdenom > qdenommx) stop 'qtabg0 1: qdenom > qdenommx'

 kmkptab(:,:,:) = 0
 do iq = 1, nqbz
   do ig01 = -2, 2
   do ig02 = -2, 2
   do ig03 = -2, 2
     qpg0(1) = qbz(1,iq) + ig01
     qpg0(2) = qbz(2,iq) + ig02
     qpg0(3) = qbz(3,iq) + ig03
     qpg0(:) = qpg0(:) * qdenom 
     ind = 1000*iq + 100*(ig01+2) + 10*(ig02+2) + (ig03+2)
     kmkptab(mnint(qpg0(1)), mnint(qpg0(2)), mnint(qpg0(3))) = ind
   enddo
   enddo
   enddo
 enddo

 if (iverbose > 1) & 
  write(uout,'(/" the denominator of the q-vectors qdenom =",i5)') qdenom
 if(iverbose > 2) call uverbose('e','qtabg0')

 end subroutine qtabg0
!!***



!{\src2tex{textfont=tt}}
!!****p* makeWq/identq_old
!! NAME
!! identq_old
!!
!! FUNCTION
!! identify q-points in whole bz
!! not used in chi!
!! note: for q close to zero only one q-point is counted (ie not the
!! rotations and reflections of the little q, nor other little
!! q-points in the qibz set)
!! input:
!!   q: q-points in ibz
!!   op: symmetry operation matrices
!! output:
!!   qbz: q-points in whole bz, and
!!   qtab: a table giving for each the q-points
!!         in the ibz to which it is related, and
!!   qtabi: whether the wfn has to be complex-conjugated (-1 if so, 1 if not)
!!   qtabo: the symmetry operation that takes kibz to each kbz
!!   nqbz: no. of such q-points
!!
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! makeWq, makeWqcoup
!!
!! CHILDREN
!! dosym
!!
!! SOURCE

  subroutine identq_old(q,nq,nqbzmx,op,nop,ninv,qbz,qtab,qtabi,qtabo,nqbz)

  use verbose
  use dpdef
  implicit none

  integer nq, nop, nqbzmx
  real q(3,nq), qbz(3,nqbzmx), op(3,3,nop)
  integer qtab(nqbzmx), qtabi(nqbzmx), qtabo(nqbzmx)

  integer nqbz,nqeq0,iq,iop,iinv,iold,iqbz,i,j,ninv
  real qnew(3)
  logical, external :: is_small
  integer, external :: icompv_fraction
  if(iverbose > 2) call uverbose('b','identq_old')

  if(iverbose > 0) then 
   write(uout,*)'   ------- identifying q-points'
   print *
  endif

! zero no. of q-points found
  nqbz = 0

  nqeq0 = 0

! loop over q-points in ibz
  do iq = 1, nq

!   if q is close to zero, treat it differently as described above
   if (is_small(q(:,iq))) then
      if (nqeq0.eq.0) then
        nqeq0 = 1
        nqbz = nqbz + 1
        if (nqbz.gt.nqbzmx) then
            write(uout,*) '**error identq 1: nqbzmx too small;'
            write(uout,*) '                  increase nqbzmx!'
            stop
          endif
        qbz(1,nqbz) = q(1,iq)
        qbz(2,nqbz) = q(2,iq)
        qbz(3,nqbz) = q(3,iq)
        qtab(nqbz) = iq
        qtabo(nqbz) = 1
        qtabi(nqbz) = 1
      endif
   else

!   loop over symmetry operations r and inversions i
    do iop = 1, nop
      do iinv = 1, ninv
!       form rik
        call dosym(op(1,1,iop),iinv,q(1,iq),qnew(1))
!       check whether it has already been found (to within a rl vector)
        iold = 0
        do iqbz = 1, nqbz
          iold = iold + icompv_fraction(qnew,qbz(1,iqbz))
        enddo
        if (iold.eq.0) then
          nqbz = nqbz + 1
          if (nqbz.gt.nqbzmx) then
            write(uout,*) '**error identq 2: nqbzmx too small;'
            write(uout,*) '                  increase nqbzmx!'
            stop
          endif
          qbz(1,nqbz) = qnew(1)
          qbz(2,nqbz) = qnew(2)
          qbz(3,nqbz) = qnew(3)
          qtab(nqbz) = iq
          qtabo(nqbz) = iop
          qtabi(nqbz) = 3 - 2 * iinv
        endif

      enddo 
    enddo

   endif

  enddo


 if(iverbose > 1) then
!print out results
 write(6,1000) nq, (j,(q(i,j),i=1,3),j=1,nq)
1000  format (' q-points in irreducible wedge (ibz) ',i3,/ &
              ' q-points [reciprocal lattice units]:'/ &
              (i5,3f12.6))
      write(6,2000) nop, nqbz, (j,(qbz(i,j),i=1,3),j=1,nqbz)
2000  format ( ' together with the',i5,' symmetry operations', &
                      ' and inversion'/ & 
               ' have yielded',i7,' q-points in brillouin zone (bz):'/ & 
                (4(i5,3f5.2)))
 write(uout,*)
 endif

 if(iverbose > 2) call uverbose('e','identq_old')
 end
!!***



!{\src2tex{textfont=tt}}
!!****p* makeWq/iqg0
!! NAME
!! iqg0
!!
!! FUNCTION
!! find the g0 associated at a certain q. this works only for kernel MT
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! makeWq,makeWqcoup
!!
!! TODO
!! this is an old subroutine. we should harmonize this with what is done in the exciton case
!!
!! SOURCE
 subroutine iqg0(k,qdenom,kmkptab,iq,g0)
 
 use ggwdef
 use dpdef

 implicit none
 integer  qdenom,iq,i !,qdenommx
 integer g0(3),kmkptab(-3*qdenommx:3*qdenommx,-3*qdenommx:3*qdenommx,-3*qdenommx:3*qdenommx)
 real k(3)

 k(:) = k(:) * qdenom
 i = kmkptab(nint(k(1)), nint(k(2)), nint(k(3)))
 if (i == 0) stop 'excheig 2: q = k-kp-g0 not found'
 iq = i / 1000 ; i = i - 1000*iq
 g0(1) = i / 100 ; i = i - 100*g0(1) ; g0(1) = g0(1) - 2
 g0(2) = i / 10  ; i = i - 10*g0(2)  ; g0(2) = g0(2) - 2
 g0(3) = i - 2

 end
!!***

!{\src2tex{textfont=tt}}
!!****f* qtabg0/mnint
!! NAME
!! mnint
!!
!! FUNCTION
!! Calculate a slightly modified version of nint = nint(q+penalty*sign(q))
!! This is because there are cases in which the value of a difference of kpoints
!! is 1.5 (whoose nint gives 2) in one place of the code and 1.499999 (whoose nint 
!! gives 1) in another part of code, for the very same kpoints difference, which imply
!! of course an error. We add then a small penalty of 10^- to avoid this.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! qtabg0, create_tablekmkp
!!
!! SOURCE

 integer function mnint(q)
 implicit none
 real :: q
 real, parameter :: penalty=0.00001
 mnint = nint(q+penalty*sign(1.,q))
 end function mnint
!!***
