!{\src2tex{textfont=tt}}
!!****p* qpol_and_q/build_qibz
!! NAME
!! build_qibz
!!
!! FUNCTION
!! Identify the q-points into the irreducible BZ
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! qpol_and_q
!!
!! CHILDREN
!! is_small, icompv_fraction, dosym, reduceto1bz
!!
!! SOURCE

  subroutine build_qibz(iparam,bzstruct,bzstructp,qstruct,symstr,latt,nopq,ninvq)

  use verbose
  use dpdef
  use ggwdef
  implicit none
  type(dpparam) iparam
  type(bzstructure) bzstruct, bzstructp   ! final and initial states
  type(symmetry) symstr
  type(lattstruct) latt
  type(qstructure) qstruct
  integer, intent(in) :: nopq,ninvq
 
  integer nqallm
  integer nqall
  real, allocatable :: qall(:,:)

  real qrot(3), qposs(3), qtemp(3)
  integer gtemp(3)
  logical, external :: is_small
  integer, external :: icompv_fraction
  integer istat, iop, iinv
  integer ifound, ik, iq


  if(iverbose > 2) call uverbose('b','build_qibz')

  write(uout,*) 'find q-points q = k - kp'

  nqallm = 1000

710   allocate(qall(3,nqallm),stat=istat)
  if(istat.ne.0) stop 'out of memory'

! zero no. of q-points found
  nqall = 0
! loop over all k-points in bz, forming k - kp(1)
  do ik = 1, iparam%nkbz
      qposs(1) = bzstruct%kbz(1,ik) - bzstructp%kbz(1,1)
      qposs(2) = bzstruct%kbz(2,ik) - bzstructp%kbz(2,1)
      qposs(3) = bzstruct%kbz(3,ik) - bzstructp%kbz(3,1)

!     check whether this q (or its equivalent) has already been found
      ifound = 0
      do iq = 1, nqall
        do iinv = 1,ninvq
          do iop = 1, nopq
            call dosym(symstr%op(1,1,iop),iinv,qall(1,iq),qrot)
             if (icompv_fraction(qrot,qposs).eq.1) ifound = ifound + 1
          enddo
        enddo
      enddo
      if (ifound.eq.0) then
        nqall = nqall + 1
        if (nqall.gt.nqallm) then
          deallocate(qall)
          nqallm = nqallm * 2
          goto 710
        endif
        qall(1,nqall) = qposs(1)
        qall(2,nqall) = qposs(2)
        qall(3,nqall) = qposs(3)
      endif
  enddo

  iparam%nqibz = nqall


  allocate(qstruct%qibz(3,iparam%nqibz),stat=istat)
  if(istat.ne.0) stop 'out of memory'
 
  qstruct%qibz(:,1:iparam%nqibz) = qall(:,1:iparam%nqibz)
  deallocate(qall)

  do iq = 1, iparam%nqibz
    call reduceto1bz(qstruct%qibz(:,iq),qtemp,gtemp,latt)
    qstruct%qibz(:,iq) = qtemp(:)
  enddo

! add a small amount at q=0
  do iq = 1, iparam%nqibz
    if(is_small(qstruct%qibz(:,iq))) then
      qstruct%qibz(1,iq) = 0.000010
      qstruct%qibz(2,iq) = 0.000020
      qstruct%qibz(3,iq) = 0.000030
    endif
  enddo

  if(iverbose > 2) call uverbose('e','build_qibz')
  end
!!***

!{\src2tex{textfont=tt}}
!!****p* qpol_and_q/identq
!! NAME
!! identq
!!
!! FUNCTION
!! identify q-points in whole bz
!!  note: for q close to zero only one q-point is counted (ie not the
!!  rotations and reflections of the little q, nor other little
!!  q-points in the qibz set)
!! input:
!!  q: q-points in ibz
!!  op: symmetry operation matrices
!! output:
!!  qbz: q-points in whole bz, and
!!  qtab: a table giving for each the q-points
!!       in the ibz to which it is related, and
!!  qtabi: whether the wfn has to be complex-conjugated (-1 if so, 1 if not)
!!  qtabo: the symmetry operation that takes kibz to each kbz
!!  nqbz: no. of such q-points
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! qpol_and_q
!!
!! CHILDREN
!! is_small, dosym, icompv_fraction
!!
!! SOURCE

 subroutine identq(iparam,qstruct,symstr,nopq,ninvq)

 use dpdef
 use ggwdef
 use verbose

 implicit none

 type(dpparam) iparam 
 type(qstructure) qstruct
 type(symmetry) symstr
 integer, intent(in) :: nopq,ninvq

 integer i,j,iq,iqbz,iop,iinv,iold,istat
 integer nqbzmx,nqeq0
 real qnew(3)
 logical, external :: is_small
 integer, external :: icompv_fraction


if(iverbose > 2) call uverbose('b','identq')

 if(iverbose > 0) then 
  write(uout,*)'  identifying q-points in full Brillouin zone'
  write(uout,*)
 endif


 nqbzmx = iparam%nqibz * nopq * ninvq
 allocate(qstruct%qbz(3,nqbzmx),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 allocate(qstruct%qtab(nqbzmx),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 allocate(qstruct%qtabo(nqbzmx),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 allocate(qstruct%qtabi(nqbzmx),stat=istat)


!zero no. of q-points found
 qstruct%nqbz = 0

 nqeq0 = 0

!loop over q-points in ibz
 do iq = 1, iparam%nqibz

!  if q is close to zero, treat it differently as described above
   if (is_small(qstruct%qibz(1,iq))) then
     if (nqeq0.eq.0) then
       nqeq0 = 1
       qstruct%nqbz = qstruct%nqbz + 1
       if (qstruct%nqbz.gt.nqbzmx) then
           write(uout,*) '**error identq 1: nqbzmx too small;'
           write(uout,*) '                  increase nqbzmx!'
           stop
         endif
       qstruct%qbz(1,qstruct%nqbz) = qstruct%qibz(1,iq)
       qstruct%qbz(2,qstruct%nqbz) = qstruct%qibz(2,iq)
       qstruct%qbz(3,qstruct%nqbz) = qstruct%qibz(3,iq)
       qstruct%qtab(qstruct%nqbz) = iq
       qstruct%qtabo(qstruct%nqbz) = 1
       qstruct%qtabi(qstruct%nqbz) = 1
     endif
   else

!  loop over symmetry operations r and inversions i
   do iop = 1, nopq
     do  iinv = 1, ninvq
       if(symstr%symafm(iop)==-1) cycle 
!      form rik
       call dosym(symstr%op(1,1,iop),iinv,qstruct%qibz(1,iq),qnew(1))
!      check whether it has already been found (to within a rl vector)
       iold = 0
       do iqbz = 1, qstruct%nqbz
         iold = iold + icompv_fraction(qnew,qstruct%qbz(1,iqbz))
       enddo
       if (iold.eq.0) then
         qstruct%nqbz = qstruct%nqbz + 1
         if (qstruct%nqbz.gt.nqbzmx) then
           write(uout,*) '**error identq 2: nqbzmx too small;'
           write(uout,*) '                  increase nqbzmx!'
           stop
         endif
         qstruct%qbz(1,qstruct%nqbz) = qnew(1)
         qstruct%qbz(2,qstruct%nqbz) = qnew(2)
         qstruct%qbz(3,qstruct%nqbz) = qnew(3)
         qstruct%qtab(qstruct%nqbz) = iq
         qstruct%qtabo(qstruct%nqbz) = iop
         qstruct%qtabi(qstruct%nqbz) = 3 - 2 * iinv
       endif
      enddo
    enddo
   endif
 enddo


 if(iverbose > 1) then
!print out results
  write(uout,1000) iparam%nqibz, (j,(qstruct%qibz(i,j),i=1,3),j=1,iparam%nqibz)
1000  format (' q-points in irreducible wedge (ibz) ',i5,/&
              ' q-points [reciprocal lattice units]:'/&
               (i5,3f12.6))
  write(uout,2000) iparam%nopq, qstruct%nqbz, (j,(qstruct%qbz(i,j),i=1,3),j=1,qstruct%nqbz)
2000  format ( ' together with the',i5,' symmetry operations',&
               ' and inversion'/&
               ' have yielded',i7,' q-points in brillouin zone (bz):'/&
                (4(i5,3f5.2)))
  write(uout,*)
 endif


 if(iverbose > 2) call uverbose('e','identq')


 end
!!***

!{\src2tex{textfont=tt}}
!!****p* qpol_and_q/checkqpol
!! NAME
!! checkqpol
!!
!! FUNCTION
!! check if qpol given in input is good
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! qpol_and_q
!!
!! CHILDREN
!! is_small, icompv_fraction
!!
!! SOURCE

  subroutine checkqpol(iparam,qstruct)

  use verbose
  use dpdef
  use ggwdef
  implicit none
  type(dpparam) iparam
  type(qstructure) qstruct

  logical, external :: is_small
  integer, external :: icompv_fraction
  integer ifound, iq, iqp, i, ierr
  integer iqbz, iqibz, icount


  if(iverbose > 2) call uverbose('b','checkqpol')

! check if polarization vector qpol is allowed
  write(uout,*) 'check if polarization vector qpol given in input is allowed:'
  write(uout,*) '  must be qpol = k - kp - g0'
  ierr = 0
  do iq = 1, iparam%nqpol
    if(.not.iparam%KP_THEORY) then  ! no check for q->0 in k.p theory
      ifound = 0
      do iqp = 1, qstruct%nqbz
        if(icompv_fraction(iparam%qpol(:,iq),qstruct%qbz(:,iqp)).eq.1) ifound = 1
      enddo
      if(ifound.eq.0) then
        ierr = 1
        write(uout,*) '** error checkq 1: q-point not allowed'
        write(6,2150) (iparam%qpol(i,iq),i=1,3)
2150        format('                   q = ',3f12.6)
      endif
    endif
  enddo

  if(ierr.eq.1.or.iparam%nqpol.eq.0 ) then
   open(unit=67,file='chi.qpt',status='unknown',form='formatted')
   write(67,'(5x,i5,35x,i5)') iparam%nqibz, qstruct%nqbz
   do iqibz = 1,iparam%nqibz
    icount = 0
    do iqbz = 1,qstruct%nqbz
     if (qstruct%qtab(iqbz)==iqibz) then
      icount = icount + 1
      if(icount == 1) then
       write(67,'(3f12.6,4x,3f12.6)') qstruct%qibz(:,iqibz), qstruct%qbz(:,iqbz)
      else
       write(67,'(40x,3f12.6)') qstruct%qbz(:,iqbz)
      endif
     endif
    enddo
   enddo
   write(uout,*) 'q-points allowed written in chi.qpt'
   if (iparam%doublegrid) then
     if (iparam%antieps) write(uout,*) '** warning: remember to swap the kss-files when using the ''antieps'' option'
   else
     write(uout,*) '** warning: a small amount has been added at q=0'
   endif
   stop 'check chi.qpt for q-points allowed'
  endif
   
  write(uout,*) 'check ok'
  if(iverbose > 2) call uverbose('e','checkqpol')
  end
!!***

!{\src2tex{textfont=tt}}
!!****p* qpol_and_q/checkqgrid
!! NAME
!! checkqgrid
!!
!! FUNCTION
!! check if grid of q points is compatible with q points in screening
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! qpol_and_q
!!
!! TODO
!! need to be rewritten
!!
!! SOURCE


 subroutine checkqgrid(iparam,screen,qstruct)

  use verbose
  use dpdef
  use ggwdef
  implicit none

 type(dpparam) iparam
 type(qstructure) qstruct
 type(screening) screen

 integer :: iq,iqr,jq,g1,g2,g3
 real :: q(3),q2bz(3)


!!!!! TEMPORARY: now it is useless; need to take into account equivalent q points in screening

! check if qpoints in scr file (screen%qibz) correspond to the ones just calculated (qstruct%qibz)  
  write(uout,*) 'check if q points in screening file match q points found' 
  allocate(screen%iq(iparam%nqibz))
  do iqr =1,iparam%nqibz
   q(:) = screen%qibz(:,iqr)
   iq=0
   do jq = 1, qstruct%nqbz
    if(all(abs(q-qstruct%qbz(1:3,jq)) < 1.e-4)) iq = jq
   enddo
   if(iq .eq. 0) then
    do g1 = -1, 1
     do g2 = -1, 1
      do g3 = -1, 1
       q2bz(1) = q(1) + g1
       q2bz(2) = q(2) + g2
       q2bz(3) = q(3) + g3
       do jq = 1,  qstruct%nqbz
        if(all(abs(q2bz-qstruct%qbz(1:3,jq)) < 1.e-4)) iq = jq
       enddo
      enddo
     enddo
    enddo
   endif
   if(iq .eq. 0) then
    write(uout,1071) q(:)
1071  format ('**ERROR, checkqgrid: q point ',3f14.6,' is not in set of q BZ!')
      stop '**ERROR, checkqgrid: q-point in screening not in the set'
   else
    screen%iq(iqr) = qstruct%qtab(iq) 
    if(iverbose > 2) then
     write(uout,100) iq, qstruct%qtab(iq), qstruct%qbz(:,iq)
100  format('  q point, BZ number ',i5,'  IBZ number ',i5,'  r.l.c. ',3f6.3)
    endif
   endif
  enddo
  deallocate(screen%qibz)
  deallocate(screen%iq)
  write(uout,*) 'check ok'


 end
!!***
