!{\src2tex{textfont=tt}}
!!****p* main/quasiparticles
!! NAME
!! Read qps and gw files (quasiparticle wfns and energies)
!!!
!! FUNCTION
!! This is the interface between the main and the subroutines reading the qps or gw files 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main, correct_initial_states
!!
!! CHILDREN
!! memstat, rdqps, rdgw
!!
!! SOURCE

 subroutine quasiparticles(iparam,files,bzstruct,elecstr)

 use dpdef
 use ggwdef
 use verbose
 use interfaces
 
 implicit none
 type(dpparam) iparam
 type(dpfiles) files
 type(bzstructure) bzstruct
 type(electronic_structure) elecstr
 integer ib,ik,is, nbv, istat
 integer filegwver

 logical :: lfile
 integer :: nshift,ishf,ib1,ib2,ib1m,ib2m
 double precision :: bottomconduction,topvalence
  
 real,allocatable :: shift(:)
 double precision :: shf
 

#ifdef MPI
 include 'mpif.h'
 integer ierr,rang,nprocs
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif

 if(iverbose > 2) call uverbose('b','quasiparticles')

 allocate(elecstr%gwenergy(iparam%nkibz,iparam%nbnds,iparam%nsppol),stat=istat)
 if(istat.ne.0) stop 'out of memory'

 if(iverbose>2) then
  call memstat('gwenergy allocated =','c',iparam%nkibz,iparam%nbnds,iparam%nsppol)
 endif


! qps case: KS eigenvalues read in KSS from 1 to nbndmax (all needed for the commutator)
 elecstr%gwenergy(:,1:iparam%nbnds,:) = elecstr%energy(:,1:iparam%nbnds,:)
 
!allocate array for lifetimes and put it to 0 
 allocate(elecstr%imsigma(iparam%nkibz,iparam%nbnds,iparam%nsppol),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 elecstr%imsigma(:,:,:) = 0.0

!================
!case1: QPS file
 if(iparam%qps .or. iparam%qpsenergy) then  

!allocate matrix representing QP wfns on Kohn-Sham basis
!for each k point and spin: m_lda_to_qp(i,j) where
!i=index of  KS basis  and j= index of QP wfn: 
!psi_QP (j) = sum_i m_lda_to_qp(i,j) psi_LDA(i)

  if (iparam%LDA) then 
    iparam%nbndqpmin = 1
  else
    iparam%nbndqpmin = iparam%lomo
  endif   
    iparam%nbndqpmax = iparam%nbnds

  allocate(elecstr%m_lda_to_qp(1:iparam%nbndmax,iparam%nbndqpmin:iparam%nbndqpmax,iparam%nkibz,iparam%nsppol),stat=istat)
  if(istat.ne.0) stop 'm_lda_to_qp out of memory'

! Initialize with KS wfns
  elecstr%m_lda_to_qp(:,:,:,:)=(0.0,0.0)
  do ib=max(1,iparam%nbndqpmin),min(iparam%nbndmax,iparam%nbndqpmax)
    elecstr%m_lda_to_qp(ib,ib,:,:)= (1.0,0.0)
  enddo


!read from QPS file: m_lda_to_qp and gwenergy
  call rdqps(files%qps,files%ql,elecstr%gwenergy,elecstr%m_lda_to_qp,& 
             iparam%nkibz,bzstruct%kibz,iparam%nbndqpmin,iparam%nbndqpmax, &
             iparam%nbndmax,iparam%nsppol,iparam%mpilocal)


! since in this case only energies are effectively useful:
  if(iparam%qpsenergy .and. .not. iparam%qps) deallocate (elecstr%m_lda_to_qp) 

 endif

!================
!CASE2 : GW file

 if(.not.iparam%SO .and. (iparam%GW.or.iparam%lifetime)) then

!  read gw energies

! if filegwver = 5 (OK for Abinit 5.*.*)
!    elecstr%gwenergy from rdgw will contain quasiparticle energies
! if filegwver = 4  
!    elecstr%gwenergy from rdgw will contain only gw perturbative corrections 

   if(iparam%gwfilenew) then
     filegwver = 5
   else
     filegwver = 4
   endif
   if (filegwver == 4)  elecstr%gwenergy(:,:,:) = 0.0

   if (iparam%qps .and. filegwver == 4) then
    write(uout,*) 'WARNING: Old version of GW file not compatible with QPS file'
    write(uout,*) 'Put the flag ''gwfilenew'' in your input file'
   endif 

!in case of calculation with lifetimes  
!check the format of the file containing the imaginary part of qp energies
   if (iparam%lifetime .and. filegwver == 4) then
    write(uout,*) 'ERROR: The format for GW file does not contain the imagianry'
    write(uout,*) 'part of the self-energy'
    write(uout,*) 'calculation will stop'
    stop
   endif 

   call rdgw(files%gw,files%gl,iparam%nkibz,iparam%nkbz,iparam%nbnds,iparam%lomo,bzstruct%kbz,bzstruct%ktab, & 
             elecstr%gwenergy,elecstr%imsigma,iparam%nsppol,filegwver,iparam%mpilocal)


! in case of lifetime calculation without GW corrections we need to restore the
! correct energies
   if (iparam%lifetime.and.(.not.iparam%GW)) then
     elecstr%gwenergy(:,:,:)=elecstr%energy(:,:,:)
   endif

   if (filegwver == 4) then
   ! in this case gwenergy from rdgw are gw corrections; now build back gwenergy as qp energies...
    elecstr%gwenergy(:,:,:) = elecstr%energy(:,:,:) + elecstr%gwenergy(:,:,:)
   endif !filegwver

  endif ! GW
 
  if (.not.iparam%lifetime) deallocate(elecstr%imsigma)

!================
!CASE3 : scissor operator

  if(iparam%SO) then


   write(uout,*) 'Applying a scissor-operator correction to Kohn-Sham eigenvalues'
   write(uout,*)

!FIXME: use fermi/occopt
   if(((iparam%nel/2)*2) /= iparam%nel) then
     write(uout,*) 'Warning, odd number of electrons: check metallic system and scissor operator' 
   endif
   nbv = iparam%nel / 2
  
   do is=1,iparam%nsppol
    
    topvalence = maxval(elecstr%gwenergy(:,nbv,is))  
    do ib = iparam%lomo, nbv
      elecstr%gwenergy(:,ib,is) = elecstr%gwenergy(:,ib,is) + & 
              (iparam%stretchvalence-1.)*(elecstr%gwenergy(:,ib,is)-topvalence)
    enddo   

    bottomconduction=minval(elecstr%gwenergy(:,nbv+1,is))
    do ib = nbv+1, iparam%nbnds
      elecstr%gwenergy(:,ib,is) = elecstr%gwenergy(:,ib,is)  + iparam%soenergy +  &
                 (iparam%stretchconduction-1.) *(elecstr%gwenergy(:,ib,is)-bottomconduction) 
    enddo

   enddo
   
  endif !iparam%so


!================
!CASE4 : Multiscissor
  
  if(iparam%somult) then 
   write(uout,*) 'Applying a scissor-operator correction to Kohn-Sham eigenvalues'
   write(uout,*) 'Multiple scissor: looking for file somult.in'
   inquire(file='somult.in',exist=lfile)
   if (.not. lfile) stop 'file somult.in not found'
   allocate(shift(iparam%nbnds))
   shift(:) = 0.0
   open(33,file='somult.in', form='formatted',status='old')
   read(33,*) nshift
   do ishf = 1, nshift
    read(33,*) ib1m,ib2m, shf
    if (ib1m > ib2m) stop 'please correct file somult.in'
    ib1=max(1,ib1m)
    ib2=min(iparam%nbnds,ib2m)  
    do ib=ib1,ib2
     shift(ib) = dble(shf)
    enddo
   enddo
   close(33)
   write(uout,*) 'File somult.in read'
   write(uout,*) 

   if (iverbose > 2) then
    write(uout,*) 'ib, shift(ib) (eV)'
    do ib=iparam%lomo,iparam%nbnds 
     write(uout,'(i3,f7.3)') ib,shift(ib)
    enddo
   endif

   shift(:) =  shift(:) / HARTREE
   do is = 1, iparam%nsppol
    do ik = 1, iparam%nkibz
     do ib = 1, iparam%nbnds
       elecstr%gwenergy(ik,ib,is) = elecstr%gwenergy(ik,ib,is) + shift(ib)
     enddo
    enddo
   enddo 

   deallocate(shift)
  endif !iparam%somult

!=================

!printing out new energies
  if (iparam%GW .or. iparam%QPS .or. iparam%qpsenergy .or. iparam%SO .or. iparam%somult) then

    if (iverbose > 0) then
     write(uout,*) 'QP energies after correction:'
     do is=1,iparam%nsppol
       if (iparam%nsppol==2) write(uout,*) 'spin component', is
       write(uout,*) ' k        Re(E_qp) [eV]'
       do ik = 1, iparam%nkibz
        write(uout,'(i3,7x,10f7.2/50(10x,10f7.2/))') &
&          ik, (HARTREE*elecstr%gwenergy(ik,ib,is),ib=1,iparam%nbnds)
       enddo
       write(uout,*)
     enddo !is
    endif 

  endif

#ifdef MPI
if(iparam%mpilocal) then
 call mpi_barrier(mpi_comm_world,ierr)
 call mpi_bcast(iparam%homo,1,mpi_integer,0,mpi_comm_world,ierr)
 call mpi_bcast(iparam%lumo,1,mpi_integer,0,mpi_comm_world,ierr)
 call mpi_bcast(elecstr%gwenergy,iparam%nkibz*iparam%nbnds*iparam%nsppol,mpi_double_precision,0,mpi_comm_world,ierr)
 if(iparam%lifetime) &
 call mpi_bcast(elecstr%imsigma,iparam%nkibz*iparam%nbnds*iparam%nsppol,mpi_double_precision,0,mpi_comm_world,ierr)
 if (iparam%qps) then
   do ib=1,iparam%nbndmax
    do ib2=iparam%nbndqpmin,iparam%nbndqpmax
     call mpi_bcast(elecstr%m_lda_to_qp(ib,ib2,:,:),iparam%nkibz*iparam%nsppol,mpi_complex,0,mpi_comm_world,ierr)  
    enddo
   enddo
 endif
endif
#endif

!commented since needed for input of calccomm should be deallocated somewhere else
! if(.not.iparam%exciton .and. .not. iparam%FXCMT .and. .not. iparam%DRUDE .and. .not. iparam%ONLYDRUDE ) deallocate(bzstruct%kbz)


 if(iverbose > 2) call uverbose('e','quasiparticles')

 
 end
!!***

