
!{\src2tex{textfont=tt}}
!!****p* Multi/rho_tw_g
!! NAME
!! rho_tw_g
!!
!! FUNCTION
!! Calculate the single oscillator strength
!!               
!!  rhotwg(G) =  < vk' | exp(-i*(q+G)*r) | ck >,   k' = k - q - G0
!!
!! with the Bloch wavefunctions |vk'> = u_vk'(r) exp(ik'r) we get
!! 
!!  rhotwg(G) =  Int dr  u*_vk'(r) u_ck(r) exp[-i(G-G0)r]
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! create_rhotw,create_rhotwt,create_rhotwq0,create_rhxtwg_coul, prep_rhotw, cexch_haydock
!!
!! CHILDREN
!! c3dfft
!!
!! SOURCE

  subroutine rho_tw_g(npwvec,nr,nrb,nfft1a,nfft1,nfft2,nfft3,igfftg0,wfn1,i1,ktabr1,wfn2,i2,ktabr2,menoplan,rhotwg)

  use dpdef
#ifdef OMP
!$ use OMP_LIB
#endif

  implicit none
  integer nr, npwvec
  integer nrb
  integer i1, i2
  integer nfft1,nfft1a,nfft2,nfft3
  integer igfftg0(npwvec)
  integer ktabr1(nr) ! tables R^-1 r for that k points
  integer ktabr2(nr)
  complex wfn1(nr)
  complex wfn2(nr)
  complex rhotwg(npwvec)
  complex rhotw(nr)
  integer ir
 integer rang
! to account for symmetries
! phi_Rk(r) = phi_k(R^-1 r)
! phi_Ik(r) = phi_k*(r)
! form rho-twiddle(r)
 integer(ILONG) :: menoplan
  
  if(i1 == 1) then
    if(i2 == 1) then
      do ir = 1, nr 
        rhotw(ir) = conjg(wfn1(ktabr1(ir))) * wfn2(ktabr2(ir))
      enddo
    else 
      do ir = 1, nr 
        rhotw(ir) = conjg(wfn1(ktabr1(ir)) * wfn2(ktabr2(ir)))
      enddo
    endif
  else
    if(i2 == 1) then
      do ir = 1, nr 
        rhotw(ir) = wfn1(ktabr1(ir)) * wfn2(ktabr2(ir))
      enddo
    else 
      do ir = 1, nr 
        rhotw(ir) = wfn1(ktabr1(ir)) * conjg(wfn2(ktabr2(ir)))
      enddo
    endif
  endif
! FFT to rho-twiddle (G)
  call c3dfft(rhotw,nfft1a,nfft2,nfft3,nfft1,nfft2,nfft3,-1,menoplan)
! Set up rho-twiddle(G-G0) for matrix in PW order, normalising for FFT
  rhotwg(:) = rhotw(igfftg0(:)) / nrb
  end

!!***

!{\src2tex{textfont=tt}}
!!****p* fxcterm/wfprod
!! NAME
!! wfprod
!!
!! FUNCTION
!! Calculate wave-functions products
!!  to account for symmetries
!!  phi_Rk(r) = phi_k(R^-1 r)
!!  phi_Ik(r) = phi_k*(r)
!!  form rho-twiddle(r)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! fxcterm
!!
!! SOURCE
  subroutine wfprod(nr,wfn1,i1,ktabr1,wfn2,i2,ktabr2,wprod)

  implicit none
  integer nr
  integer i1, i2
  integer ktabr1(nr) ! tables R^-1 r for that k points
  integer ktabr2(nr)
  complex wfn1(nr)
  complex wfn2(nr)
  complex wprod(nr)
  integer ir
  
  if(i2 == 1) then
    if(i1 == 1) then
      do ir = 1, nr
        wprod(ir) = conjg(wfn2(ktabr2(ir))) * wfn1(ktabr1(ir))
      enddo
    else
      do ir = 1, nr
        wprod(ir) = conjg(wfn2(ktabr2(ir)) * wfn1(ktabr1(ir)))
      enddo
    endif
  else
    if(i1 == 1) then
      do ir = 1, nr
        wprod(ir) = wfn2(ktabr2(ir)) * wfn1(ktabr1(ir))
      enddo
    else
      do ir = 1, nr
        wprod(ir) = wfn2(ktabr2(ir)) * conjg(wfn1(ktabr1(ir)))
      enddo
    endif
  endif

  end
!!***
