!{\src2tex{textfont=tt}}
!!****p* main/estimate_time_chi0
!! NAME
!! estimate_time_chi0
!!
!! FUNCTION
!! Performs a time estimation for creation_chi0.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! NOTES
!!  The idea is to call creation_chi0 with a very small number of transitions.
!!  Therefore, all necessary quantities have to be set properly, and the total
!!  number of transitions must be correct (calculated in prep_trans(), needs KS-energies)
!!  
!! BUGS
!!  The testrun does only work for DP (not EXC) in serial mode.
!!  Eventually speed up the testrun by skipping parts of the dp-calculation prior
!!  to creation_chi0 (fft, commutator, fxc, ... ?)
!!
!! PARENTS
!!  dp
!!
!! CHILDREN
!! creation_chi0
!!
!! SOURCE

subroutine estimate_time_chi0(iparam,bzstruct,bzstructp,tablekp,elecstr,elecstrp,&
                              res,latt,Wd,trans,tr,fftmesh,coul)
  use ggwdef
  use dpdef
  use verbose
  implicit none

  type(dpparam) iparam
  type(transition) trans(iparam%nt)
  type(transgen) tr
  type(response) res
  type(bzstructure) bzstruct, bzstructp
  type(polarizationq) tablekp
  type(electronic_structure) elecstr, elecstrp
  type(lattstruct) latt
  type(mesh) fftmesh
  type(coulomb) coul
  double precision Wd
   
  type(dpparam) iparam_test
  real ct_start, wt_start, ct_end, wt_end
  real ct_chi0, wt_chi0
  real old,new


  if (.not.iparam%testrun) return

  if(iverbose > 2) call uverbose('b','estimate_time_chi0')

  ! protect original structures from changes
  iparam_test=iparam;       iparam_test%testrun=.true.
  iverbose=0;


  ! testrun for chi0 with reduced number of transitions iparam_test%nt
  iparam_test%nt=1;
  old=-100.
  do 
    
    call get_time(ct_start, wt_start)
    call creation_chi0(iparam_test,bzstruct,bzstructp,tablekp,elecstr,elecstrp,&
                       res,latt,Wd,trans,tr,fftmesh,coul)
    call get_time(ct_end, wt_end)
    deallocate(res%chi0, res%lwing, res%uwing)

    ct_chi0 = (ct_end-ct_start) / iparam_test%nt * iparam%nt
    wt_chi0 = (wt_end-wt_start) / iparam_test%nt * iparam%nt
    if (iverbose>2) then
      write(uout,*) "nt=", iparam_test%nt, ", cputime:  ", ct_chi0
      write(uout,*) "nt=", iparam_test%nt, ", walltime: ", wt_chi0
    endif

    iparam_test%nt = iparam_test%nt * 2
    if (iparam_test%nt > iparam%nt) exit
! either use this criterium 
    new=ct_chi0
      write(uout,*) "new, old  ",new, old,abs(new-old)
    if (abs(new-old)<1) then
      exit
    else
      old=new
    endif
! or this other one
!    if (ct_end-ct_start > 3) exit
  enddo

  write(uout,*) " Estimated calculation time for chi0: "
  write(uout,*) "     cputime: ", int(ct_chi0/36)/100.0, " hours"
  write(uout,*) "    walltime: ", int(wt_chi0/36)/100.0, " hours"


  ! memory check
  call estimate_memory(iparam,fftmesh)
  if(iverbose > 2) call uverbose('e','estimate_time_chi0')
  stop
end subroutine estimate_time_chi0
!!***


!{\src2tex{textfont=tt}}
!!****p* main/estimate_memory
!! NAME
!! estimate_memory
!!
!! FUNCTION
!! Estimation of memory needed for DP-run.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! NOTES
!!  - Check, that at the point of calling this function, all parameters
!!    are already properly set (at least after preptrans() call).
!!  - The check does only work for DP (not EXC) in serial mode.
!!  - The calls of memory_count() could be also performed at the point of
!!    allocation.
!!
!! BUGS
!!  memory always underestimated by a small offset (neglected contributions)
!!  eventually add an overhead of ~5% ?
!!  vkb() contribution missing ?
!!
!! PARENTS
!!  estimate_time_chi0
!!
!! CHILDREN
!!  mem_count
!!
!! SOURCE
subroutine estimate_memory(iparam,fftmesh)
  use ggwdef
  use dpdef
  use verbose
  implicit none

  type(dpparam) iparam
  type(mesh) fftmesh
 
  real mem_wfg, mem_wfr, mem_wf, mem_chi0, mem_ktabr, mem_comm, mem_trans, mem_eps, mem_tot
  integer          idum,nt, doublegrid 
  ! avoid use of non-standard intrinsic function sizeof() 
  integer, parameter :: sizeof_int    = bit_size(idum)/8
  integer, parameter :: sizeof_real   = selected_real_kind(6)
  integer, parameter :: sizeof_double = selected_real_kind(12)
  integer, parameter :: sizeof_complex= 2*sizeof_real
  integer, parameter :: sizeof_transition = 4*sizeof_int+2*sizeof_double+sizeof_real
#ifdef MPI
 include 'mpif.h'
 integer ierr,rang,nprocs,startcycle,stopcycle
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
 nt = iparam%nt/nprocs + 1
#else 
  nt = iparam%nt
#endif

  ! factor for doublegrid calculation
  doublegrid = 1
  if (iparam%doublegrid) doublegrid  = 2

  write(uout,*) ""
  write(uout,*) " Estimated memory usage: "

  ! readstructure()
  mem_wfg = sizeof_complex*iparam%npwwfn*(iparam%nbnds-iparam%nbndmin+1) &
              *iparam%nkibz*iparam%nsppol*doublegrid ! wavefunction in G-space
  call mem_count('wfk(g) ~ pw*nk*nb',mem_wfg,mem_tot)
  mem_wf  = mem_wfg

  ! identk()
  mem_ktabr = sizeof_int*fftmesh%nr*iparam%nkibz*iparam%nop*2*doublegrid 
  call mem_count('ktabr  ~    nr*nk',mem_ktabr,mem_tot)

  ! preptrans()
  mem_trans =  sizeof_transition*iparam%nt+ & ! trans + transtab
               sizeof_int*iparam%nkbz*(iparam%nbnds-iparam%lomo+1) &
                                    *(iparam%nbnds-iparam%lomo+1)*iparam%nsppol
  call mem_count('trans  ~       nt',mem_trans,mem_tot)

  ! calccomm()
  mem_comm = 0
  if(.not.iparam%JDOS .and. iparam%KP_THEORY) then
    mem_comm = sizeof_complex*nt*4 + &         ! rhotwg0 + rhotwxgen
               sizeof_real   *nt*10            ! qrotgen + opinvgen
    call mem_count('commutator   ~ nt',mem_comm,mem_tot)
  endif

  ! fftwfn()  
  if (.not.iparam%savememory) then
    mem_wfr = sizeof_complex*fftmesh%nr*(iparam%nbnds-iparam%nbndmin+1) &
                *iparam%nkibz*iparam%nsppol*doublegrid ! wavefunction in R-space
    mem_wf  = mem_wfr
    call mem_count('wfk(r) ~ nr*nk*nb',mem_wfr,mem_tot)
    call mem_count('',-mem_wfg,mem_tot)              ! deallocate wfg
  endif

  ! creation_chi0()
  mem_chi0=sizeof_complex*iparam%npwmat*iparam%npwmat*iparam%nomega*iparam%nsppol & ! body
       + 2*sizeof_complex*iparam%npwmat*iparam%nomega*iparam%nqpol *iparam%nsppol   ! wings
  call mem_count('chi0   ~ pw*nE*pm',mem_chi0,mem_tot)
  call mem_count('',-mem_comm -mem_wf -mem_ktabr,mem_tot) ! deallocations

  ! creation_epsilon()
  mem_eps=sizeof_complex*iparam%nomega*(iparam%nqpol+2)*iparam%nsppol*2  ! epsmlf
  call mem_count('eps    ~       nE',mem_eps,mem_tot)

  ! total memory (not just the sum of all - accounts for deallocates)
  write(uout,*) "   estimated memory usage: ", int(mem_tot/1048576), " Mb" 

end subroutine estimate_memory
!!***


!{\src2tex{textfont=tt}}
!!****p* main/mem_count
!! NAME
!! mem_count
!!
!! FUNCTION
!! Counting amount of estimated total memory
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! NOTES
!!  - if new memory is allocated, mem_count should be called with
!!    the estimated number of bytes (positive)
!!  - if memory is deallocated, a negative number should be used
!!  - if <label> is used, the amount of memory added will be printed
!!
!! PARENTS
!!  estimate_memory
!!
!! CHILDREN
!!
!! SOURCE
subroutine mem_count(label, add_mem, ret)
  use dpdef
  implicit none

  character(len=*), intent(in) :: label
  real,             intent(in) :: add_mem
  real,            intent(out) :: ret
  
  real, save :: mem_current = 0.
  real, save :: mem_max     = 0.

  if (label /= '')  write(uout,'(a22,":",2x,i6," Mb")') label, int(add_mem/1048576)
  mem_current = mem_current + add_mem
  if (mem_current > mem_max) mem_max = mem_current

  ret = mem_max
end subroutine mem_count
!!***
