!{\src2tex{textfont=tt}}
!!****p* calccomm/cfnl
!! NAME
!! cfnl
!!
!! FUNCTION
!! compute Vnl(K) and grad_K Vnl(K)
!! three reciprocal lattice units components
!! input:
!!   vkb and vkbd kleinman-bylander pseudopotential in separable form
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! calccomm, calcomm_drude
!!
!! CHILDREN
!! ylm, ylmd
!!
!! SOURCE

  subroutine cfnl(iparam,latt,trspace,bzstruct,kb,chemstr)

  use ggwdef
  use dpdef
  use verbose
  use interfaces
  implicit none
  type(kb_potential) kb
  type(bzstructure) bzstruct
  type(transposed_space) trspace
  type(lattstruct) latt
  type(chemistry) chemstr
  type (dpparam) iparam

! Kleiman-Bylander non-local separable form: structure factor and first derivative      
! example: fnl(ig,im+(il-1)*(il-1),is,ik)
! example: fnld (i,ig,im+(il-1)*(il-1),is,ik)
!  complex  :: fnl(npwwfn,mpsang*mpsang,natom,nkibz),
! &            fnld(3,npwwfn,mpsang*mpsang,natom,nkibz)

  complex (kind(0.d0)) :: sfac
  complex (kind(0.d0)) :: dylmcart(3), dylmcrys(3), dth, dphi,gradth(3), gradphi(3)
  real (kind(0.d0)) :: costh, sinth, cosphi, sinphi, sq
  real (kind(0.d0)) :: xdotg

! kg = k + G in crystallographic coordinates
  real (kind(0.d0)) :: kg(3)
  real (kind(0.d0)) :: mkg, mkg2
  real (kind(0.d0)) :: pig, factor
  integer lmax

  integer i, ig, ik, ia, is, il, im, iml, idum
  real (kind(0.d0)) :: kcart(3), gcart(3)
  integer istat

  complex, external :: ylm

! spherical harmonics and their first derivatives 
  integer, parameter :: nlx = 4
#ifdef MPI
  include 'mpif.h'
  integer ierr,rang,nprocs
  real starttime, endtime
  call mpi_comm_size(mpi_comm_world,nprocs,ierr)
  call mpi_comm_rank(mpi_comm_world,rang,ierr)
  starttime = mpi_wtime()
#endif

  if(iverbose > 2) call uverbose('b','cfnl')
  
  if(iverbose>0) then
  write(uout,*)'  limit q->0,including term <n,k|[Vnl,iqr]|n"k>'
  endif


  pig = dacos(-1.d0)
 
  if(iparam%DRUDE .or. iparam%ONLYDRUDE) then
   allocate(kb%fnl(iparam%npwvec,kb%mpsang*kb%mpsang,kb%natom,iparam%nkibz),stat=istat)
  else
   allocate(kb%fnl(iparam%npwvec,kb%mpsang*kb%mpsang,kb%natom,kb%kdim),stat=istat)
  endif
  if(istat/=0) stop 'out of memory fnl'
  if(iparam%DRUDE .or. iparam%ONLYDRUDE) then
   allocate(kb%fnld(3,iparam%npwvec,kb%mpsang*kb%mpsang,kb%natom,iparam%nkibz),stat=istat)
  else
   allocate(kb%fnld(3,iparam%npwvec,kb%mpsang*kb%mpsang,kb%natom,kb%kdim),stat=istat)
  endif
  if(istat/=0) stop 'out of memory fnld'
  
  if(iverbose > 2) then
    call memstat('fnl and fnld allocated =','c',4,iparam%npwvec,kb%mpsang*kb%mpsang,kb%natom,kb%kdim)
  endif
  lmax = kb%mpsang
  if(kb%mpsang>nlx) then
    call mesg('warning','too many angular components (only s p d f are programmed)',idum,idum)
    lmax = nlx
  endif

  if(iverbose > 2) then
    write(uout,*) 'mpsang ',  kb%mpsang
    write(uout,*) 'ntype ', kb%ntype, 'natom ', kb%natom
    write(uout,*) 'type ', chemstr%typeat
    write(uout,*) 'vkbsign ', kb%vkbsign
    write(uout,*) 'vkb(1:3,1,1,1) ', kb%vkb(1:3,1,1,1)
    write(uout,*) 'vkbd(1:3,1,1,1) ', kb%vkbd(1:3,1,1,1)
  endif

  do ik = 1, iparam%nkibz
   if (.not.iparam%DRUDE .and. .not.iparam%ONLYDRUDE) then
    if (kb%k_count(ik) == 0) cycle
   endif
   do ig = 1, iparam%npwvec
    kg(:) = bzstruct%kibz(:,ik) + real(trspace%gvec(:,ig))
    kcart(:) = kg(1)*latt%b1(:)+kg(2)*latt%b2(:)+kg(3)*latt%b3(:)
!   the next to solve the problem with sinth=0. or sinphi=0.
    if(dabs(kcart(2))<1.d-8) kcart(2) = kcart(2)+1.d-8
    mkg2 = kcart(1)**2+kcart(2)**2+kcart(3)**2
    mkg = dsqrt(mkg2)
    sq = dsqrt(kcart(1)**2+kcart(2)**2)

    gcart(:) = real(trspace%gvec(1,ig))*latt%b1(:)+ &
               real(trspace%gvec(2,ig))*latt%b2(:)+ & 
               real(trspace%gvec(3,ig))*latt%b3(:)

! (th,phi) spherical coordinates
    costh = kcart(3)/mkg
    sinth = sq/mkg
    cosphi = kcart(1)/sq
    sinphi = kcart(2)/sq
    gradth(1) = kcart(1)*kcart(3)/mkg**3/sinth
    gradth(2) = kcart(2)*kcart(3)/mkg**3/sinth
    gradth(3) = -(1.d0/mkg-kcart(3)**2/mkg**3)/sinth
    gradphi(1) = -(1.d0/sq - kcart(1)**2/sq**3)/sinphi
    gradphi(2) = kcart(2)*kcart(1)/sq**3/sinphi
    gradphi(3) = (0.d0,0.d0)

    do ia = 1, kb%natom
     is=chemstr%typeat(ia)
     xdotg = gcart(1)*chemstr%xcart(1,ia)+ &
             gcart(2)*chemstr%xcart(2,ia)+ & 
             gcart(3)*chemstr%xcart(3,ia)
     sfac = cmplx(dcos(xdotg),dsin(xdotg),kind(0.d0)) 

     do il = 1, lmax
      factor = dsqrt(4.d0*pig/real(2*(il-1)+1,kind(0.d0)))
      do im= 1, 2*(il-1)+1
!  index on im and il
        iml = im+(il-1)*(il-1)

!  first k-b factor
        if(iparam%DRUDE .or. iparam%ONLYDRUDE)then
          kb%fnl(ig,iml,ia,ik) = factor * sfac * ylm(im-il,il-1,kcart) * & 
                                   kb%vkb(ig,is,il,ik) * kb%vkbsign(il,is)
        else
          kb%fnl(ig,iml,ia,kb%k_count(ik)) = factor * sfac * ylm(im-il,il-1,kcart) * & 
                                   kb%vkb(ig,is,il,ik) * kb%vkbsign(il,is)
        endif
!  second k-b factor (involving first derivatives)
        call ylmd(im-il,il-1,kcart,dth,dphi)
!  dYlm/dK = dYlm/dth * grad_K th + dYlm/dphi + grad_K phi
        dylmcart(:) = dth * gradth(:) + dphi * gradphi(:)
!  cartesian to crystallographic axis
!december 2009 Marco Cazzaniga, Giovanni Onida and Nicola Manini
!the transformation matrix A=(a1,a2,a3) acts *on its left* on the
!covariant vector dylmcart (a *row* vector), hence:
dylmcrys(1)=(latt%a1(1)*dylmcart(1)+latt%a1(2)*dylmcart(2)+latt%a1(3)*dylmcart(3))/(2.d0*pig)
dylmcrys(2)=(latt%a2(1)*dylmcart(1)+latt%a2(2)*dylmcart(2)+latt%a2(3)*dylmcart(3))/(2.d0*pig)
dylmcrys(3)=(latt%a3(1)*dylmcart(1)+latt%a3(2)*dylmcart(2)+latt%a3(3)*dylmcart(3))/(2.d0*pig)
!the previous (wrong) implementation assumed A acting on its right on a column vector
!dylmcrys(:) = (dylmcart(1)*latt%a1(:)+dylmcart(2)*latt%a2(:)+dylmcart(3)*latt%a3(:))/(2.d0*pig)
!yielding wrong results for the non local contributions to the spectra,
!such as a spurous anisotropy in isotropic systems
        do i = 1, 3
         if(iparam%DRUDE .or. iparam%ONLYDRUDE)then
          kb%fnld(i,ig,iml,ia,ik) = factor * sfac*( kg(i)/mkg * ylm(im-il,il-1,kcart) * & 
                                  kb%vkbd(ig,is,il,ik) + dylmcrys(i) * kb%vkb(ig,is,il,ik) )
         else
          kb%fnld(i,ig,iml,ia,kb%k_count(ik)) = factor * sfac*( kg(i)/mkg * ylm(im-il,il-1,kcart) * & 
                                  kb%vkbd(ig,is,il,ik) + dylmcrys(i) * kb%vkb(ig,is,il,ik) )
         endif
        enddo ! i


      enddo ! im
     enddo ! il
    enddo ! ia
   enddo ! ig
  enddo ! ik

 if(iverbose > 2) call uverbose('e','cfnl')

  end
!!***


!{\src2tex{textfont=tt}}
!!****f* cfnl/ylm
!! NAME
!! ylm
!!
!! FUNCTION
!! This function gives all spherical harmonics for il<=2
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! cfnl
!!
!! SOURCE

  complex function ylm(im,il,kcart)

  implicit none
! input variables 
  integer im, il
  real (kind(0.d0)) :: kcart(3)
! local variables
  real (kind(0.d0)) :: pig
  real (kind(0.d0)) :: costh, sinth, cosphi, sinphi, costwophi, sintwophi, costhreephi, sinthreephi

  pig = dacos(-1.d0)

! (th,phi) spherical coordinates
  costh = kcart(3)/dsqrt(kcart(1)**2+kcart(2)**2+kcart(3)**2)
  sinth = dsqrt(kcart(1)**2+kcart(2)**2)/dsqrt(kcart(1)**2+kcart(2)**2+kcart(3)**2)
  cosphi = kcart(1)/dsqrt(kcart(1)**2+kcart(2)**2)
  sinphi = kcart(2)/dsqrt(kcart(1)**2+kcart(2)**2)
  costwophi = 2.d0*cosphi**2 - 1.d0
  sintwophi = 2.d0*sinphi*cosphi
  costhreephi = cosphi*costwophi-sinphi*sintwophi
  sinthreephi = cosphi*sintwophi+sinphi*costwophi

  if (il.eq.0) then
   if (abs(im).eq.0) then
    ylm = 1.d0 / dsqrt(4.d0*pig)
   else
    stop 'Bug ylm'
   endif
  elseif (il.eq.1) then
   if (abs(im).eq.0) then
    ylm = dsqrt(3.d0/(4.d0*pig))*costh
   elseif (abs(im).eq.1) then
    ylm = -dsqrt(3.d0/(8.d0*pig))*sinth*cmplx(cosphi,sinphi)
   else
    stop 'Bug ylm'
   endif
  elseif (il.eq.2) then
   if (abs(im).eq.0) then
    ylm = dsqrt(5.d0/(16.d0*pig))*(3.d0*costh**2-1.d0)
   elseif (abs(im).eq.1) then
    ylm = -dsqrt(15.d0/(8.d0*pig))*sinth*costh*cmplx(cosphi,sinphi)
   elseif (abs(im).eq.2) then
    ylm = dsqrt(15.d0/(32.d0*pig))*(sinth)**2*cmplx(costwophi,sintwophi)
   else
    stop 'Bug ylm'
   endif
  elseif (il.eq.3) then 
   if (abs(im).eq.0) then
    ylm= sqrt(7.d0/(16.d0*pig))*(5.d0*costh**3 -3.d0*costh)
   else if (abs(im).eq.1) then
    ylm= -sqrt(21.d0/(64.d0*pig))*sinth*(5.d0*costh**2-1.d0)*cmplx(cosphi,sinphi)
   else if (abs(im).eq.2) then
    ylm= sqrt(105.d0/(32.d0*pig))*sinth**2*costh*cmplx(costwophi,sintwophi)
   else if (abs(im).eq.3) then
    ylm=-sqrt(35.d0/(64.d0*pig))*sinth**3*cmplx(costhreephi,sinthreephi)
   end if
  else
   stop 'Ylm not yet implemented'
  endif

! treats the case im < 0
  if (im.lt.0) then
   ylm=(-1.d0)**(im)*conjg(ylm)
  endif

  end function ylm
!!***

!{\src2tex{textfont=tt}}
!!****p* cfnl/ylmd
!! NAME
!! ylmd
!!
!! FUNCTION
!! This subroutine computes dth and dphi, the first derivatives of
!! Ylm as a function of th and phi (the angles of the spherical coordinates)
!! It works for all spherical harmonics with il <=2
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! cfnl
!!
!! SOURCE

  subroutine ylmd(im,il,kcart,dth,dphi)

  implicit none

! input variables
  integer im, il
  real (kind(0.d0)) :: kcart(3)
! output variable
  complex (kind(0.d0)) :: dth, dphi
! local variables
  real (kind(0.d0)) :: pig
  real (kind(0.d0)) :: costh, sinth, cosphi, sinphi, costwophi, sintwophi, costhreephi, sinthreephi

  pig = dacos(-1.d0)
! (th,ph) spherical representation.
  costh = kcart(3)/dsqrt(kcart(1)**2+kcart(2)**2+kcart(3)**2)
  sinth = dsqrt(kcart(1)**2+kcart(2)**2)/dsqrt(kcart(1)**2+kcart(2)**2+kcart(3)**2)
  cosphi = kcart(1)/dsqrt(kcart(1)**2+kcart(2)**2)
  sinphi = kcart(2)/dsqrt(kcart(1)**2+kcart(2)**2)
  costwophi = 2.d0*cosphi**2 - 1.d0
  sintwophi = 2.d0*sinphi*cosphi
  costhreephi = cosphi*costwophi-sinphi*sintwophi
  sinthreephi = cosphi*sintwophi+sinphi*costwophi 

  if (il.eq.0) then
   if (abs(im).eq.0) then
    dth  = (0.d0,0.d0)
    dphi = (0.d0,0.d0)
   else
    stop 'Bug ylm'
   endif
  elseif (il.eq.1) then
   if (abs(im).eq.0) then
    dth  = -dsqrt(3.d0/(4.d0*pig))*sinth
    dphi = (0.d0,0.d0)
   elseif (abs(im).eq.1) then
    dth  = -dsqrt(3.d0/(8.d0*pig))*costh *cmplx(cosphi,sinphi)
    dphi = -dsqrt(3.d0/(8.d0*pig))*sinth *cmplx(-sinphi,cosphi)
   else
    stop 'Bug ylm'
   endif
  elseif (il.eq.2) then
   if (abs(im).eq.0) then
    dth  = -dsqrt(5.d0/(16.d0*pig))*6.d0*costh*sinth
    dphi = (0.d0,0.d0)
   elseif (abs(im).eq.1) then
    dth  = -dsqrt(15.d0/(8.d0*pig))*(costh**2-sinth**2)*cmplx(cosphi,sinphi)
    dphi = -dsqrt(15.d0/(8.d0*pig))*costh*sinth*(0.d0,1.d0)*cmplx(cosphi,sinphi)
   elseif (abs(im).eq.2) then
    dth  = dsqrt(15.d0/(32.d0*pig))*2.d0*costh*sinth*cmplx(costwophi,sintwophi)
    dphi = dsqrt(15.d0/(32.d0*pig))*sinth**2*(0.d0,2.d0)*cmplx(costwophi,sintwophi)
   else
    stop 'Bug ylm'
   endif
  elseif (il.eq.3) then
   if (abs(im).eq.0) then
    dth = sqrt(7.d0/(16*pig))*(-15.d0*costh**2*sinth + 3.d0**sinth)
    dphi= (0.d0,0.d0)
   else if (abs(im).eq.1) then
    dth= -sqrt(21.d0/(64.d0*pig))*cmplx(cosphi,sinphi)*(5.d0*costh**3-costh-10.d0*sinth**2*costh)
    dphi=-sqrt(21.d0/(64.d0*pig))*sinth*(5.d0*costh**2-1)*(0.d0,1.d0)*cmplx(cosphi,sinphi)
   else if (abs(im).eq.2) then
    dth =sqrt(105.d0/(32.d0*pig))*(2.d0*sinth*costh**2-sinth**3)*cmplx(costwophi,sintwophi)
    dphi=sqrt(105.d0/(32*pig))*sinth**2*costh*(0.d0,2.d0)*cmplx(costwophi,sintwophi)
   else if (abs(im).eq.3) then
    dth =-sqrt(35.d0/(64.d0*pig))*3.d0*sinth**2*costh*cmplx(costhreephi,sinthreephi)
    dphi= sqrt(35.d0/(64.d0*pig))*sinth**3*(0.d0,3.d0)*cmplx(costhreephi,sinthreephi)
   else
    stop 'Bug ylm'
   end if
 
  endif


! treats the case im < 0
  if (im.lt.0) then
   dth  = (-1.d0)**(im)*conjg(dth)
   dphi = (-1.d0)**(im)*conjg(dphi)
  endif

  end subroutine ylmd
!!***

