!{\src2tex{textfont=tt}}
!!****p* main/ckxclda
!! NAME
!! ckxclda
!!
!! FUNCTION
!! Calculate the lda fxc (Ceperley-Alder parametrization) in GG' space
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! modx2,ckxcldar,difvxcspin,c3dfft,printcm
!!
!! SOURCE

  subroutine ckxclda(iparam,fftmesh,lda,trspace)

  use verbose
  use dpdef
  use ggwdef
  implicit none
  type(dpparam) iparam
  type(transposed_space) trspace
  type(ldakernel) lda
  type(mesh) fftmesh 

  complex,allocatable :: Kxcldagmg(:,:)
  integer ig2,ig3,ig4,ig4x,ig4y,ig4z,istat
  integer isppol,jsppol
  integer, external :: modx2
  real fftfct
  integer nkxc,ikxc
  character (len=6) :: s1,s2

#ifdef MPI
  include 'mpif.h'
  integer ierr,rang,nprocs
  call mpi_comm_size(mpi_comm_world,nprocs,ierr)
  call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif

  if(iverbose > 2) call uverbose('b','ckxclda')
  if(iverbose > 0) then
   headline="TDLDA kernel"
   call print_headline()
  endif

 if (iparam%nsppol == 1) then
  nkxc = 1
 else
  nkxc = 3
 endif
 allocate(Kxcldagmg(fftmesh%nr,nkxc))

 if(iparam%exciton) then
  allocate(lda%fxcldar(fftmesh%nr,iparam%nsppol,iparam%nsppol),stat=istat)
  if(istat.ne.0) stop 'out of memory'
 else
   allocate(lda%fxclda(iparam%npwmat,iparam%npwmat,iparam%nsppol,iparam%nsppol),stat=istat)
   if(istat.ne.0) stop 'out of memory'
 endif  

#ifdef MPI
!if mpilocal does only if rang = 0 (=> only condition for not going on: mpilocal=.true. .and. rang/=0)
  if (.not.(iparam%mpilocal .and. rang /= 0))  then 
#endif


 
 if(iverbose > 0) write(uout,*) 'calculating KxcLDA'

 if (iparam%nsppol == 1) then
  call ckxcldar(fftmesh%nr,lda%rho2,Kxcldagmg)
 else  
  if(iverbose > 0) write(uout,*) 'using Perdew-Wang 92 (spin-polarised) functional' 
  call difvxcspin(lda%rho2,fftmesh%nr,iparam%nsppol,nkxc,Kxcldagmg)
 endif

 deallocate(lda%rho2)

 if(iparam%exciton) then

   do isppol = 1, iparam%nsppol
     lda%fxcldar(:,isppol,isppol) = Kxcldagmg(:,2*isppol-1)
   enddo
   if (iparam%nsppol==2) then
     lda%fxcldar(:,1,2) = Kxcldagmg(:,2)
     lda%fxcldar(:,2,1) = Kxcldagmg(:,2)
   endif

 else

! calculate kernel in G-G' form
    if(iverbose > 1) then
       write(uout,*) 'fft Kxclda(r) -> Kxclda(G)'
       write(uout,*) 
    endif

    do ikxc  =1,nkxc
     call c3dfft(Kxcldagmg(:,ikxc),fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),fftmesh%nrr(1),& 
                       fftmesh%nrr(2),fftmesh%nrr(3),-1,fftmesh%menoplan)
     fftfct = 1 / real(fftmesh%nrr(1)*fftmesh%nrr(2)*fftmesh%nrr(3))
     Kxcldagmg(:,ikxc) = Kxcldagmg(:,ikxc)*fftfct
!    instead of
!    call ckxcldag(fftmesh,%nrr(1),fftmesh,%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),nr,lda%rho2,Kxcldagmg)
    enddo

     do ig2 = 1, iparam%npwmat
       do ig3 = 1, iparam%npwmat
         ig4x = modx2(trspace%gvec(1,ig3)-trspace%gvec(1,ig2),fftmesh%nrr(1))
         ig4y = modx2(trspace%gvec(2,ig3)-trspace%gvec(2,ig2),fftmesh%nrr(2))
         ig4z = modx2(trspace%gvec(3,ig3)-trspace%gvec(3,ig2),fftmesh%nrr(3))
         ig4 = 1 + ig4x + ig4y*fftmesh%nrr(1) + ig4z*fftmesh%nrr(1)*fftmesh%nrr(2)
         do isppol = 1, iparam%nsppol
          lda%fxclda(ig3,ig2,isppol,isppol) = Kxcldagmg(ig4,2*isppol-1)
         enddo
         if (iparam%nsppol==2) then
           lda%fxclda(ig3,ig2,1,2) = Kxcldagmg(ig4,2)
           lda%fxclda(ig3,ig2,2,1) = Kxcldagmg(ig4,2)
         endif
       enddo
     enddo

     if(iverbose > 1) then
       write(uout,*) 'fxclda(G,G")'
       do isppol = 1, iparam%nsppol
        do jsppol = 1, iparam%nsppol
          if (iparam%nsppol ==2) then
           if (isppol == 1) then
             s1='UP'
           else
             s1='DOWN'
           endif
           if (jsppol == 1) then
             s2='UP'
           else
             s2='DOWN'
           endif
            write(uout,*) 'spin component: ', s1, s2
            write(uout,*)
          endif      
          call printcm(lda%fxclda(:,:,isppol,jsppol),iparam%npwmat,iparam%npwmat)
        enddo
       enddo 
     endif

   endif !if exciton

  deallocate(Kxcldagmg)

  if(iverbose > 0) then
   call print_footline()
  endif

 
  if(iverbose > 2) call uverbose('e','ckxclda')

#ifdef MPI
endif ! .not. mpilocal .or. rang = 0  
if(iparam%mpilocal) then
  call mpi_barrier(mpi_comm_world,ierr)
  if (iparam%exciton) then
   call mpi_bcast(lda%fxcldar,fftmesh%nr*iparam%nsppol*iparam%nsppol,mpi_complex,0,mpi_comm_world,ierr)
  else  
   call mpi_bcast(lda%fxclda,iparam%npwmat*iparam%npwmat*iparam%nsppol*iparam%nsppol,mpi_complex,0,mpi_comm_world,ierr)
  endif
endif
#endif

  end


!!***

!{\src2tex{textfont=tt}}
!!****f* ckxclda/modx2
!! NAME
!! modx2
!!
!! FUNCTION
!! more elaborate module function
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! ckxclda
!!
!! SOURCE

  integer function modx2(i,j)

  implicit none
  integer i,j
  modx2 = mod( mod(i,j) + ((1-sign(1,i))/2)*j , j)
  end 
!!***


!{\src2tex{textfont=tt}}
!!****p* ckxclda/ckxcldar
!! NAME
!! ckxcldar
!!
!! FUNCTION
!! Calculate the lda fxc (Ceperley-Alder parametrization) in r space
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! ckxclda
!!
!! CHILDREN
!! difvxc
!!
!! SOURCE

 subroutine ckxcldar(nr,rho2,Kxclda)

 implicit none
 integer nr,ir
 real rho2(nr)
 complex Kxclda(nr)
 real, external :: difvxc
 do ir = 1, nr
   Kxclda(ir) = difvxc(rho2(ir))
 enddo
 end
!!*** 

!{\src2tex{textfont=tt}}
!!****f* ckxcldar/difvxc
!! NAME
!! difvxc
!!
!! FUNCTION
!! TEMP Include copy of DIFVXC to print out rho for testing
!! Given the charge density of a homogenous e- gas, this calculates
!! the differential of the Ceperley Alder exchange correlation potential
!! wrt rho (includes relativistic effects see VXCCA).
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! ckxcldar
!!
!! CHILDREN
!! diffvx,diffvc
!!
!! SOURCE

 real function difvxc(rho)

 implicit none
 integer :: iprint=0
 real rho
 real, external :: diffvx, diffvc
 if (rho.lt.1.0e-10) then
   if (iprint.ne.1) then
     write(6,1030) rho
1030 format (//' *** Warning: RHO very small in DIFVXC:',e14.6/ & 
               '  This message will not be printed again')
     iprint = 1
   endif
   difvxc = 0.0
 else
   difvxc=diffvc(rho)+diffvx(rho)
 endif
 return
 end
!!***

!{\src2tex{textfont=tt}}
!!****f* difvxc/diffvc
!! NAME
!! diffvc
!!
!! FUNCTION
!! Given the charge density of a homogenous e- gas, this calculates
!! the differential of the correlation potential wrt rho.
!! It does this by evaluating the differential of the Ceperley-Alder
!! expression given by PERDEW & ZUNGER (PRB,Vol 23,No 10,P.5048,15 May '81)
!! Gamma,Beta1,Beta2,A,B,C,D are parameters for the fit, in atomic
!! units.
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! difvxc
!!
!! SOURCE

 real function diffvc(rho)

 implicit none
 real, parameter :: pi=3.141592654, athird=0.33333333333333
 real, parameter :: a=     0.0311 
 real, parameter :: b=    -0.0480 
 real, parameter :: c=     0.0020 
 real, parameter :: d=    -0.0116 
 real, parameter :: gamma=-0.1423 
 real, parameter :: beta1= 1.0529 
 real, parameter :: beta2= 0.3334 
 real rho,stor1,stor2,stor3,rs

!calculate Rs from the density
 rs=(3.0/(4.0*pi*rho))**athird
!two different calculations depending if Rs <> 1
 if (rs .gt. 1.0) then
  stor1=(1.0+beta1*sqrt(rs)+beta2*rs)**(-3.0)
  stor2=-0.41666667*beta1*(rs**(-0.5))-0.5833333*(beta1**2)-0.66666667*beta2
  stor3=-1.75*beta1*beta2*sqrt(rs)-1.3333333*rs*(beta2**2)
  diffvc=gamma*stor1*(stor2+stor3)
 else
  diffvc=a/rs+0.66666667*(c*log(rs)+d)+0.33333333*c
 endif
 diffvc=diffvc*(-4.0*pi/9.0)*(rs**4)
 end
!!***


!{\src2tex{textfont=tt}}
!!****f* difvxc/diffvx
!! NAME
!! diffvx
!!
!! FUNCTION
!! calculate the differential of the exchange potential wrt rho for
!! a homogenous e- gas of density rho.does this by differenting the
!! relativistic term (see Vxcca).
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! difvxc
!!
!! SOURCE

 real function diffvx(rho)

 implicit none
 real, parameter :: athird=0.3333333333,pi=3.141592654
 real rho,rs,b,bb,bb1
 real rel, vxnr, difrel
 rs=(3.0/(4.0*pi*rho))**athird
 vxnr=-(3*rho/3.141592654)**0.3333333333
 b=0.0140/rs
 rel=-0.5+1.5*log(b+sqrt(1.0+b*b))/(b*sqrt(1.0+b*b))
 bb=b*b
 bb1=1.0+bb
 difrel=(1.5/(b*bb1))-1.5*log(b+sqrt(bb1))*(1.0+2.0*bb)*(bb1**(-1.5))/bb
 difrel=difrel*(-0.0140)/(rs*rs)
 diffvx=(0.610887057/(rs*rs))*rel+ vxnr*difrel
 diffvx=diffvx*(-4.0*pi/9.0)*(rs**4)
 end
!!***


!!==================================================================
! function vxnr(rho)
!!==================================================================
!!calculate the exchange potential without any rel corrections.
!!------------------------------------------------------------------
! real rho
! vxnr=-(3*rho/3.141592654)**0.3333333333
! end
!
!
!!==================================================================
! function rel(rho)
!!==================================================================
!!rel(rho) calculates the relativistic to vx.
!!------------------------------------------------------------------
! parameter (athird=0.3333333333,pi=3.141592654)
! real rho,rs,b
! rs=(3.0/(4.0*pi*rho))**athird
! b=0.0140/rs
! rel=-0.5+1.5*log(b+sqrt(1.0+b*b))/(b*sqrt(1.0+b*b))
! end
!
!
!!==================================================================
! function difrel(rho)
!!==================================================================
!!difrel(rho) calculates drel/drs.
!!------------------------------------------------------------------
! parameter (athird=0.3333333333,pi=3.141592654)
! real rho,rs,b,bb,bb1
! rs=(3.0/(4.0*pi*rho))**athird
! b=0.0140/rs
! bb=b*b
! bb1=1.0+bb
! difrel=(1.5/(b*bb1))-1.5*log(b+sqrt(bb1))*(1.0+2.0*bb)*(bb1**(-1.5))/bb
! difrel=difrel*(-0.0140)/(rs*rs)
! end



!{\src2tex{textfont=tt}}
!!****f* ckxclda/difvxcspin
!! NAME
!! diffvxcspin
!!
!! FUNCTION  
!!  rho_updn(npts,nsppol)=spin-up and spin-down density
!!  => also in spin unpolarized case
!!  dvxci=partial second derivatives of the xc energy
!!  dvxci(npts,1+nsppol)=              (Hartree*bohr^3)
!!   if(nsppol=1): dvxci(:,1)=dvxc/d$\rho$ 
!!   if(nsppol=2): dvxci(:,1)=dvxc($\uparrow$)/d$\rho(\downarrow)$,
!!                 dvxci(:,2)=dvxc($\uparrow$)/d$\rho(\downarrow)$,
!!                 dvxci(:,3)=dvxc($\downarrow$)/d$\rho(\downarrow)$
!! PARENTS
!! ckxclda 
!!  
!! SOURCE
 subroutine difvxcspin(rho_updn,npts,nsppol,ndvxci,dvxci)


 use dpdef 
 implicit none


 integer, intent(in) :: nsppol,npts,ndvxci
 real,intent(in) :: rho_updn(npts,nsppol)
 complex,intent(out) :: dvxci(npts,ndvxci)

!!! parameters
 real,parameter :: m1thrd=-1./3.
 real,parameter :: piinv = 1./pi 
 real,parameter :: threefourth_divpi=(3./4.)*piinv
 real,parameter :: alpha_zeta2=1.0-1.0e-6,alpha_zeta=1.0-1.0e-6
 real,parameter :: b_wc=0.123456790123,beta=0.066725
 real,parameter :: beta_inv=1.0/beta,c_wc=0.00793746933516
 real,parameter :: fsec_inv=1.0/1.709921,kappa_pbe=0.804
 real,parameter :: mu=0.2195149727645171
 real,parameter :: mu_divkappa_pbe=mu/kappa_pbe
 real,parameter :: rsfac=0.6203504908994000 
 
 real           :: twom1_3,sixpi2_1_3,sixpi2m1_3
 real           :: gamma,gamma_inv,beta_gamma
 real           :: factf_zeta,factfp_zeta,factfpp_zeta
 real           :: coeff_tt
 real           :: sq_rsfac,sq_rsfac_inv
    
!Parameters for the Perdew-Wang 92 LSD
!see Table I of Phys.Rev.B 45,13244 (1992)
 real,parameter ::  ec0_aa=0.031091,ec1_aa=0.015545, mac_aa=0.016887
 real,parameter ::  ec0_a1=0.21370, ec1_a1=0.20548, mac_a1=0.11125
 real,parameter ::  ec0_b1=7.5957, ec1_b1=14.1189, mac_b1=10.357
 real,parameter ::  ec0_b2=3.5876, ec1_b2=6.1977, mac_b2=3.6231
 real,parameter ::  ec0_b3=1.6382, ec1_b3=3.3662, mac_b3=0.88026
 real,parameter ::  ec0_b4=0.49294, ec1_b4=0.62517, mac_b4=0.49671

!!! local variables
 integer :: isppol,ipts
 real :: rho,rhomot,rhotmot,rhotot_inv,zeta
 real :: rhotot,rhotmo6,rhoto6
 real :: rs,sqr_rs,rsm1_2

 real,allocatable :: rho_updnm1_3(:,:),rhoarr(:),rhom1_3(:)
 real,allocatable :: zetm(:),zetp(:),zetmm1_3(:),zetpm1_3(:)

 real :: ec0_q0,ec0_q1,ec0_q1p,ec0_den,ec0_log,ecrs0,decrs0_drs,ec0_q1pp,d2ecrs0_drs2
 real :: mac_q0,mac_q1,mac_q1p,mac_den,mac_log,macrs,dmacrs_drs
 real :: ec1_q0,ec1_q1,ec1_q1p,ec1_den,ec1_log,ecrs1,decrs1_drs 
 real :: zetp_1_3,zetm_1_3,f_zeta,fp_zeta,zeta4,gcrs,ecrs
 real :: dgcrs_drs,decrs_drs,dfzeta4_dzeta,decrs_dzeta
 real :: ec1_q1pp,d2ecrs1_drs2,mac_q1pp,d2macrs_drs2,d2gcrs_drs2,fpp_zeta
 real :: d2fzeta4_dzeta2,d2ecrs_drs2,d2ecrs_drsdzeta,d2ecrs_dzeta2
 real :: dvcrs_drs,d2ecrs_drho2,d2ecrs_drup2,d2ecrs_drdndrup,d2ecrs_drdn2

 logical :: onlyx =.false.

!parameters
 twom1_3=2.0**(-1./3.)
 sixpi2_1_3=(6*pi**2)**(1./3.)
 sixpi2m1_3=1./sixpi2_1_3
 gamma=(1.-log(2.))*piinv**2.
 gamma_inv=1./gamma
 beta_gamma=beta*gamma_inv
 factf_zeta= 1. / ( 2.0**(4./3.)-2. )
 factfp_zeta= 4./3. * factf_zeta * alpha_zeta2
 factfpp_zeta= (1./3.) * factfp_zeta * alpha_zeta2
 coeff_tt= 1./ (4.*4.*piinv*(3.*pi**2.)**(1./3.))
 sq_rsfac=sqrt(rsfac)
 sq_rsfac_inv=1./sq_rsfac
!end parameters

 allocate(rhoarr(npts),rhom1_3(npts),rho_updnm1_3(npts,nsppol))
 if (nsppol == 2) then
  allocate(zetm(npts),zetmm1_3(npts),zetp(npts),zetpm1_3(npts))
 endif  
  
 do isppol=1,nsppol
  rho_updnm1_3(:,isppol)=sign( (abs(rho_updn(:,isppol)))**m1thrd,rho_updn(:,isppol))
 end do

 if (nsppol == 2) then   
  rhoarr(:)=rho_updn(:,1)+rho_updn(:,2)
 else
  rhoarr(:)=rho_updn(:,1)
 endif
 rhom1_3(:)=sign( (abs(rhoarr(:)))**m1thrd,rhoarr(:))


 if (nsppol == 2) then
  do ipts=1,npts
   rhotmot=rhom1_3(ipts)
   rhotot_inv=rhotmot*rhotmot*rhotmot
   zeta=(rho_updn(ipts,1)-rho_updn(ipts,2))*rhotot_inv
   zetp(ipts)=1.0+zeta*alpha_zeta
   zetm(ipts)=1.0-zeta*alpha_zeta
  end do
  zetpm1_3(:) =sign( (abs(zetp(:)))**m1thrd,zetp(:))
  zetmm1_3(:) =sign( (abs(zetm(:)))**m1thrd,zetm(:))
 endif  
  
 do ipts=1,npts
  
 rhotot=rhoarr(ipts)
 rhotmot=rhom1_3(ipts)
 rhotot_inv=rhotmot*rhotmot*rhotmot
 rhotmo6=sqrt(rhotmot)
 rhoto6=rhotot*rhotmot*rhotmot*rhotmo6

! Exchange
  do isppol=1,nsppol
   rho   =rho_updn(ipts,isppol)
   rhomot=rho_updnm1_3(ipts,isppol)

   dvxci(ipts,2*isppol-1)=-(4./9.) * threefourth_divpi*sixpi2_1_3*rhomot*rhomot
  enddo

   if (nsppol == 2 ) dvxci(ipts,2)=0.
!  If non-spin-polarized, first component of dvxci is second
!  derivative with respect to TOTAL density.
   if (nsppol == 1 ) dvxci(ipts,1)=dvxci(ipts,1)/2.


! Correlation
 if (.not. onlyx) then     

   rs=rsfac*rhotmot
   sqr_rs=sq_rsfac*rhotmo6
   rsm1_2=sq_rsfac_inv*rhoto6

   if (nsppol == 2 ) then
    zeta=(rho_updn(ipts,1)-rho_updn(ipts,2))*rhotot_inv
   else
    zeta = 0.
   endif 

! Formulas A6-A8 of PW92LSD
   ec0_q0=-2.0*ec0_aa*(1.0+ec0_a1*rs)
   ec0_q1=2.0*ec0_aa*(ec0_b1*sqr_rs+ec0_b2*rs+ec0_b3*rs*sqr_rs+ec0_b4*rs*rs)
   ec0_q1p=ec0_aa*(ec0_b1*rsm1_2+2.*ec0_b2+3.*ec0_b3*sqr_rs+4.*ec0_b4*rs)
   ec0_den=1.0/(ec0_q1*ec0_q1+ec0_q1)
   ec0_log=-log( ec0_q1*ec0_q1*ec0_den )
   ecrs0=ec0_q0*ec0_log
   decrs0_drs= -2.0*ec0_aa*ec0_a1*ec0_log - ec0_q0*ec0_q1p *ec0_den
   ec0_q1pp=0.5*ec0_aa*(-ec0_b1*rsm1_2**3+3.*ec0_b3*rsm1_2+8.*ec0_b4)
   d2ecrs0_drs2= 4.0*ec0_aa*ec0_a1*ec0_q1p*ec0_den            &
&   -ec0_q0*ec0_q1pp*ec0_den                        &
&   +ec0_q0*ec0_q1p**2*ec0_den**2*(2.*ec0_q1+1.0)

   mac_q0=-2.0*mac_aa*(1.0+mac_a1*rs)
   mac_q1=2.0*mac_aa*(mac_b1*sqr_rs+mac_b2*rs+mac_b3*rs*sqr_rs+mac_b4*rs*rs)
   mac_q1p=mac_aa*(mac_b1*rsm1_2+2.*mac_b2+3.*mac_b3*sqr_rs+4.*mac_b4*rs)
   mac_den=1.0/(mac_q1*mac_q1+mac_q1)
   mac_log=-log( mac_q1*mac_q1*mac_den )
   macrs=mac_q0*mac_log
   dmacrs_drs= -2.0*mac_aa*mac_a1*mac_log - mac_q0*mac_q1p*mac_den
   
   if (nsppol == 1) then

    ecrs=ecrs0
    decrs_drs=decrs0_drs
    decrs_dzeta=0.0
    d2ecrs_drs2=d2ecrs0_drs2
    d2ecrs_dzeta2=alpha_zeta**2*(-macrs)
    d2ecrs_drsdzeta=0.0
    zeta=0.0

   else

    ec1_q0=-2.0*ec1_aa*(1.0+ec1_a1*rs)
    ec1_q1=2.0*ec1_aa*(ec1_b1*sqr_rs+ec1_b2*rs+ec1_b3*rs*sqr_rs+ec1_b4*rs*rs)
    ec1_q1p=ec1_aa*(ec1_b1*rsm1_2+2.*ec1_b2+3.*ec1_b3*sqr_rs+4.*ec1_b4*rs)
    ec1_den=1.0/(ec1_q1*ec1_q1+ec1_q1)
    ec1_log=-log( ec1_q1*ec1_q1*ec1_den )
    ecrs1=ec1_q0*ec1_log
    decrs1_drs= -2.0*ec1_aa*ec1_a1*ec1_log - ec1_q0*ec1_q1p *ec1_den
  
!   alpha_zeta is introduced in order to remove singularities for fully
!   polarized systems.
    zetp_1_3=(1.0+zeta*alpha_zeta)*zetpm1_3(ipts)**2
    zetm_1_3=(1.0-zeta*alpha_zeta)*zetmm1_3(ipts)**2
  
    f_zeta=( (1.0+zeta*alpha_zeta2)*zetp_1_3 +                      &
&    (1.0-zeta*alpha_zeta2)*zetm_1_3 - 2.0 ) * factf_zeta
    fp_zeta=( zetp_1_3 - zetm_1_3 ) * factfp_zeta
    zeta4=zeta**4
  
    gcrs=ecrs1-ecrs0+macrs*fsec_inv
    ecrs=ecrs0+f_zeta*(zeta4*gcrs-macrs*fsec_inv)
  
    dgcrs_drs=decrs1_drs-decrs0_drs+dmacrs_drs*fsec_inv
    decrs_drs=decrs0_drs+f_zeta*(zeta4*dgcrs_drs-dmacrs_drs*fsec_inv)
    dfzeta4_dzeta=4.0*zeta**3*f_zeta+fp_zeta*zeta4
    decrs_dzeta=dfzeta4_dzeta*gcrs-fp_zeta*macrs*fsec_inv
  
    ec1_q1pp=0.5*ec1_aa*(-ec1_b1*rsm1_2**3+3.*ec1_b3*rsm1_2+8.*ec1_b4)
    d2ecrs1_drs2= 4.0*ec1_aa*ec1_a1*ec1_q1p*ec1_den            &
&    -ec1_q0*ec1_q1pp*ec1_den                        &
&    +ec1_q0*ec1_q1p**2*ec1_den**2*(2.*ec1_q1+1.0)
  
    mac_q1pp=0.5*mac_aa*(-mac_b1*rsm1_2**3+3.*mac_b3*rsm1_2+8.*mac_b4)
    d2macrs_drs2= 4.0*mac_aa*mac_a1*mac_q1p*mac_den            &
&    -mac_q0*mac_q1pp*mac_den                        &
&    +mac_q0*mac_q1p**2*mac_den**2*(2.*mac_q1+1.0)
  
    d2gcrs_drs2=d2ecrs1_drs2-d2ecrs0_drs2+d2macrs_drs2*fsec_inv
    fpp_zeta=(zetpm1_3(ipts)**2+zetmm1_3(ipts)**2) * factfpp_zeta
  
    d2fzeta4_dzeta2=12.0*zeta**2*f_zeta  &
&    + 8.0*zeta**3*fp_zeta &
&    +       zeta4  *fpp_zeta
  
    d2ecrs_drs2=d2ecrs0_drs2+&
&    f_zeta*(zeta4*d2gcrs_drs2-d2macrs_drs2*fsec_inv)
    d2ecrs_drsdzeta=dfzeta4_dzeta*dgcrs_drs-fp_zeta*dmacrs_drs*fsec_inv
    d2ecrs_dzeta2=d2fzeta4_dzeta2*gcrs-fpp_zeta*macrs*fsec_inv

   endif !nsppol

   dvcrs_drs=(1./3.)*(2.*decrs_drs-rs*d2ecrs_drs2)
   d2ecrs_drho2= -rs**4*(4*pi*(1./3.))*(1./3.)*dvcrs_drs

   if (nsppol == 1) then
! Add correlation to exchange
    dvxci(ipts,1)=dvxci(ipts,1) + d2ecrs_drho2

   else

    d2ecrs_drup2=d2ecrs_drho2+&
&    2.0*(-(1./3.)*rs*d2ecrs_drsdzeta)*(1.-zeta)*rhotot_inv+ &
&    d2ecrs_dzeta2*(1.-zeta)**2*rhotot_inv

    d2ecrs_drdndrup=d2ecrs_drho2+&
&    2.0*(-(1./3.)*rs*d2ecrs_drsdzeta)*(-zeta)*rhotot_inv+ &
&    d2ecrs_dzeta2*(1.-zeta)*(-1.-zeta)*rhotot_inv

    d2ecrs_drdn2=d2ecrs_drho2+&
&    2.0*(-(1./3.)*rs*d2ecrs_drsdzeta)*(-1.-zeta)*rhotot_inv+ &
&    d2ecrs_dzeta2*(-1.-zeta)**2*rhotot_inv
   
! Add correlation to exchange
    dvxci(ipts,1)=dvxci(ipts,1)+d2ecrs_drup2
    dvxci(ipts,2)=dvxci(ipts,2)+d2ecrs_drdndrup
    dvxci(ipts,3)=dvxci(ipts,3)+d2ecrs_drdn2

   endif !nsppol

  endif !onlyx

 enddo 


! if (rho.lt.1.0e-10) then
!   if (iprint.ne.1) then
!     write(uout,1030) rho
!1030 format (//' *** Warning: RHO very small in DIFVXC:',e14.6/ & 
!               '  This message will not be printed again')
!     iprint = 1
!   endif
!   difvxc = 0.0
!  endif

  deallocate(rhoarr,rhom1_3,rho_updnm1_3)
  if (nsppol == 2) then
   deallocate(zetm,zetmm1_3,zetp,zetpm1_3)
  endif  

  end 
!!***
